# -*- Tcl -*-
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.IDENT     $Id: writeControlProc.tcl,v 1.1 2004/09/06 14:47:13 cguirao Exp $
#.LANGUAGE  Tcl/Tk
#.AUTHOR    Fabio Sogni, ESO
#.PURPOSE   Client program executed by asto and used to submit requests
#           to romanic.
#.COMMENTS 
#  Script launched by mediaWrite. It creates requests file readable
#  by romanic.
#
#.VERSION   1.0  15-Jul-00: FS, Creation
#--------------------------------------------------------------------


#Initialize script.
proc writeControlInitX {} {
    global gvar env

    set gvar(HOST) $env(HOST)
    Log "Initialize writeControl" Info
    
    #Define constants and initalize variables.
    Debug "Define constants and initalize variables." 1
    writeControlDefineVars

    #Parse and check arguments.
    Debug "Parse and check arguments." 1
    writeControlParseArgs
    
    #Create request/s
    Debug "Create request(s)" 1
    writeControlCreateRequest
    
    Log "writeControl initialized" Info
    #Loop
    Debug "Loop" 1
    writeControlLoop    
}


#Define some constants.
proc writeControlDefineVars {} {
    global gvar env cv

    set gvar(reqFlag) 0
    set gvar(tmpMessage) ""
    set gvar(counterFlag) 0
    
    if {[file exists $env(ASTO_CONFIG_DIR)/writeControl.config]} {
	ScanConfigFile $env(ASTO_CONFIG_DIR)/writeControl.config gvar
    } else {
	Log "$env(ASTO_CONFIG_DIR)/writeControl.config doesn't exist. Exit" Error
	Info "$env(ASTO_CONFIG_DIR)/writeControl.config doesn't exist. Exit"
	exit 1 
    }
    
    # List with all AstoControl Preferences Parameters.
    #This list at the same time
    # defines the default values for each paramater
    set gvar(prefsList) {\
			     {romanicAction      {writePrint}} \
			     {dummyMode          {0}}}

    set savePrefs 0
    foreach el $gvar(prefsList) {
	set key [lindex $el 0]	
	if {![info exists gvar($key)]} {
	    set value [lindex $el 1]
	    set gvar($key) $value
	    set savePrefs 1	   
	}
    }

    $cv.writeControlOutputWdg config -justify left
}


#Parse the arguments passed by mediaWrite.
proc writeControlParseArgs {} {
    global gvar env
    
    set gvar(volumeName)            [lindex $gvar(args) 0]
    set gvar(cdrecordrc_file_path)  [lindex $gvar(args) 1]
    set gvar(log_file_path)         [lindex $gvar(args) 2]
    set gvar(work_path)             [lindex $gvar(args) 3]
    set device_info                 [lindex $gvar(args) 4]
    set gvar(num_copies)            [lindex $gvar(args) 5]
    set gvar(num_files)             [lindex $gvar(args) 6]; # note: this arg. is not used by this script.
    set gvar(data_path)             [lindex $gvar(args) 7];  # note: this arg. is not used by this script.
    set gvar(volume_file) $gvar(work_path)/$gvar(volumeName)

    if { $gvar(volume_file) == ""}  { 
	Log "writeControl: file name not specified" Error
	puts "File name not specified"
	exit 1
    }

    Debug "volumeName:           $gvar(volumeName)" 1
    Debug "cdrecordrc_file_path: $gvar(cdrecordrc_file_path)" 1
    Debug "log_file_path:        $gvar(log_file_path)" 1
    Debug "work_path:            $gvar(work_path)" 1
    Debug "device_info:          $device_info" 1
    Debug "num_copies:           $gvar(num_copies)" 1
    Debug "num_files:            $gvar(num_files)" 1
    Debug "data_path:            $gvar(data_path)" 1
    Debug "volume_file:          $gvar(volume_file)" 1

    #if {![file exists $gvar(volume_file)] || ![file readable $gvar(volume_file)]} { 
	#puts "File $gvar(volume_file) doesn't exist or is not readable"
	#exit 1 }
    #if {![file exists $gvar(log_file_path)] || ![file writable $gvar(log_file_path)] } { 
	#puts "File $gvar(log_file_path) doesn't exist or is not writable"
	#exit 1 }
    if {![file exists $gvar(work_path)]} {
	Log "writeControl: File $gvar(work_path) doesn't exist" Error
	puts "File $gvar(work_path) doesn't exist"
	exit 1
    }

    if {[catch {cd $gvar(work_path)} msg ]} { 
	Log "writeControl: Cannot change directory to $gvar(work_path)\n($msg)" Error
	puts "Cannot change directory to $gvar(work_path)\n($msg)"
	exit 1
    }
    
    if {![info exists gvar(reqDir)] && ![info exists env(ASTO_REQ_DIR)]} {
	Log "writeControl: Environment variable ASTO_REQ_DIR not defined\nor -reqDir option not specified." Error
	puts "Environment variable ASTO_REQ_DIR not defined"
	puts "or -reqDir option not specified."
	exit 1
    } elseif {[info exists env(ASTO_REQ_DIR)]} {
	set gvar(reqDir) $env(ASTO_REQ_DIR)
    }
    
    set deviceArgs       [split $device_info ","]
    set gvar(device)     [join [lrange $deviceArgs 1 3] ","]
    set gvar(writeSpeed) [lindex $deviceArgs 5]    
    set gvar(medium)     [lindex $deviceArgs 6]
    set gvar(printerAvailable) [string trim [lindex $deviceArgs 7]]
}


#Core procedure. Creates a file containing the request for romanic.
proc writeControlCreateRequest {} {
    global gvar 

    #Create a unique id using the high resolution clock.
    set id "$gvar(volumeName)_[string trimleft [clock clicks] "-"]"
    Debug  "My ID is: $id" 1
    set gvar(id) $id
    set options ""
    if {$gvar(dummyMode)} {
	set options " -dummyMode"
    }
    
    if {$gvar(printerAvailable) == "noPrinter"} {
	if {$gvar(romanicAction) == "writePrint"} {
	    set gvar(romanicAction) "writeOnly"
	} elseif {$gvar(romanicAction) == "printOnly"} {
	    Log "writeControl: this robot doesn't support a printer, please chec the devices table in the database and the writeControl.config file." Error
	    Info "This robot doesn't support a printer, please check the devices table in the database and the writeControl.config file."
	exit 1 
	}
    }
 
    set request "-action $gvar(romanicAction) -volumeFile $gvar(volume_file) -volumeName $gvar(volumeName) -writerAddress $gvar(device) -writerSpeed $gvar(writeSpeed) -medium $gvar(medium) $options "
    
    writeControlLog "Create request $gvar(reqDir)/request-$id.romanic: $request"
    Debug  "Write file $gvar(reqDir)/request-$id.romanic" 1
    Debug  "Request is: $request" 1
    
    if {[catch {write_file $gvar(reqDir)/request-$id.romanic $request}]} {
	Log "writeControl: Cannot write file $gvar(reqDir)/request-$id.romanic" Error
	puts "Cannot write file $gvar(reqDir)/request-$id.romanic"
	exit 1 
    }
    
    Log "Submitted request:\n$id" Info
    set gvar(message) "Submitted request:\n$id"
    Debug "$gvar(message)" 1
    after 1000   
}

#Loop: read temp files, counter and error messages.
proc writeControlLoop {} {
    global gvar cv

    while {[file exists $gvar(reqDir)/request-$gvar(id).romanic]} {
	#The request was submitted.

	update; update idletasks
	set gvar(reqFlag) 1
	
	if {[glob -nocomplain $gvar(reqDir)/inProcess-$gvar(id)-*.romanic] != ""} {
	    #Romanic queued my request.
	    updateMessage
	    set firstWord [lindex $gvar(message) 0]
	    if {$firstWord != "Submitted:" && $firstWord != "Processing:"} {
		#Romanic is serving my request.
		$cv.cancelReqButWdg disable
	    }
	}	

	if {[file exists $gvar(reqDir)/error-$gvar(id).romanic]} {
	    #An error occourred. Ask the user to fix it.
	    manageError
	}
	
	if {[file exists $gvar(reqDir)/fatalError-$gvar(id).romanic]} {
	    #A fatal error occourred. Exit.
	    manageFatalError
	}
	
	if {[file exists $gvar(reqDir)/count-$gvar(id).romanic]} {
	    #cdrecord is cutting a CD. Update the progress bar.
	    updateCounter
	}
	
	after 500
    }

    #If everything went fine write a good report.
    if {$gvar(reqFlag)} {
	if {[file exists $gvar(reqDir)/fatalError-$gvar(id).romanic]} {
	    manageFatalError
	} else {
	    writeControlWriteReport good
	    writeControlQuit 0}
    }
}

#The request was cancelled by the user.
proc cancelRequest {} {
    global gvar 

    file delete $gvar(reqDir)/request-$gvar(id).romanic
    file delete [glob -nocomplain $gvar(reqDir)/*-$gvar(id).lock]
    writeControlWriteReport bad
    writeControlQuit 1
}

#Quit the GUI.
proc writeControlQuit {returnCode} {

    #panClosePanel . no
    exit $returnCode

}

#Write a log.
proc writeControlLog {txt} {
    global gvar 

    if {[catch {set file [open $gvar(log_file_path) a+]} msg ]} {
	Log "Cannot write to the log file: $gvar(log_file_path)\n($msg)" Error
 	puts "Cannot write to the log file: $gvar(log_file_path)\n($msg)"
	exit 1
    }

    Log "writeControl: $txt" Info
    puts $file "$txt"
    close $file
}

#Write a message on the GUI.
proc updateMessage {} {
    global gvar 

    set msgFile [glob -nocomplain $gvar(reqDir)/inProcess-$gvar(id)-*.romanic]
    if {[catch {set txt [read_file $msgFile]} msg ]} {
	#popUp "Cannot read the file $msgFile\nPlease fix it."
	#updateMessage
	return 
    }
    
    set gvar(message) $txt
    if {$txt != $gvar(tmpMessage)} {
	writeControlLog $txt
	set gvar(tmpMessage) $txt
    } 
    
}

#Update the progress bar.
proc updateCounter {} {
    global gvar

    if {[catch {set line [read_file $gvar(reqDir)/count-$gvar(id).romanic]} msg]} {
	if {$gvar(counterFlag) < 10} {
	    incr gvar(counterFlag) +1
	    return
	} else {
	    popUp "Cannot read $gvar(reqDir)/count-$gvar(id).romanic\n($msg)\nPlease fix it."
	    set gvar(counterFlag) 0
	    updateCounter
	}
    } else {
	set gvar(counterFlag) 0
    } 

    set num  [lindex $line 0]
    set tot  [lindex $line 2]

    if {$num == "Total"} {
	set gvar(perc) 100
	return
    }
    
    if {![ctype digit $num] || ![ctype digit $tot]}    { return }
    if {[catch {set gvar(perc) [expr $num*100/$tot]}]} { return }
}


#Manage a non fatal error.
proc manageError {} {
    global gvar 
    
    if {[catch {set errorMsg [read_file $gvar(reqDir)/error-$gvar(id).romanic]} msg]} {
	popUp "Cannot read the file: $gvar(reqDir)/error-$gvar(id).romanic\n($msg)\nPlease fix it"
	manageError
    }
    set text "id:$gvar(id)\n$errorMsg"
    Log "$text" Error
    popUp $text
    file delete $gvar(reqDir)/error-$gvar(id).romanic
    if {[catch {write_file $gvar(reqDir)/fixed-$gvar(id).romanic ""} msg]} {
	popUp "Cannot write the file: $gvar(reqDir)/fixed-$gvar(id).romanic\n($msg)\nPlease fix it"
	manageError
    }
}

#Manage a fatal error.
proc manageFatalError {} {
    global gvar 
   
    
    if {[catch {set text [read_file $gvar(reqDir)/fatalError-$gvar(id).romanic]} msg]} {
	popUp "Cannot read the file: $gvar(reqDir)/fatalError-$gvar(id).romanic\n($msg)\nPlease fix it"
	manageFatalError
    }
    
    Log "$text" Error
    writeControlLog $text
    file delete $gvar(reqDir)/fatalError-$gvar(id).romanic
    writeControlWriteReport bad
    writeControlQuit 1
}


#Pop up a message window.
proc popUp {text} {
    
    set choice [popUpText $text]
    if {$choice == "cancel"} { cancelRequest }
}


proc popUpText {text} {

    #Pop up here a window with the choice cancel "request".
    set choice [tk_messageBox -icon warning -type retrycancel \
		    -title  "Write Control Dialog" -message $text \
		    -default retry]
    return $choice
}


#Write a report for mediaWrite.
proc writeControlWriteReport {result} {
    global gvar env


    if {$result == "good"} {
	set numErrors 0
	set numCopies 1
	set numVerified 1
	set header "writeControl Success: Copy 1 completed.\n"
    } else {
	set numErrors 1
	set numCopies 0
	set numVerified 0
	set header "writeControl Error: Copy one failed.\n"
    }


    set output "$header
writeControl - Number of copies:           $numCopies
writeControl - Number verified:            $numVerified
writeControl - Number of errors:           $numErrors
writeControl - Number of verify errors:    0"

    #Write the output in a file recognized by mediaWrite
    writeControlLog $output
    if {[catch {write_file $env(DHS_LOG)/pssst $output} msg]} {
	set choice [popUpText "Error writing file $env(DHS_LOG)/pssst \n($msg)"]
	if {$choice == "cancel"} {
	    writeControlQuit 1
	} else {
	    writeControlWriteReport $result
	}
    }
}

#Print debug messages.
proc Debug {text level} {
    global gvar

    #if {![info exists gvar(HtVerbose)] || $gvar(HtVerbose) == "0" } { return }
    if {![info exists gvar(debugMode)]} { return }

    if { $gvar(debugMode) <  $level } { return }
    set date [clock format [clock seconds] -format "%Y-%m-%d %T"]
    if {[info exists gvar(logFile)]} {
	puts $gvar(logFile) "Debug>\[$date\] $text"
    } else { 
	puts "Debug>\[$date\] $text" 
    }
    
    Log $text Debug    
}


proc Log {text priority} {
    global gvar
    
    if {$text == ""} { set text "-" }
    if {[catch { eval exec "dfsmsg -host $gvar(HOST) -component astoControl -priority $priority -msg \{$text\}"} msg]} {
	puts "Text: $text"
	puts "Priority: $priority"
	Info "Error logging message with dfsLog: $msg"
    }    
}