#Procedure file containing functions to pilot the LSK robot.


#Define the robot commands for the LSK robot.
proc romanicDefineCommandsLsk {} {
    global gvar 

    set gvar(enquiryString)      "V"
    set gvar(requestSenseString) "S"
    set gvar(moveArmString)      ""

    set gvar(escape)            ""
    set gvar(header1)           ""
    set gvar(trail)             ""
    set gvar(emptyMagazines)    0

    #Addresses
    set gvar(blankAddress)      "0101"; #blank tray address
    set gvar(writtenAddress)    "0102"; #written tray address
    set gvar(badAddress)        "0104"; #bad tray address
    set gvar(cdAddress)         "0201"; #cd address
    set gvar(printerAddress)    "0401"; #printerAddress

    #Actions
    set gvar(takeMedia)            "A"; #take media from selected tray
    set gvar(putMedia)             "B"; #put media from selected tray
    set gvar(noAction)          "GTB"; #move arm on top
    set gvar(openPrinter)       "PO$gvar(printerAddress)z"; #open printer
    set gvar(closePrinter)      "PC$gvar(printerAddress)z"; #close printer
    set gvar(fanOn)             ""; #Cooling on
    set gvar(fanOff)            ""; #Cooling off
    set gvar(lastCommandLsk)    ""; #Last command performed.
}


#Low level command to open the printer tray
proc openPrinterLsk {} {
    global gvar 
    
    Log "Open Printer Lsk"
    sendMessage $gvar(openPrinter)
    set result [readPort 3]
    return $result
}

#Low level command to close the printer tray
proc closePrinterLsk {} {
    global gvar 
    
    Log "Close Printer Lsk"
    sendMessage $gvar(closePrinter)
    set result [readPort 3]
    return $result
}


#There is no fan in the LSK, just create a file.
proc fanOnLsk {} {
    global gvar

    write_file $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic ""
    Log "Switch on fan"
}


#There is no fan in the LSK.
proc fanOffLsk {} {
    global gvar

    Log "Switch off fan"
}

#Retrieve the enquiry string. (not used)
proc romanicPilotEnquiryLsk {} {
    global gvar

    Debug "Enquiry" 3
    sendMessage $gvar(enquiryString)
    set result [readPort 20]
    return $result
}


#Return the number of CDs in a particular magazine
#of a particular robot (LSK)
proc checkMagazineLsk {tty magazine} {
    
    set value [countMediaLsk $magazine]
    if {$value == "E"} { return "0" }
    set numCD [lrange $value 0 1]
    return $numCD
}

#Move the arm and count the CD in a magazine.
proc countMediaLsk  {magazine} {
    global gvar 

    Log "Count media in $magazine"
    switch $magazine {
	"blank"    {set returnValue [checkPositionLsk "$gvar(blankAddress)"]}
	"written"  {set returnValue [checkPositionLsk "$gvar(writtenAddress)"]}
	"bad"      {set returnValue [checkPositionLsk "$gvar(badAddress)"]}
    }
    return $returnValue
}

#Enquiry the robot if the previous command gave an error.
#It gives more information about an error. LSK
proc romanicRequestSenseLsk {} {
    global gvar 

    cleanBuffer
    sendMessage $gvar(requestSenseString)
    set sense [readPort 3]
    Debug "sense: $sense" 3
    
    switch  $sense {
	"X??" {set sense1 "Ok"}
	"0??" {set sense1 "Ok"}
	"1??" {set sense1 "Wrong parameter."}
	"2??" {set sense1 "No CD at the position."}
	"3??" {set sense1 "Can not identify the position."}
	"4??" {set sense1 "Autoloader reached the right margin."}
	"5??" {set sense1 "Autoloader reached the left margin."}
	"6??" {set sense1 "Autoloader failed to move the carrier up."}
	"7??" {set sense1 "Autoloader failed to move the carrier down."}
	"8??" {set sense1 "Autoload failed to calibrate the carrier."}
	"9??" {set sense1 "Autoloader could not pick the CD."}
	"A??" {set sense1 "Autoloader failed to move."}
	"B??" {set sense1 "Autoloader reports time-out conditions."}
	"C??" {set sense1 "Autoloader unloaded bad CD to good bin."}
	"D??" {set sense1 "Autoloader failed to find index marks."}
	"E??" {set sense1 "Autoloader reports error during power-on initialisation."}
	"F??" {set sense1 "Autoloader lost CD at unknown position."}
	"G??" {set sense1 "Autoloader failed to find position."}
	"H??" {set sense1 "Autoloader failed to rotate disc."}
	"I??" {set sense1 "Reject bin is full. Please remove bad discs."}
	"J??" {set sense1 "Autoloader reports failure of drive."}
	"K??" {set sense1 "Autoloader reports failure of printer."}
	"L??" {set sense1 "-"}
	"M??" {set sense1 "Autoloader could not put the disc because it got stuck at the gripper. Remove disc manually."}
	"N??" {set sense1 "Door is open - most commands disabled !"}
	default { set sense1 "Wrong Sense." }
    }

    Debug  "Codes: $sense1 --- Ok" 3
    calibrateLsk
    return "$sense1 --- Ok"
}

#Get a disc from magazine A
proc getBlankLsk {} {
    global gvar

    Log "getBlank from LSK"
    set result [loopCommand "getDiskFromMagazine $gvar(blankAddress)" 10]
    if {$result != "ok"} {
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Blank magazine of robot $gvar(ttyTail) is empty"
	    }
	    "Autoloader could not pick the CD.*" {
		waitForFixError "Robot under $gvar(ttyTail) could not pick the blank CD"
	    }
	    default {
		waitForFixError "Error while taking blank CD for robot $gvar(ttyTail)"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	getBlankLsk
    }
    set gvar(lastCommandLsk) "getBlank"
}

#Get a disc from magazine B
proc getWrittenLsk {} {
    global gvar

    Log "getWritten from LSK"
    set result [loopCommand "getDiskFromMagazine $gvar(writtenAddress)" 10]
    if {$result != "ok"} {
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Written magazine of robot $gvar(ttyTail) is empty"
	    }
	    "Autoloader could not pick the CD.*" {
		waitForFixError "Robot under $gvar(ttyTail) could not pick the written  CD"
	    }
	    default {
		waitForFixError "Error while taking written CD for robot $gvar(ttyTail)"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	getWritten
    }
    set gvar(lastCommandLsk) "getWritten"
}

#Get a disc from the bad magazine.
proc getBadLsk {} {
    global gvar

    Log "getBad from LSK"
    set result [loopCommand "getDiskFromMagazine $gvar(badAddress)" 10]
    if {$result != "ok"} {
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Bad magazine of robot $gvar(ttyTail) is empty"
	    }
	    "Autoloader could not pick the CD.*" {
		waitForFixError "Robot under $gvar(ttyTail) could not pick the bad CD"
	    }
	    default {
		waitForFixError "Error while taking bad CD for robot $gvar(ttyTail)"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	getBad
    }
    set gvar(lastCommandLsk) "getBad"
}


#Get a disk from the specified magazine.
proc getDiskFromMagazine {magazine} {
    
    sendMessage "A$magazine"
    set result [ readPort 3]
    return $result
}

#Get a disc from the printer.
proc getDiskFromPrinterLsk {} {
    global gvar 

    Log "getPrinter LSK"
	set result [grabPrinterLsk]
    if {$result != "X??"} { 
	set sense [romanicRequestSense]
	set attempts 1

	while {$attempts < 3 && $sense != "Autoloader reports failure of printer." && $result != "X??"} {
	    set result [grabPrinterLsk]
	    set sense  [romanicRequestSense]
	}

	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Error for robot $gvar(ttyTail) while taking medium from printer."
	    }
	    "Autoloader could not pick the CD.*" {
		waitForFixError "Robot under $gvar(ttyTail) could not pick the medium from the printer."
	    }		
	    "Autoloader lost CD at unknown position.*" {
		waitForFixError "Robot under $gvar(ttyTail) lost the medium \
while taking it from the printer."
	    }		
	    "Autoloader reports failure of printer.*" {
	Debug "Error grabbing CD from printer. Skip error" 1
	}

	    default {
		waitForFixError "Error for robot $gvar(ttyTail) while taking medium from printer: $sense"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	getDiskFromPrinter
    }
    closePrinter
    set gvar(lastCommandLsk) "getDiskFromPrinter"
}

#Low level command to get a disc from the printer.
proc grabPrinterLsk {} {
    global gvar 
    
    sendMessage "A$gvar(printerAddress)"
    set result [readPort 3]
    return $result
}


#Put a disc in the printer.
proc putDiskInPrinterLsk {} {
    global gvar 
    
    Log "putPrinter LSK"
    set result [loopCommand putPrinterLsk 3]
    
    if {$result != "ok"} { 
	if {![info exists gvar(id)]} { 
	    sendMessage "$gvar(requestSenseString)"
	    set result [readPort 3]
	    return 1 
	}
	
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Error for robot $gvar(ttyTail) while putting medium to the printer."
	    }
	    "Autoloader lost CD at unknown position.*" {
		waitForFixError "Robot under $gvar(ttyTail) lost medium."
	    }		
	    default {
		waitForFixError "Error for robot $gvar(ttyTail) while putting medium to printer: $sense"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	eval $gvar(lastCommandLsk)
	putDiskInPrinter
	
    }
}

#Low level command to put a disc in the printer.
proc putPrinterLsk {} {
    global gvar 
    
    sendMessage "B$gvar(printerAddress)"
    set result [readPort 3]
    return $result
}


#Get a disc from the writer.
proc getDiskFromWriterLsk {writer} {
    global gvar

    Log  "Get disk in writer $writer"
    set writerAddress [getWriterAddress $writer]
    set result [loopCommand "getDiskLsk 0$writerAddress" 10]
    
    if {$result != "ok"} { 
	if {![info exists gvar(id)]} { 
	    sendMessage "$gvar(requestSenseString)"
	    set result [readPort 3]
	    return 1 
	}
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
 		waitForFixError "Error for  robot $gvar(ttyTail) while taking a medium from the writer."
	    }
	    "Autoloader could not pick the CD.*" {
		waitForFixError "Robot under $gvar(ttyTail) could not pick medium from writer."
	    }		
	    default {
		waitForFixError "Error for robot $gvar(ttyTail) while taking medium from writer: $sense"
	    }
	} 
	calibrateArmLsk $gvar(tty) 
	getDiskFromWriterLsk $writer
    }
    set gvar(lastCommandLsk) "getDiskFromWriterLsk $writer"
}

#Low level command to ger a disk from the writer.
proc getDiskLsk {writer} {
    global gvar 

    sendMessage "A02$writer"
    set result [readPort 3]
    return $result
}

#Send a file to the parallel port.
proc printLabelLsk {file} {
    global gvar 
    
    catch {exec cat $file > $gvar(parallelList)}
}


#Put the arm on top.
proc armOnTopLsk {} {
    global gvar 

    Log "Arm on top LSK"
    set result [calibrateLsk]
    
    if {$result != "X??"} { 
	Log "Error putting arm on top of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error putting arm on top of robot $gvar(tty)"
	armOnTop
    }
}

#Check if there are discs in the writers.
proc checkWritersLsk {tty} {
    global gvar

    sendMessage "A0201"
    set result [readPort 3]
    calibrateLsk
    sendMessage "A0202"
    set result [readPort 3]
    calibrateLsk
}


#Check if there is a CD in the arm.
proc checkArmLsk {tty} {

    calibrateLsk
    return "0"    
}

#Take a disc from the printer in case there is one.
proc checkPrinterLsk {tty} {
    global gvar

    #Pick a disc from the printer, doesn't matter if there is
    #one or not, execute the calibration
    grabPrinterLsk
    calibrateLsk
}

#Send a calibration command to LSK, it's a kind of reset.
proc calibrateLsk {} {

    sendMessage "C"
    set result [readPort 3]
    return $result
}


#Retrieve the position of the arm.
proc checkPositionLsk {magazine} {

    #puts "magazine: >$magazine<"
    set address [string range $magazine 2 end]
    #puts "Querying address $address"
    sendMessage "GTHC$address"
    set result [readPort 4]
    set numHex [string range $result 0 1]
    scan $numHex {%x} numDec
    if {![info exists numDec]} {
	calibrateLsk
	checkPositionLsk $magazine
    }
    #puts "Result from address $address: $result"
    #puts "Result: $result -- hex: $numHex -- Dec: $numDec"
    return $numDec
}

#Put a disc in the magazine A
proc putBlankLsk {} {
    global gvar

    Log "putBlank (LSK)"
    set result [loopCommand "putDiskInMagazine $gvar(blankAddress)" 3]
    if {$result != "ok"} { 
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Robot $gvar(ttyTail) doens't have a medium in the picker."
	    }
	    "Autoloader lost CD at unknown position.*" {
		waitForFixError "Robot under $gvar(ttyTail) lost medium while putting to Blank magazine."
	    }		
	    default {
		waitForFixError "Error while putting medium to Blank magazine for robot $gvar(ttyTail): $sense"
		
	    }
	}
	calibrateArmLsg $gvar(tty)
	eval $gvar(lastCommandLsk)
	putBlank
    }
}

#Put a disc in magazine B
proc putWrittenLsk {} {
    global gvar

    Log "putWritten (LSK)"
    set result [loopCommand "putDiskInMagazine $gvar(writtenAddress)" 3]
    
    if {$result != "ok"} { 
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Robot $gvar(ttyTail) doens't have a medium in the picker."
	    }
	    "Autoloader lost CD at unknown position.*" {
		waitForFixError "Robot under $gvar(ttyTail) lost medium while putting to Written magazine."
	    }		
	    default {
		waitForFixError "Error while putting medium to Written magazine for robot $gvar(ttyTail): $sense"
	    }
	} 
	calibrateArmLsk $gvar(tty)
	eval $gvar(lastCommandLsk)
	putWritten
    }
}

#Put a disc in the red magazine.
proc putBadLsk {} {
    global gvar

    Log "putBad from LSK"
    set result [loopCommand "putDiskInMagazine $gvar(badAddress)" 3]
    if {$result != "ok"} { 
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		waitForFixError "Robot $gvar(ttyTail) doens't have a medium in the picker."
	    }
	    "Autoloader lost CD at unknown position.*" {
		waitForFixError "Robot under $gvar(ttyTail) lost medium while putting to Bad magazine."
	    }		
	    default {
		waitForFixError "Error while putting medium to Bad magazine for robot $gvar(ttyTail):$sense"
	    }
	} 
	calibrateArmLsg $gvar(tty)
	eval $gvar(lastCommandLsk)
	putBad
    }
}

#Low level command to put a disc in a specified magazine.
proc putDiskInMagazine {magazine} {
    
    sendMessage "B$magazine"
    set result [ readPort 3]
    return $result
}

#Get a disc from the input magazine.
#The variable gvar(inputMagazine) defines which one is considered input.
proc getInputCdLsk {} {
    global gvar
    
    set gvar(inputMagazine) [read_file -nonewline $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic]

    switch $gvar(inputMagazine) {
	"1" {set result [loopCommand "getDiskFromMagazine $gvar(blankAddress)" 10]
	    set gvar(outputMagazine) 3
	}
	"2" {set result [loopCommand "getDiskFromMagazine $gvar(writtenAddress)" 10]
	    set gvar(outputMagazine) 1
	}
    }
    
    if {$result != "ok"} {
	set sense [romanicRequestSense]
	switch -glob $sense {
	    "No CD at the position.*"    {
		if {$gvar(inputMagazine) == 1} {
		    Log "Switch to second magazine"
		    set gvar(inputMagazine)  2
		    set gvar(outputMagazine) 1
		    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
		    getInputCd 
		} else {
		    waitForFixError "Magazine 1 and 2\nOf robot $gvar(ttyTail) is empty.\nInsert new CDs in the magazines."
		    set gvar(inputMagazine)  1
		    set gvar(outputMagazine) 3
		    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
		    getInputCd
		}
	    }
	}
    }
    set gvar(lastCommandLsk) "getInputCd"
}

#Put a disc to the input magazine.
#The variable gvar(outputMagazine) defines which one is considered output.
proc putOutputCdLsk {} {
    global gvar

    Log "Put CD to magazine: $gvar(outputMagazine)"
    
    switch $gvar(outputMagazine) {
	"3" {set result [putDiskInMagazine $gvar(badAddress)]}
	"1" {set result [putDiskInMagazine $gvar(blankAddress)]}
    }
    
    if {$result != "X??"} {
	Log "Error putting CD to magazine $gvar(outputMagazine)"
	Log "Reason: $result"
    	if {[waitForOk]} {
	    waitForFixError "Error putting CD to magazine $gvar(outputMagazine)\nof robot $gvar(ttyTail)"
	}
	eval $gvar(lastCommandLsk)
	putOutputCd
    }
}


proc putDiskInWriterLsk {writer} {
    global gvar

    Log  "Put disk in writer $writer"
    set writerAddress [getWriterAddress $writer]
    set result [putDiskLsk "0$writerAddress"]
        
    if {$result != "X??"} { 
	if {![info exists gvar(id)]} { 
	    sendMessage "$gvar(requestSenseString)"
	    set result [readPort 3]
	    return 1 
	}
	
	set inc 0
	while {$inc < 3 && $result != "X??" } {
	    incr inc
	    calibrateArmLsk $gvar(tty) 
	    set result [putDiskLsk "0$writerAddress"]
	}
        
	if {$inc == 3} {
	    Log "Error putting medium in writer $writer of $gvar(tty) robot."
	    Log "Reason: $result"
	    waitForFixError "Error putting medium in writer $writer\nof robot $gvar(tty)."
	    eval $gvar(lastCommandLsk)
	    calibrateArmLsk $gvar(tty) 
	    putDiskInWriterLsk $writer
	}
    }
}

proc putDiskLsk {writer} {
    global gvar 

    sendMessage "B02$writer"
    set result [readPort 3]
    return $result
}

#Obsolete.
proc putMediaLsk {} {
    global gvar 
    

    set result [ positionMedium $gvar(cdAddress) $gvar(putMedia) ]
	
    if {$result != ""} {
		set inc 0
        while {$inc < 3 && $result != "" } {
			incr inc
            set result [ positionMedium $gvar(cdAddress) $gvar(putMedia)]
        }
		
        if {$inc == 3} {
            Log "Error putting CD to writer $gvar(cdAddress) of $gvar(tty) robot."
            Log "Reason: $result"
            waitForFixError "Error putting CD to writer\nof robot $gvar(ttyTail)."
            putMedia

        }
    }
}

proc calibrateArmLsk {tty} {
    
    calibrateLsk
    
    #sendMessage "C"
    #readPort 3
}

#Executes a command for $attempts times and in case it
#failed all the times, gives an error.
proc loopCommandLsk {cmd attempts} {
    global gvar 
    
    Debug "Executing command: $cmd with $attempts attempts" 2
    set result [$cmd]
    
    if {$result != "X??" && $result != "K??"} {
	set inc 1
        while {$inc < $attempts && $result != "" } {
			incr inc
            set result [ $cmd ]
        }
		
        if {$inc == $attempts} {
	    Debug "Failed executing $cmd after $attempts attempts" 1
	    return $result
        } else {
	    return "ok"
	}
    } else {
	return "ok"
    }
}



##########################################
