package org.eso.dfs.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.Reader;
import java.nio.BufferUnderflowException;
import java.nio.CharBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

/**
 * @author nkornwei
 *
 */
public class MemMappedFileReader extends Reader {

    private MappedByteBuffer buffer;
    private FileChannel fc;

    public MemMappedFileReader(File f) throws IOException {
        // Open the file and then get a channel from the stream
        FileInputStream fis = new FileInputStream(f);
        fc = fis.getChannel();

        // Get the file's size and then map it into memory
        int sz = (int) fc.size();
        MappedByteBuffer b = fc.map(FileChannel.MapMode.READ_ONLY, 0, sz);
        buffer = fc.map(FileChannel.MapMode.READ_ONLY, 0, sz);
    }

    /**
     * @see java.io.Reader#read(char, int, int)
     */
    public int read(char[] arg0, int arg1, int arg2) throws IOException {
        int i;
        if (buffer.hasRemaining()) {
            i = 0;
            do {
                if (i == arg2) {
                    break;
                }
                arg0[i + arg1] = (char)buffer.get();
                ++i;
            } while (buffer.hasRemaining());
        } else {
            i = -1;
        }
        return i;
    }

    /**
     * @see java.io.Reader#close()
     */
    public void close() throws IOException {
        fc.close();
    }

    /**
     * @see java.io.Reader#mark(int)
     */
    public void mark(int arg0) throws IOException {
        throw new IOException("MemMappedFileRead does not support marking.");
    }

    /**
     * @see java.io.Reader#markSupported()
     */
    public boolean markSupported() {
        return false;
    }

    /**
     * @see java.io.Reader#read()
     */
    public int read() throws IOException {
        if (buffer.hasRemaining()) {
            return buffer.get();
        } else {
            return -1;
        }    
    }

    /**
     * @see java.io.Reader#read(char)
     */
    public int read(char[] arg0) throws IOException {
        return read(arg0, 0, arg0.length);
    }

    /**
     * @see java.io.Reader#ready()
     */
    public boolean ready() throws IOException {
        return true;
    }

    /**
     * @see java.io.Reader#reset()
     */
    public void reset() throws IOException {
        buffer.reset();
    }

    /**
     * @see java.io.Reader#skip(long)
     */
    public long skip(long arg0) throws IOException {
        buffer.position(buffer.position() + (int) arg0);
        return buffer.position();
    }

    public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("Please list files on command line.");
        } else {
            for (int i = 0; i < args.length; i++) {
                File f = new File(args[i]);
                try {
                    Reader r = new MemMappedFileReader(f);
                    
                    while (true) {
                        int c = r.read();
                        if (c == -1) {
                            break;
                        }
                        System.out.print((char) c);
                    }
                } catch (IOException e) {
                    System.out.println("Error reading file: " + e.toString());
                }
            }
        }
    }
}
