package org.eso.gasgano.viewers;

/**
 * @author nkornwei
 *
 * Implementation of ExternalViewer which simply runs
 * the viewer as an external command.
 */
import java.io.IOException;
import java.util.Vector;

public class ShellCommandExternalViewer implements ExternalViewer {

    private String shCommand;
    
    /**
     * @see org.eso.gasgano.ExternalViewer#setCommand(String)
     */
    public void setCommand(String command) {
        shCommand = command;
    }

    /**
     * @see org.eso.gasgano.ExternalViewer#displayFile(String)
     */
    public void displayFiles(Vector filePaths)
        throws ExternalViewerException {
        if (filePaths == null || filePaths.size() < 1) {
            throw new ExternalViewerException("Invalid or empty file paths argument.");
        }
        StringBuffer files = new StringBuffer();
        for (int i = 0; i < filePaths.size(); i++) {
            files.append(" "+filePaths.elementAt(i).toString());
        }
        runCommand(shCommand+files.toString());
            
    }
        
    /**
     * @param command The command and arguments to be executed.
     * @return The return code of the completed execution.
     */
    private int runCommand(String command) throws ExternalViewerException {
        int rc = -1;
        try {
            Runtime r = Runtime.getRuntime();
            Process p = r.exec(command);
            try {
                p.waitFor();
                rc = p.exitValue();
            } catch (InterruptedException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
                throw new ExternalViewerException(ie.getMessage());
            } catch (IllegalThreadStateException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
                throw new ExternalViewerException(ie.getMessage());
            }
        } catch (IOException e) {
            System.out.println("Error executing command: " + e.toString());
            throw new ExternalViewerException(e.getMessage());
        }
        return rc;
    }
}
