package org.eso.ohs.core.dbb.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlChunkGroupMember;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.dbb.xml.DbbXMLException;
import org.eso.ohs.core.utilities.MsgManager;
import org.jdom.Element;



public abstract class Dbb {
	
	/** The fabricated DbbView */
	protected DbbView dbbView = null;
    
	/** The list of dafault column chunks. */
	protected List defaultCols = null;
    
	/** A registry of objects we create from the XML file. */
	protected ElementRegistry elementRegistry = null;
	
	/** A registry of DBB widgets we create from the XML file.*/
	protected WidgetRegistry widgetRegistry = null;
	
	/** A registry of DbbSqlChunkGroupMember we create from the XML file. */
	protected ChunkGroupRegistry chunkGroupRegistry = null;

	/** To keep track of where we are in the file */
	/*protected Locator locator = null;*/

	/** The fabricated DbbSession */
	protected DbbSession session = null;
    
	/** 
	 *  SqlEngine. It make sense having this object as global. We need it in the recursion of the xml
	 *  Many makeDbbSqlWhatever methods need to add their processing result to the engine (in example, default joinCondintion)
	 */
    protected DbbSqlEngineImpl dbbEngine = null;
    
	/** Private constructor (this is NOT a Singleton) */
	protected Dbb() {
		elementRegistry = new ElementRegistry();
		widgetRegistry = new WidgetRegistry();
		chunkGroupRegistry = new ChunkGroupRegistry(); 
		defaultCols         = new LinkedList();
	}
	/**
	 * 
	 * @return the fabricated DbbEngine
	 */
	public DbbSqlEngineImpl getDbbEngine() {
		return dbbEngine;
	}

	/** @return The fabricated DbbSession */
	public DbbSession getDbbSession() {
		return session;
	}
    
	/** @return The fabricated DbbView */
	public DbbView getDbbView() {
		return dbbView;
	}


	/**
	 * 
	 * @return the fabricated columns
	 */
	public List getDefaultCols() {
		return defaultCols;
	}
	
	/**
	 * 
	 * @return the fabricated WidgetRegistry
	 */
	public WidgetRegistry getWidgetRegistry() {
		return widgetRegistry;
	}
	
	/**
	 * 
	 * @return the fabricated ElementRegistry
	 */
	public ElementRegistry getElementRegistry() {
		return elementRegistry;
	}
	
	/*
	 * 	[Design by contract preconditions]
	 *	
	 *	As this method is private/protected, we use assertion as precondition.
	 *	If you want to make this method public, please don't use assertions 
	 */
	protected ArrayList getConditionChunks(String aConditionChunk) {
		//DbC//
		assert (aConditionChunk != null): new IllegalArgumentException(MsgManager.errContract("Condition chunk is null"));
		assert (aConditionChunk.length() != 0): new IllegalArgumentException(MsgManager.errContract("condition chunk is empty"));
		//DbC//
		
		ArrayList result = new ArrayList();
		StringTokenizer st = new StringTokenizer(aConditionChunk, ","); 
		String chunkName = "";
	     while (st.hasMoreTokens()) {
	    	 chunkName = st.nextToken();
	    	 Object next = elementRegistry.getElement(chunkName);
	    	 if (next == null)
	    		 throw new IllegalArgumentException(MsgManager.errInvariant("No such " + chunkName + " in the elementRegistry"));
			 result.add(next);
	     }
		return result;
	}
	
	protected void addDefaultCol(DbbSqlChunk defaultChunk) {
		//DbC//
		assert (defaultChunk != null): new IllegalArgumentException(MsgManager.errContract("default chunk is null"));
		//Dbc//
		defaultCols.add(defaultChunk);
	}
	
	/**
	 * Every class that extend Dbb should implement these methods
	 * Note that Dbb.java doesn't refer in anyway to DbbWidget objects!
	 */
	abstract protected DbbSelectionList makeDbbSelectionList(HashMap attrs) throws DbbXMLException;
	abstract protected DbbChoice makeDbbChoice(HashMap attrs) throws DbbXMLException;
	abstract protected DbbArrowButton makeDbbArrowButton(HashMap attrs) throws DbbXMLException;
	abstract protected DbbComboBox makeDbbComboBox (HashMap attrs) throws DbbXMLException;
	abstract protected DbbCheckBox makeDbbCheckBox(HashMap attrs) throws DbbXMLException;
	abstract protected DbbPanel makeDbbPanel(HashMap attrs, Element element)throws DbbXMLException;
	abstract protected DbbSession makeDbbSession(HashMap attrs) throws DbbXMLException;
	abstract protected DbbSqlChunk makeDbbSqlChunk(HashMap attrs) throws DbbXMLException;
	abstract protected DbbSqlOperator makeDbbSqlOperator(HashMap attrs) throws DbbXMLException;    
	abstract protected DbbSqlTable makeDbbSqlTable( HashMap attrs ) throws DbbXMLException;
	abstract protected DbbTextField makeDbbTextField( HashMap attrs ) throws DbbXMLException;
	abstract protected DbbView makeDbbView( Element element ) throws DbbXMLException;
	abstract protected DbbTextFieldPattern makeDbbTextFieldPattern(  HashMap attrs ) throws DbbXMLException;
	abstract protected DbbIntegerTextField makeDbbIntegerTextField(  HashMap attrs ) throws DbbXMLException;
	abstract protected DbbCoordinateTextField makeDbbCoordinateTextField(  HashMap attrs ) throws DbbXMLException;
	abstract protected DbbRadioList makeDbbRadioList(  HashMap attrs ) throws DbbXMLException;
	abstract protected DbbSqlChunkGroupMember makeDbbSqlChunkGroupMember(  HashMap attrs ) throws DbbXMLException;
	
	
	
	
	
	
}
