package org.eso.ohs.core.dbb.client;

// $Id: DbbWidget.java,v 1.8 2005/07/08 13:38:59 gdonino Exp $

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.utilities.MsgManager;

/** 
 * DbbWidget is an abstract class defining the general behavior
 * of SQL widget objects. These GUI objects allow the user to define
 * DbbSqlChunk objects.
 * 
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * @author A M Chavan, ESO 27-Jul-99
 *
 * @see DbbSqlChunk
 * 
 */
public abstract class DbbWidget {

    /** Default value to be displayed by the widget. */
    protected String default_;
	/**
		* Field stdlog_
		*/
	   protected static Logger stdlog_ = Logger.getLogger(DbbWidget.class);
  
 /** 
  * parent frame to which cursor modification is applied during fireDbbWidgetAction. 
  */
  JFrame parentFrame_ = null;

    /** GUI label component. */
    protected JLabel label_;

    /** GUI widget component. */
    protected JComponent widget_;

    /** SQL chunk whose values the user can define. */
    protected DbbSqlChunk sqlChunk_;

    /**
     * List of additional sql chunks defined in order to specify more
     * deeply the sql clauses.
     */
    protected Vector sqlAdditionalChunks_ = new Vector();

    /** List of listeners to be notified when firing a DbbWidgetEvent. */
    protected EventListenerList listenerList_ = new EventListenerList();

    /** Last fired DbbWidgetEvent object. */
    protected DbbWidgetEvent widgetEvent_ = null;

    /** 
     * Flag: if true, the widget should extend to fill its assigned space.
     */
    // amchavan, 13-Jun-2002: added
    protected boolean shouldFill_ = true;

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /** 
     * Create a DbbWidget object.
     * 
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object.
     *  
     */
    public DbbWidget(String labeltext, DbbSqlChunk chunk) {

	sqlChunk_ = chunk;
	label_ = new JLabel(labeltext, JLabel.RIGHT);
    }

    /** 
     * Create a DbbWidget object.
     * If the parentFrame is != null, when the fireDbbWidgetAction
     * is started, a watchCursor is applied. Otherwise the cursor won't be changed
     *
     * @param parentFrame frame to which the cursor changing is applied.
     * @param labeltext	Label text to be displayed.
     * @param chunk	DbbSqlChunk object.
     *  
     */
    public DbbWidget(JFrame parentFrame, String labeltext, DbbSqlChunk chunk) {
	parentFrame_ = parentFrame;
	sqlChunk_ = chunk;
	label_ = new JLabel(labeltext, JLabel.RIGHT);
    }

     /**
      * Create a DbbWidget object.
      * If the parentFrame is != null, when the fireDbbWidgetAction
      * is started, a watchCursor is applied. Otherwise the cursor won't be changed
      *
      * @param parentFrame frame to which the cursor changing is applied.
      * @param labeltext	Label text to be displayed.
      * @param chunk	DbbSqlChunk object.
      */
	public DbbWidget(JFrame parentFrame, String labeltext, DbbSqlChunk chunk, DbbSqlChunk[] condChunk){
		parentFrame_ = parentFrame;
		sqlChunk_ = chunk;
		label_ = new JLabel(labeltext, JLabel.RIGHT);
		//needs code to manage conditional chunk
	}
	
	public DbbWidget(String labeltext, DbbSqlChunk chunk, DbbSqlChunk[] condChunk){
		sqlChunk_ = chunk;
		label_ = new JLabel(labeltext, JLabel.RIGHT);
//		needs code to manage conditional chunk
	}
    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /**
     * @return True if the widget should extend to fill its assigned
     *         space, false otherwise. 
     */
    // amchavan, 13-Jun-2002: added
    public boolean shouldFill() { return shouldFill_; }

    /**
     * @return The current text (or the value) displayed by the widget.
     * @see #widget_
     */
    public abstract String get();

    /**
     * Set the value displayed in the interactive widget.
     * @param value String to be displayed as new value.
     * @see #widget_
     */
    public abstract void set(String value);
    
	/**
	  * Set a specific tooltip for the widget.
	  * @param s Tooltip text
	  */
	 public void setToolTipText(String s) {
	 widget_.setToolTipText(s);
	 }

	 /** @return The tooltip text defined for the widget */
	 public String getToolTipText() {
	 return widget_.getToolTipText();
	 }

    /**
     * @return The widget attribute.
     * @see #widget_
     */
    public Component getWidget() { return widget_; }

    public void setPreferredCharWidth(int char_number, boolean expand) {
	if(widget_ != null) {
	    int width = getCharWidth(widget_,char_number);
	    int height = (widget_.getPreferredSize()).height;

	    try {
		if(expand)
		    widget_.setPreferredSize(new Dimension(width, height));
		else {
		    widget_.setMaximumSize(new Dimension(width, height));
		    widget_.setMinimumSize(new Dimension(width, height));
		    widget_.setSize(width, height);
		}
	    }
	    catch (Exception e) {
		widget_.setSize(width, height);
	    }

	}
	
    }

    /**
     * @param widget
     * @param char_number
     * @return int
     */
    protected int getCharWidth(JComponent widget, int char_number) {
        FontMetrics fm = widget.getFontMetrics(widget.getFont());
	    int char_len = fm.stringWidth("0");
	    int width = char_len * char_number;
        return width;
    }

    public void setPreferredCharWidth(int char_number) {
	setPreferredCharWidth(char_number, true);
    }

    /**
     *
     * Add a sql chunk to the list of additional chunks in ortder to
     * allow the user to specify more deeply the sql conditions.
     *    
     * @param new_chunk Additional sql chunk.
     */
    public void addSqlChunk(DbbSqlChunk new_chunk) {
    if (new_chunk == null)
    	throw new IllegalArgumentException(MsgManager.errPrecondition("The conditional chunk is null"));
	sqlAdditionalChunks_.addElement(new_chunk);
    }


    /**
     * Get an array containing the additional sql chunks defined to get
     * a more specific clause.
     * @return sql chunks
     * @exception DbbInvalidSqlOperatorType not a known sql operator
     */
    public DbbSqlChunk[] getAdditionalSqlChunks() 
	throws DbbInvalidSqlOperatorType {
    
	    int len = sqlAdditionalChunks_.size();
	    DbbSqlChunk[] ret_array = new DbbSqlChunk[len];
	    sqlAdditionalChunks_.copyInto(ret_array);
      
	    return ret_array;
	}


    /**
     * @return An object reference to the DbbSqlChunk object associated
     *          to this widget, updated with the current displayed
     *          values.
     *      
     *  @exception DbbInvalidSqlOperatorType If the text/value currently
     *                                       displayed in the widget
     *                                       does not translate to a
     *                                       valid operator. 
     */
    public DbbSqlChunk getSqlChunk() throws DbbInvalidSqlOperatorType { 

	setChunk();
	return sqlChunk_; 
    }

    /**
     * @return The widget label.
     * @see #label_
     */
    public JLabel getLabel() { return label_; }

    /**
     * Set the default value to be displayed in the interactive widget.
     * @param value New value default (given as String).
     * @see #widget_
     */
    public void setDefault(String value) {
	default_ = value;
    }
  
    /**
     * Reset the widget to its default state.
     */
    public void reset() {
	set(default_);
    }

    /**
     * Check if the widget contents have been cleared.
     *
     * @return A boolean value: <code>true</code> if it is empty,
     *         <code>false</code> otherwise.  
     */
    public abstract boolean isEmpty();

    /**
     * Set the text displayed in the label widget.
     * @param text New label text.
     * @see #label_
     */
    public void setLabelText(String text) { 
	label_.setText(text); 
    }

    /** 
     * Update the associated chunk with the text/value currently
     * displayed in the widget.
     * Will update the chunk's value, possibly the operator as well.
     *
     * @exception DbbInvalidSqlOperatorType If the text/value currently
     *                                      displayed in the widget
     *                                      does not translate to a
     *                                      valid operator. 
     */
    protected void setChunk() throws DbbInvalidSqlOperatorType {
        if (sqlChunk_!= null) {
        	sqlChunk_.setValue(get()); 
        }
    }

    /** 
     * Set the widget visible or invisible.
     *  
     * @param state boolean: if <code>true</code> make the
     * widget visible, if <code>false</code> make it invisible.
     */
    public void setVisible(boolean state) {
	label_.setVisible(state);
	widget_.setVisible(state);
    }
	
	/**
	 * 
	 * @return True if the widget is visible.
	 */
	public boolean isVisible(){
		return widget_.isVisible();
	}
    /** 
     * Enable or disable the widget.
     *  
     * @param state boolean: if <code>true</code> enable the
     * widget, if <code>false</code> disable it.  
     */
    public void setEnabled(boolean state) {
	widget_.setEnabled(state);
    }

    //----------------------------------------
    // Event methods
    //----------------------------------------

    /**
     * Add a listener to the listeners' list. This listener will be
     * notified when firing an event of the DbbWidgetEvent class.  
     * 
     * @param l DbbWidgetListener reference.
     */
    public void addListener(DbbWidgetListener l) {
	listenerList_.add(DbbWidgetListener.class, l);
    }

    /**
     * Remove a listener from the listeners' list.
     * @param l DbbWidgetListener reference.
     */
    public void removeListener(DbbWidgetListener l) {
	listenerList_.remove(DbbWidgetListener.class, l);
    }

    /** 
     * Notify all listeners that have registered interest for
     * notification on this event type. The event instance is lazily
     * created using the parameters passed into the fire method.  
     */
  public void fireDbbWidgetAction() {
      Cursor currentCursor =  null;
      if (parentFrame_ != null) {
	  currentCursor = parentFrame_.getCursor();
	  parentFrame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      }
      
        // update all sqlChunks before firing the event....
        stdlog_.debug("############### Fire WidgetAction "+ getLabel().getText()+" value \""+get()+"\"");
        boolean active = !isEmpty();
        sqlChunk_.setActive(active);
	stdlog_.debug("############### getActive WidgetAction "+sqlChunk_.isActive());
    for (int i = 0; i < sqlAdditionalChunks_.size(); i++) {
            ((DbbSqlChunk)sqlAdditionalChunks_.elementAt(i)).setActive(active);
        }

	// Guaranteed to return a non-null array
	Object[] listeners = listenerList_.getListenerList();

	// Process the listeners last to first, notifying
	// those that are interested in this event
	for(int i = listeners.length-2; i>=0; i-=2) {

	    if (listeners[i] == DbbWidgetListener.class) {

		// Lazily create the event:
		if (widgetEvent_ == null)
		    widgetEvent_ = new DbbWidgetEvent(this);

		((DbbWidgetListener)listeners[i+1]).dbbWidgetAction(widgetEvent_);
	    }              
	}

	if (parentFrame_ != null) {
	    parentFrame_.repaint();
	    parentFrame_.setCursor(currentCursor);
	}   
    }      
}
