package org.eso.ohs.core.dbb.gui;

// $Id: DbbArrowIcon.java,v 1.1 2004/02/17 11:16:25 tcanavan Exp $

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Polygon;

import javax.swing.Icon;

/**
 * Create an icon drawing an up or a down arrow.
 * It shall be used when creating an arrow three state button.
 *
 * @author Carlo Boarotto, ESO 13-Jul-1999
 **/
public class DbbArrowIcon implements Icon {

    /** Icon max width */
    private final static int defWidth = 12;

    /** Icon max height */
    private final static int defHeight = 12;

    /** X margin */
    private final static int x_margin = 2;

    /** Y margin */
    private final static int y_margin = 2;

    /** Orientation flag (0 down arrow, 1 up arrow). */
    private int state = 0;

    /** Icon width. */
    private int width = defWidth;

    /** Icon height. */
    private int height = defHeight;

    /** Array of y coordinates to draw the icon */
    private int[] yn = new int[3];

    /** Array of x coordinates to draw the icon */
    private int[] xn = new int[3];


    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create an icon drawing an up or a down arrow.
     *
     * @param w Arrow width.
     * @param h Arrow height.
     * @param s Arrow orientation (0 down arrow, 1 up arrow).
     */
    public DbbArrowIcon(int w, int h, int s) {
	width = w;
	height = h;
	state = s;
    }


    /**
     * Create an icon drawing an up or a down arrow.
     * The icon dimension will have the default (9,9) value.
     *
     * @param s Arrow orientation (0 down arrow, 1 up arrow).
     */
    public DbbArrowIcon(int s) {
	this(defWidth, defHeight, s);
    }


    /**
     * Create an icon drawing a down arrow.
     * The icon dimension will have the default (9,9) value.
     *
     * @param s Arrow orientation (0 down arrow, 1 up arrow).
     */
    public DbbArrowIcon() {
	this(defWidth, defHeight, 0);
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Interface's paintIcon implementation: paint the icon itself.
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {

	xn[0] = x + x_margin;
	xn[1] = x + width/2;
	xn[2] = x + width - x_margin;

	int top = y + y_margin;
	int bottom = y + height - y_margin;

	if(state == 1) {
	    yn[0] = bottom;
	    yn[1] = top;
	    yn[2] = bottom;
	}
	else {
	    yn[0] = top;
	    yn[1] = bottom;
	    yn[2] = top;
	}

	g.draw3DRect(x, y, width, height, true);

	Polygon p = new Polygon(xn, yn, 3);
	if(state != 0) {
	    g.setColor(Color.black);
	    g.drawPolygon(p);
	    g.fillPolygon(p);
	}
    }

    /** @return The icon width. */
    public int getIconWidth() { return width; }


    /** @return The icon height. */
    public int getIconHeight() { return height; }
}
