package org.eso.ohs.core.docview.gui;

import java.awt.Color;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;

/**
 * A GUI adaptor to interface between DocView and Swing JComboBox objects.
 * By using a GuiAdaptor, DocView itself can work with a much wider range
 * of GUI components.
 * 
 * <p>
 * $Id: ComboBoxAdaptor.java,v 1.2 2004/07/17 14:59:58 janwunah Exp $
 * <p>
 * @author Bob Kemp
 */
public class ComboBoxAdaptor implements GuiAdaptor, ItemListener {

    /** GUI component whose value is to be read/updated */
    private JComboBox cBox_;

    /** Has the GUI component been modified?  Reset by DocView. */
    private boolean modified_;


    /**
     * Constructor
     * 
     * @param cBox	GUI component to be linked to
     */
    public ComboBoxAdaptor(JComboBox cBox) {
	cBox_ = cBox;
	cBox.addItemListener(this);
    }


    /**
     * Set the background colour of the underlying GUI component
     * 
     * @param bg	new value of background colour
     */
    public void setForeground(Color bg) {
	cBox_.setForeground(bg);
    }

    /**
     * Enable the underlying GUI component
     * 
     * @param enabled	true if component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
	cBox_.setEnabled(enabled);
    }

    /**
     * Is the underlying GUI component enabled?
     * 
     * @return	true if component should be enabled, false otherwise
     */
    public boolean isEnabled() {
	return cBox_.isEnabled();
    }

    /**
     * Set a new value for the GUI components value
     * 
     * @param newValue	new value represented as a String
     */
    public void setValue(String newValue) {
	if (newValue != null && newValue.length() > 0) {
	    cBox_.setSelectedItem(newValue);
	}
    else {
        cBox_.setSelectedIndex(0);
    }
    }

    /**
     * Get the current value of the GUI component
     * 
     * @return	current value represented as a String
     */
    public String getValue() {
    	String result = null;
		if (cBox_.getSelectedItem() != null) {
			result =cBox_.getSelectedItem().toString(); 
		}
		return result;
    }

    /**
     * Flag the component as having been modified (or not modified)
     * 
     * @param	new value of the flag
     */
    public void setModified(boolean modified) {
	modified_ = modified;
    }

    /**
     * Has the GUI component been modified?
     * 
     * @return	current value of the flag
     */
    public boolean isModified() {
	return modified_;
    }


    /** A listener method for changes to the underlying GUI component */
    public void itemStateChanged(ItemEvent ev) {
        //org.eso.ohs.utilities.DebugUtils.stackTrace();  	
         cBox_.requestFocus();
         setModified(true);
    }

}

 
