package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of integer value between 1-99 
 * 
 * <p>
 * $Id: PositiveIntegerModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */

public class PositiveIntegerModel extends InputMaskModel {

    private final float MAX  = 100;
    private final float MIN = 0;

    /**
     * The regular expression describing legal buffer values.
     * Looking for values between 0 and 99
     */
    private final static String legalValuesPattern_ =
	"(\\d{0,4})?";

    /**
     * Constructor for the model.
     */
    public PositiveIntegerModel() {
	super(legalValuesPattern_);
    }
    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();

            Float f = Float.valueOf(buffer_val);
            if(f.floatValue() < MIN) {
                remove(offset, str.length());
            }
            else if(f.floatValue() > MAX) {
                remove(offset, str.length());
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}

