package org.eso.ohs.core.gui.widgets;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.HTMLDocument;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.utilities.PrintManager;
import org.eso.ohs.core.utilities.FileIOUtils;

/**
 * Class HTMLDisplayWidget
 * 
 * This is a  HTML Display widget which allows you to Display and
 * Save Text in a TextArea. 
 * 
 * @version $Revision: 1.2 $
 * @author pnunes, Feb 11, 2005
 */

// $Id: HTMLDisplayWidget.java,v 1.2 2005/02/25 10:20:34 pnunes Exp $ 
public class HTMLDisplayWidget extends JFrame {    
    
    private static Logger stdlog_ = Logger.getLogger(HTMLDisplayWidget.class);

    /** Menu bar   */
    JMenuBar menuBar_;
    
    /** Save Report MenuItem */
    private JMenuItem saveReport_;

    /** Open Report MenuItem */
    private JMenuItem openReport_;

    /** Print Report Menu item */
    private JMenuItem printReport_;

    /** Area to display contents in */
    private JEditorPane editorPane_ ;
    
    /** Close the widget down */
    private JMenuItem exit_;

    /** File menu */
    private final JMenu fileMenu_ = new JMenu("File");

    /** used to keep track of paths */
    private static String previousPath_ = null;
    private String currentPath_ = null;
         
    /**  used to keep track of the HTML string and CSS styles */
    private String contents_;
    private String styleSheetFilename_;
    
    /**
     * HTMLDisplayWidget widget constructor
     * 
     * @param title
     *            of Widget
     * @param saveOption
     *            Add Save button
     * @param openOption
     *            Add Open button
     * @param printOption
     *            Add Print Button
     * @throws IOException
     */    
    public HTMLDisplayWidget(String title , boolean saveOption , boolean openOption , boolean printOption , String styleSheetFilename ) {        
        initialize(true, true, true , title);
        styleSheetFilename_ = styleSheetFilename;
    }


    /**
     * set the path user for saving the content if the String is not a valid
     * directory the value is not set
     * 
     * @param currentPath
     *            the directory
     */
    public void setCurrentPath(String currentPath) {
        File d = new File(currentPath_);
        if (d.isDirectory()) {
            currentPath_ = currentPath;
        } else {
            stdlog_.warn("Directory not found: " + currentPath);
        }
    }

    /**
     * set the path user for saving the content as the last used by the
     * application
     */
    public void setCurrentPathAsLast() {
        if (previousPath_ != null) {
            File d = new File(previousPath_);
            if (d.isDirectory()) {
                currentPath_ = previousPath_;
            } else {
                stdlog_.warn("Directory not found: " + previousPath_);
            }
        } else {
            stdlog_.warn("previous path not set");
        }
    }

    /**
     * Initialise the HTML Display Widget
     * 
     * @param saveOption
     * @param openOption
     * @param printOption
     * @throws IOException
     */
    private void initialize(boolean saveOption, boolean openOption, boolean printOption , String title) {
        setTitle(title);               
        setVisible(false);
                
        menuBar_ = new JMenuBar();
        setJMenuBar(menuBar_);
        fileMenu_.setMnemonic('F');
        menuBar_.add(fileMenu_);
        saveReport_ = new JMenuItem("Save", 'S');
        openReport_ = new JMenuItem("Open", 'E');
        printReport_ = new JMenuItem("Print", 'P');
        
        exit_ = new JMenuItem("Close", 'w');
        exit_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
                InputEvent.CTRL_MASK, false));

        //add menu items 
        if (saveOption)
            fileMenu_.add(saveReport_);
        if (openOption)
            fileMenu_.add(openReport_);
        if (printOption)
            fileMenu_.add(printReport_);
        fileMenu_.add(exit_);
                            
        try {
            // initialize text pane
            initializeEditorPane();
        } catch (IOException e) {
            
            e.printStackTrace();
        }        
        getContentPane().add( new JScrollPane( editorPane_  ));        
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(screenSize.width*3/4,screenSize.height*3/4);       
        createActionListeners();
               
    }
    
    private void initializeEditorPane() throws IOException {
        editorPane_ = new JEditorPane();        
        editorPane_.setContentType("text/html");
        editorPane_.getDocument().putProperty("IgnoreCharsetDirective", Boolean.valueOf(true)); 
        editorPane_.setEditable(false);      
        
    }
    
    /**
     * Set the text area with string data
     * 
     * @param  String Contents
     * @throws BadLocationException
     * @throws IOException
     * @throws IOException
     */
    public void setContents(String contents) throws IOException, BadLocationException  {
        
                       
        contents_ = contents;
        editorPane_.setText(contents_);
        editorPane_.repaint();
        editorPane_.setCaretPosition(0);

    }
    
    

    /**
     * Clear text Area
     */
    public void clearContents() {
        editorPane_.setText("");
    }


    
    /**
     * Shows a message dialog
     * 
     * @param parent
     * @param title
     * @param line1
     * @param line2
     * @param line3
     */
    private void messageDialog( Component parent, String title , String line1, String line2, String line3 ) {
        
        JOptionPane.showMessageDialog(
                parent,
     		new Object[] {
     		  line1,
     		  line2,
     		  line3
     		},
     		title,
     		JOptionPane.ERROR_MESSAGE
     	    );
    }
    
    
    /**
     * Create new Save action listener
     */

    public void createActionListeners() {
        saveReport_.addActionListener(new SaveActionListener());
        openReport_.addActionListener(new OpenActionListener());
        printReport_.addActionListener(new PrintActionListener());
        exit_.addActionListener(new ExitActionListener());
         
    }

    /**
     * Set text for Save menu
     * 
     * @param s
     *            Set the save text
     */
    public void setSaveMenuText(String s) {
        saveReport_.setText(s);
    }

    /**
     * Set the open Text
     * 
     * @param s
     *            Set the open text
     */

    public void setOpenMenuText(String s) {
        openReport_.setText(s);
    }

    protected class SaveActionListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {
            JFileChooser chooser = null;

            if (currentPath_ != null) {
                File d = new File(currentPath_);
                if (d.exists())
                    chooser = new SwingFileChooser(currentPath_);
            } else
                chooser = new SwingFileChooser();

            //chooser.setFileFilter(new FileFilter("html","HTML documents"));
            int option = chooser.showSaveDialog(fileMenu_);

            if (option == JFileChooser.APPROVE_OPTION) {
                
                try {
                    File f = new File(chooser.getSelectedFile().getAbsolutePath());
                    int returnValue = JOptionPane.OK_OPTION;
                    
                    if (f.exists()) {
                        JOptionPane.showConfirmDialog(chooser,
                            "Do you wish to overwrite file " + f.getName() + " ?");
                    }
                    

                    /** Write the document together with the CSS file   */
                    if (returnValue == JOptionPane.OK_OPTION) {
                        
                        // save HTML file
                        FileOutputStream outHTML = new FileOutputStream(f);                                                
                        outHTML.write(contents_.getBytes());                        
                        currentPath_ = chooser.getCurrentDirectory().toString();
                        previousPath_ = currentPath_;
                        
                    }
                } catch (Throwable t) {
                    t.printStackTrace();

                }
            }
        }
    }

    protected class OpenActionListener implements ActionListener {
        
            public void actionPerformed(ActionEvent ae) {
                JFileChooser chooser = new SwingFileChooser();
                int option = chooser.showOpenDialog(fileMenu_);
                if (option == JFileChooser.APPROVE_OPTION) {
                    editorPane_.setText("");
                    try {
                        setContents(FileIOUtils.fileToString(chooser
                                .getSelectedFile().getAbsolutePath()) );

                        setTitle(chooser.getSelectedFile().getName());
                    } catch (Throwable t) {
                        editorPane_.setText("Failed to read file contents.");
                    }
                }
            }
        
        
    }
    
    protected class PrintActionListener implements ActionListener {
        public void actionPerformed(ActionEvent ae)  {
           PrintManager pm = new PrintManager();
           pm.printHTML( (HTMLDocument)editorPane_.getDocument());
          
            
        }           
    }
    
    protected class ExitActionListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            dispose();
        }
    }

    

}





