package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFileChooser;

import org.eso.ohs.core.utilities.FileIOUtils;

/* ImagePreview.java is a 1.4 example used by FileChooserDemo2.java. */
public class ImagePreview
    extends JComponent
    implements PropertyChangeListener {
    ImageIcon thumbnail = null;
    File file = null;

    public ImagePreview(JFileChooser fc) {
        setPreferredSize(new Dimension(300, 150));
        fc.addPropertyChangeListener(this);
    }

    public void loadImage() {
        //thumbnail = null;

        if (file == null) {
            thumbnail = null;
            return;
        }

        //Don't use createImageIcon (which is a wrapper for getResource)
        //because the image we're trying to load is probably not one
        //of this program's own resources.
        try {
            String absPath = file.getAbsolutePath();
            byte[] bData = FileIOUtils.readBinaryFile(absPath);
            ImageIcon tmpIcon = new ImageIcon(bData);
            if (tmpIcon != null) {
                if (tmpIcon.getIconWidth() > 200) {
                    thumbnail =
                        new ImageIcon(
                            tmpIcon.getImage().getScaledInstance(
                                200,
                                -1,
                                Image.SCALE_DEFAULT));
                } else { //no need to miniaturize
                    thumbnail = tmpIcon;
                }
            }
        } catch (IOException e) {
            ErrorMessages.announceIOError(null, e);
        }
    }

    public void propertyChange(PropertyChangeEvent e) {
        boolean update = false;
        String prop = e.getPropertyName();

        //If the directory changed, don't show an image.
        if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(prop)) {
            file = null;
            update = true;

            //If a file became selected, find out which one.
        } else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop)) {
            file = (File) e.getNewValue();
            update = true;
        }

        //Update the preview accordingly.
        if (update) {
            thumbnail = null;
            if (isShowing()) {
                loadImage();
            }
            repaint();
        }
    }

    public void paintComponent(Graphics g) {
        if (thumbnail == null) {
            loadImage();
        }
        if (thumbnail != null) {
            int x = getWidth() / 2 - thumbnail.getIconWidth() / 2;
            int y = getHeight() / 2 - thumbnail.getIconHeight() / 2;

            if (y < 0) {
                y = 0;
            }

            if (x < 5) {
                x = 5;
            }
            thumbnail.paintIcon(this, g, x, y);
        }
    }
    
    public static void main(String[] args) {

        // ExtensionFileFilter jpegFilter =
        //    new ExtensionFileFilter("jpg", "Finding Charts");
        JFileChooser chooser = new JFileChooser();

        ///chooser.addChoosableFileFilter(new ImageFilter());
        //chooser.setAcceptAllFileFilterUsed(false);

        //Add custom icons for file types.
        //chooser.setFileView(new ImageFileView());

        //Add the preview pane.
        chooser.setAccessory(new ImagePreview(chooser));

        //chooser.setMultiSelectionEnabled(true);
        chooser.showDialog(null, "Attach Finding Chart");
    }

}
