package org.eso.ohs.core.gui.widgets;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.models.SubListModel;

/**
 * This class creates two list boxes: the one on the left contains a
 * list of entries. Every time an item is selected the right list box
 * is updated with values referring to the selected item.
 * 
 *  @see org.eso.ohs.core.gui.models.SubListModel
 */

public class NestedListBox extends JDialog implements ListSelectionListener {

	private static final String rcsid =
		"$Id: NestedListBox.java,v 1.3 2004/04/08 12:09:51 tcanavan Exp $";

	private static Logger stdlog_ = Logger.getLogger(NestedListBox.class);

	/**
	 * OptionPane status
	 */

	private int returnValue_ = JOptionPane.CLOSED_OPTION;

	/**
	 * label above main column
	 */

	private JLabel label1_ = new JLabel();

	/**
	 * List to hold main data
	 */
	private JList main_ = new JList();

	/**
	 * label above sub column
	 */
	private JLabel label2_ = new JLabel();

	/**
	 * List for sub data.
	 */

	private JList sub_ = new JList();

	/** 
	 * Main data model
	 */

	private DefaultListModel mainModel_;

	/**
	 * Sub data model
	 */

	private SubListModel subModel_;

	/**
	 * Button to commit changes
	 */

	private JButton ok_ = new JButton();

	/**
	 * Button to cancel changes
	 */

	private JButton cancel_ = new JButton();

	/**
	 * GridBagConstraints
	 */

	private GridBagConstraints c_ = new GridBagConstraints();

	/** 
	 * Panel to add components to.
	 */

	private JPanel panel_ = new JPanel();

	/**
	 * Create a NestedListBox Dialog box
	 * @param f Frame of Dialog box
	 * @param mainModel model for main data
	 * @param subModel model for desination list 
	 */

	public NestedListBox(
		JFrame f,
		DefaultListModel mainModel,
		SubListModel subModel) {
		super(f, "Export", true);
		mainModel_ = mainModel;
		subModel_ = subModel;
		main_.setModel(mainModel_);
		main_.addListSelectionListener(this);
		main_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		main_.setValueIsAdjusting(false);
		sub_.setModel(subModel_);
		init();
		setContentPane(panel_);
		initOKCancel();
		pack();
	}

	/**
	 * Sub model
	 * @return sub model
	 */

	public SubListModel getSubModel() {
		return subModel_;
	}

	/**
	 * Main Model
	 * @return main model
	 */

	public DefaultListModel getMainModel() {
		return mainModel_;
	}

	/**
	 * Return Value used by OptionPane
	 * 
	 * @return returnValue
	 */

	public int getValue() {
		return returnValue_;
	}

	/**
	 * Set the main model for the main list
	 * @param mainData data to be copied into mainModel
	 */

	public void setMainModel(Vector mainData) {
		for (int i = 0; i < mainData.size(); i++)
			mainModel_.addElement(mainData.elementAt(i));
	}

	/**
	 * Set the main model for the main list
	 * @param mainData data to be copied into mainModel
	 */

	public void setMainModel(Object[] mainData) {
		for (int i = 0; i < mainData.length; i++)
			mainModel_.addElement(mainData[i]);
	}

	/**
	 * Set the sub model for the sub list
	 * @param mainData data to be copied into subModel
	 */

	public void setSubModel(Vector subData) {
		for (int i = 0; i < subData.size(); i++)
			subModel_.addElement(subData.elementAt(i));
	}

	/**
	 * Set the sub model for the sub list
	 * @param mainData data to be copied into subModel
	 */

	public void setSubModel(Object[] subData) {
		for (int i = 0; i < subData.length; i++)
			subModel_.addElement(subData[i]);
	}

	/**
	 * Return the sub list as an array of objects
	 * @return sub list as an array of objects
	 */

	public Object[] getSubList() {
		return subModel_.toArray();
	}

	/**
	 * Return the selected entry in the main list
	 * @return the selected entry in the main list
	 */

	public String getSelMain() {
		return (String) main_.getSelectedValue();
	}

	/**
	 * Return the selected entry in the sub list
	 * @return the selected entry in the sub list
	 */

	public String getSelSub() {
		return (String) sub_.getSelectedValue();
	}

	/**
	 * Return an array containing the selected values in the sub list
	 * @return the selected entries in the sub list
	 */

	public String[] getSelSubValues() {

		Object[] values = sub_.getSelectedValues();
		String[] stringValues = new String[values.length];

		for (int i = 0; i < values.length; i++) {
			stringValues[i] = (String)values[i];
		}
		return stringValues;
	}

	/** 
	 * Set the cell renders for the main and subl list
	 * 
	 * @param renderer renderer for items in sub and main lists
	 */
	public void setCellRenderers(ListCellRenderer renderer) {
		main_.setCellRenderer(renderer);
		sub_.setCellRenderer(renderer);
	}

	/**
	 * Set the title for the main list
	 * @param title title for main list 
	 */

	public void setMainTitle(String title) {
		label1_.setText(title);
	}

	/**
	 * Set the tite for the sub list
	 * @param title for sub list
	 */

	public void setSubTitle(String title) {
		label2_.setText(title);
	}

	/**
	 * Inirialise the member variables and add gui components 
	 * to the frame.
	 */
	private void init() {
		main_.setModel(mainModel_);
		sub_.setModel(subModel_);
		sub_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
		main_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
		panel_.setLayout(new GridBagLayout());

		c_.gridx = 0;
		c_.gridy = 0;
		c_.gridwidth = 1;
		c_.gridheight = 1;
		label1_.setText("Main list");
		final JScrollPane mainScrollPane =
			new JScrollPane(
				main_,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		label1_.setHorizontalAlignment(JLabel.LEFT);
		panel_.add(label1_, c_);
		c_.gridheight = 4;
		c_.gridy = 1;
		panel_.add(mainScrollPane, c_);
		c_.gridx = 1;
		c_.gridy = 1;
		c_.gridheight = 1;

		c_.gridx = 2;
		c_.gridy = 0;
		c_.fill = GridBagConstraints.BOTH;
		label2_.setText("Sub list");
		label2_.setHorizontalAlignment(JLabel.LEFT);
		panel_.add(label2_, c_);
		c_.gridheight = 4;
		c_.gridy = 1;
		final JScrollPane subScrollPane =
			new JScrollPane(
				sub_,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		panel_.add(subScrollPane, c_);
	}

	/**
	 * Add listeners for Dialog buttons
	 */

	public void setOkAction(ActionListener okAction) {

		ok_.addActionListener(okAction);
	}

	public void setCancelAction(ActionListener cancelAction) {

		cancel_.addActionListener(cancelAction);
	}

	public void setOkLabel(String okLabel) {
		ok_.setText(okLabel);
	}

	public void setCancelLabel(String cancelLabel) {
		cancel_.setText(cancelLabel);
	}

	/**
	 * Initialise ok and cancl buttons and there grid bag constraints
	 */

	private void initOKCancel() {
		c_.gridheight = 1;
		c_.gridy = 5;
		c_.gridx = 1;
		addOKButton();
		c_.gridx = 2;
		addCancelButton();
	}

	/**
	 * Add OK button to panel
	 */

	public void addOKButton() {
		ok_.setText("Save");
		panel_.add(ok_, c_);
		c_.gridx = 2;

	}

	/**
	 * Add Cancel button to Panel
	 */

	public void addCancelButton() {
		cancel_.setText("Cancel");
		panel_.add(cancel_, c_);
	}

	public void valueChanged(ListSelectionEvent e) {

		if (!e.getValueIsAdjusting()) {
			stdlog_.debug(e);
			String proposalCode = (String) main_.getSelectedValue();
			if (proposalCode != null) {
				subModel_.updateList(proposalCode);
			}
		}
	}
}
