/* Created on Apr 24, 2004 */

package org.eso.ohs.core.utilities;


/**
 * Describes a range of integers: lower -> upper.
 * 
 * @author amchavan
 * @version $Revision: 1.1 $
 */

// $Id: Range.java,v 1.1 2004/04/26 15:52:42 amchavan Exp $

public class Range {

    /** Separator string for lower-upper input strings */
    public static final String SEPARATOR = "..";
    
    /** Lower limit of the range */ 
    protected int lower;

    /** Upper limit of the range */ 
    protected int upper;
    
    /** Absolute lower limit of the range */ 
    protected int absLower = Integer.MIN_VALUE;

    /** Absolute upper limit of the range */ 
    protected int absUpper = Integer.MAX_VALUE;
    
    /** Default constructor: build a range with Integer.MIN_VALUE
     * as lower limit and Integer.MAX_VALUE as upper limit
     */
    public Range() {
        this( Integer.MIN_VALUE, Integer.MAX_VALUE );
    }
    
    /** 
     * Construct a range with the given lower and upper limits
     * @throws RuntimeException if upper smaller than lower 
     */
    public Range( int lower, int upper ) {
        this.lower = lower;
        this.upper = upper;
        checkLowerUpper();
    }

    /**
     * @throws RuntimeException if value of field upper smaller than 
     *         value of field lower
     */
    protected void checkLowerUpper() throws RuntimeException {
        if( this.upper < this.lower )
            throw new RuntimeException( "upper (" + upper
                    + ") is smaller than lower (" + lower + ")" );
    }
    
    /** @return <code>true</code> if input range has the same upper
     *          and lower limits as this range; <code>false</code>
     *          otherwise.
     */
    public boolean equals( Range r  ) {
		return (getLower() == r.getLower()) && (getUpper() == r.getUpper());
    }
    
    /**
     * @return Returns the lower limit of the range.
     */
    public int getLower() {
        return lower;
    }
    
    /**
     * @return Returns the upper limit of the range.
     */
    public int getUpper() {
        return upper;
    }
    
    /**
     * @return <code>true</code> if input number is within this range; 
     *         <code>false</code> otherwise. The limits of the range
     *         are considered within the range itself: so, for instance,
     *         0 is within 0..9.
     */
    public boolean includes( int num ) {
        return ( num >= getLower() && num <= getUpper() );
    }
    
    /**
     * @return A description of this range, suitable for composing
     *         error messages. For instance, <em>between 1 and 10</em>
     *         or <em>at most 5</em>. 
     */
    public String desc() {
        
        StringBuffer sb = new StringBuffer();
        
        if( lower == upper )
            return Integer.toString( lower );
        
        if( lower == absLower && upper == absUpper )
            return "any number";
        
        if( upper == absUpper ) 
            sb.append( "at least " ).append( lower );
        else if( lower == absLower ) 
            sb.append( "at most " ).append( upper );
        else if( lower == upper -1 )
            sb.append( lower )
              .append( " or " )
              .append( upper );
        else
            sb.append( "between " )
              .append( lower )
              .append( " and " )
              .append( upper );
       
        return sb.toString();
    }
}
