package org.eso.ohs.dfs;

import java.io.IOException;
import java.io.Serializable;

import org.eso.ohs.core.utilities.AssocList;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.ReadmeAttribute;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * 
 * Class Readme
 * 
 * Readme file. This contains the standard template that the user fills out the
 * following data for the usg support astronomer (mts abstact &&
 * specialComments) TODO Javadoc
 * 
 * @version $Revision: 1.36 $
 * @author tcanavan, Oct 27, 2004
 */

// $Id: Readme.java,v 1.36 2005/02/10 09:40:36 tcanavan Exp $
public class Readme implements Serializable {

    private static final long serialVersionUID = -4378250031958131400L;

    public final static String DEFINED_STATUS = "D";

    public final static String CHECKEDOUT_STATUS = "P";

    public final static String ACCEPTED_STATUS = "+";

    public final static String REJECTED_STATUS = "-";
    
    private String status;
    
    
    public static final AssocList statusValues_ =
        new AssocList()
            .add("(-)Rejected", REJECTED_STATUS)
            .add("(+)Accepted", ACCEPTED_STATUS);

    
    private int version;
    

    private boolean checkedIn = false;
    
    private float ipVersion_;
    
    
    private ReadmeHistory [] readmeHistoryItems;
    
    // only ever initialised from the database. 
    
    private transient USDReadmeDatails usdReadmeFileDetails;
    
    // only ever iniitalised from the DB
    
    private transient CommentDetails [] commentArray;
    
    
    public Readme(float ipVersion) {
        ipVersion_ = ipVersion;
    }

    /**
     * The readme attributes associated with a given readme file.
     *  
     */

    private ReadmeAttribute[] readmeAttributes_;

    /**
     * Return the list of readme attributes that have beeb set.
     * 
     * @return Returns the redmeAttributes.
     */
    public ReadmeAttribute[] getRedmeAttributes() {
        if (readmeAttributes_ == null)
            return new ReadmeAttribute[0];
        return readmeAttributes_;

    }

    /**
     * Set the list of readme attributes.
     * 
     * @param redmeAttributes
     *            The redmeAttributes to set.
     */
    public void setRedmeAttributes(ReadmeAttribute[] redmeAttributes) {
        readmeAttributes_ = redmeAttributes;
    }


    /**
     * Return an XML representation of the readme defintion.
     * 
     * @return An XML presentation of the readme file.
     */

    public String toAttributesXMLString() {
        Element root = new Element("Readme");
        Document doc = new Document(root); 
        doc.setRootElement(root);
        for (int i = 0; i < readmeAttributes_.length; i++) {
            Element element = new Element(readmeAttributes_[i].getName());
            Element labelElement = new Element("Label");
            labelElement.setText(readmeAttributes_[i].getReadmeConfig().getLabel());
            Element dataElement = new Element("Data");
            dataElement.setText(readmeAttributes_[i].getValue());
            element.addContent(0,labelElement);
            element.addContent(1,dataElement);
            root.addContent(element);
        }
        XMLOutputter outputter = new XMLOutputter();
        Format format = Format.getPrettyFormat();
        outputter.setFormat(format);
        return outputter.outputString(doc);
    }

    /**
     * Read varianle ob details from local storage
     * 
     * @param in
     *            Stream to capture data from cache
     * @exception IOException
     *                IO Exception if record cannot be read from disk
     * @exception ClassNotFoundException
     *                class not found exception
     */

    

    public String verifyReport() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < readmeAttributes_.length; i++) {
            String verify = readmeAttributes_[i].verify();
            if (verify.equals(ReadmeAttribute.VERIFY_EMPTY)) {
                buf.append("ERROR Not Defined:- "
                        + readmeAttributes_[i].getReadmeConfig().getLabel()
                        + "\n");
            }
            if (verify.equals(ReadmeAttribute.VERIFY_INVALID)) {
                buf.append("ERROR Invalid Data:- "
                        + readmeAttributes_[i].getReadmeConfig().getLabel()
                        + "\n");
            }
        }
        return buf.toString();
    }

    /**
     * @return Returns the checkedIn.
     */
    public boolean isCheckedIn() {
        return checkedIn;
    }
    /**
     * @param checkedIn The checkedIn to set.
     */
    public void setCheckedIn(boolean checkedIn) {
        this.checkedIn = checkedIn;
    }
    
    /**
     * Get the ipVersion.
     * @return The ipVersion.
     */
    public float getIpVersion() {
        return ipVersion_;
    }

    /**
     * @param f
     */
    public void setIpVersion(float f) {
        ipVersion_ = f;
    }

    /**
     * @return Returns the readmeHistoryItems.
     */
    public ReadmeHistory[] getReadmeHistoryItems() {
        if(readmeHistoryItems == null)
            return new ReadmeHistory[0];
        return readmeHistoryItems;
    }
    /**
     * @param readmeHistoryItems The readmeHistoryItems to set.
     */
    public void setReadmeHistoryItems(ReadmeHistory[] readmeHistoryItems) {
        this.readmeHistoryItems = readmeHistoryItems;
    }
    /**
     * The status value
     * @return The status value.
     */
    public String getStatus() {
        return status;
    }

    /**
     * @param string
     */
    public void setStatus(String string) {
        status = string;
    }

    /**
     * Get Readme version.
     * @return Get Readme version.
     */
    public int getVersion() {
        return version;
    }

    /**
     * @param i
     */
    public void setVersion(int i) {
        version = i;
    }

    /**
     * @return Returns the usdReadmeFileDetails.
     */
    public USDReadmeDatails getUsdReadmeFileDetails() {
        return usdReadmeFileDetails;
    }
    /**
     * @param usdReadmeFileDetails The usdReadmeFileDetails to set.
     */
    public void setUsdReadmeFileDetails(USDReadmeDatails usdReadmeFileDetails) {
        this.usdReadmeFileDetails = usdReadmeFileDetails;
    }
    /**
     * @return Returns the commentArray.
     */
    public CommentDetails[] getCommentArray() {
        return commentArray;
    }
    /**
     * @param commentArray The commentArray to set.
     */
    public void setCommentArray(CommentDetails[] commentArray) {
        this.commentArray = commentArray;
    }

    public static Readme createReadme(Instrument inst) {
        Readme readme = new Readme(inst.getVersionNumber()); 
        readme.setRedmeAttributes(ReadmeAttributesFactory.createReadmeAttributes(inst));
        return readme;
    }
}