package org.eso.ohs.dfs.docviewdatatrans;

import org.eso.ohs.core.docview.datatrans.ObjAdaptor;
import org.eso.ohs.core.utilities.Convert;


/**
 * A transducer to convert set/get of String values into set/get
 * on a long integer property.  The target property's name is given
 * to the constructor.
 *
 * 
 * $Id: ISODateAdaptor.java,v 1.2 2004/02/17 14:31:57 tcanavan Exp $
 */

public class ISODateAdaptor extends ObjAdaptor {


    /**
     * Construct a transducer.
     */
    public ISODateAdaptor() {
	super(long.class);
    }


    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public ISODateAdaptor(Object obj, String fieldName) {
	super(long.class,obj,fieldName);
    }


    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Long
     * representing a long int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
        if(value == null)
	    return new Object();
        try{ 
	    return new Long( Convert.ISOdateToLong( value ));
        }catch (Exception e){
	    return new Object();
        }
    }


    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {

	return Convert.longToISODate( ((Long)obj).longValue() );
    }

}

