package org.eso.ohs.instruments;

import java.util.StringTokenizer;

import org.apache.log4j.Logger;

/**
 * Instancaites a paramater class that represents a pixel. 
 * The string that contains the pixel values is checked for its validity
 * <p>
 * $Id: PixelParameter.java,v 1.9 2004/04/08 12:09:52 tcanavan Exp $
 * @author Tim Canavan  
 */

public class PixelParameter extends Parameter {

    static final long serialVersionUID = -3146141549905045995L;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( PixelParameter.class );


   
   /**
    * Constructo for Pixel Parameter
    */
    public PixelParameter(ParamMetaData pMeta) {
        super(pMeta);
    }

    /**
     * Get tooltip text for Pixel parameter usinf the range value
     * if supplied
     * @return tooltip text
     */
    
    public String getToolTipText() {
        String range = getRange();
        if(range != null || range.length() > 0)
            return "Enter two integer values within range " + getRange();
        else
            return "Enter two integer values";
    }


    public char getDBTypeCode() {
        return 'P';
    }

    /**
     * This method verifies that a string is a particular pixel
     * @param value String repesenting a particular pixel
     * @return OK,NaN,Number of items,Range of allows values.
     */

    public boolean verifyValue(String value) {

        Integer xmin;Integer ymin;
        Integer xmax;Integer ymax;
        Integer xpixel;Integer ypixel;

        if(value == null || value.equals("NODEFAULT") || value.length() <= 0)
            return false;
        StringTokenizer valueToken = new StringTokenizer(value," ");
        Integer countTokens = new Integer(valueToken.countTokens());
        if(countTokens.intValue() != 2)
            return false;
        String range = getRange();
        if(value.equals(range))
            return true;
        StringTokenizer rangeToken = new StringTokenizer(range);
        Integer countRangeTokens = new Integer(rangeToken.countTokens());
        if(countRangeTokens.intValue()!=4) {
            return false;
        }
        try {
            xmin = new Integer(rangeToken.nextToken());
            ymin = new Integer(rangeToken.nextToken());
            xmax = new Integer(rangeToken.nextToken());
            ymax = new Integer(rangeToken.nextToken());
            xpixel = new Integer(valueToken.nextToken());
            ypixel = new Integer(valueToken.nextToken());
	}
	catch(NumberFormatException e) {
            stdlog_.error("Invalid coordinates " + e.getMessage());
            return false;
	}

        if (xpixel.intValue() < xmin.intValue() || ypixel.intValue() < ymin.intValue()  ||
            xpixel.intValue() > xmax.intValue() || ypixel.intValue() > ymax.intValue() )
	    return false;
        return true;
    }

   


    
}

