package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for Schedules stored in the database.
 * 
 * @version $Revision: 1.2 $
 * @author Giorgio Passera <ESO\DMD> <gpassera@eso.org>
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerQueue extends DbaseIOHandler {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(DbaseHandlerQueue.class);

    private static final String rcsid =
        "$Id: DbaseHandlerQueue.java,v 1.2 2005/09/05 15:09:08 ddorigo Exp $";

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());
    public static String TMP_SCHEDULE_ITEMS = "#tmp_schedule_items";
    public static String SCHEDULE_ITEMS = "schedule_items";
    public static String SCHEDULES = "schedules";

    /**
     * Refers to the appropriate DFS class
     * @return Queue class
     */

    public Class getManagedClass() {
        return Queue.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * @exception ObjectIOException IO Eror occured while reading from the database
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException {
        ResultSet rs = null;
        Queue aQueue = (Queue) newInstance();
        UnpackedStorableObject obj = new UnpackedStorableObject(aQueue);

        String sql =
            "SELECT * FROM "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + " WHERE "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".sch_id = "
                + id;
        /*
        if ( ! Config.getCfg().isOperationsModeId(userId_) ) {
        sql = sql + " AND " + obrepDbName + "..schedules.user_id=" + userId_;
        }
        */

        stdlog_.info("sql = " + sql);

        rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        aQueue.setId(
            Config.getCfg().tableToUniqueId(rs.getLong("sch_id"), Queue.class));

        //aQueue.setId(rs.getLong("sch_id"));

        aQueue.setOwnerId(rs.getInt("user_id"));

        aQueue.setName(rs.getString("item_name"));
        aQueue.setDescription(rs.getString("item_description").trim());
        aQueue.setStartTime(rs.getInt("item_start"));
        aQueue.setDuration(rs.getInt("item_duration"));
        aQueue.setTelescope(rs.getString("tel_code"));
        rs.close();
        QueueItem[] queueItems = QueueItemDBIO.read(stmt, id);
        aQueue.setQueueList(queueItems);
        aQueue.setSize(queueItems.length);
        TmpDBIO.deleteQueueReferenceData(
            aQueue.getId(),
            TMP_SCHEDULE_ITEMS,
            stmt);
        TmpDBIO.copyQueueReferenceData(
            aQueue.getId(),
            obrepDbName_ + ".." + SCHEDULE_ITEMS,
            TMP_SCHEDULE_ITEMS,
            stmt);
        return obj;
    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
        throws SQLException {
        // stdlog_.debug("write");
        Queue aQueue = (Queue) obj.getObject();
        StringBuffer sb = new StringBuffer();

        sb.append(id);
        sb.append(",");
        sb.append(aQueue.getOwnerId());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(aQueue.getName()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(aQueue.getDescription()));
        sb.append(",");
        sb.append(aQueue.getStartTime());
        sb.append(",");
        sb.append(aQueue.getDuration());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(aQueue.getTelescope()));
        //sb.append("'");

        String sqlText =
            "INSERT INTO "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + " ( "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".sch_id, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".user_id, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".item_name, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".item_description, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".item_start, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".item_duration, "
                + obrepDbName_
                + ".."
                + SCHEDULES
                + ".tel_code)"
                + " VALUES ("
                + sb
                + ")";

        stdlog_.debug("sql = " + sqlText);
        stmt.executeUpdate(sqlText);
        QueueItemDBIO.write(stmt, id, aQueue.getQueueList());
        TmpDBIO.deleteQueueReferenceData(
                aQueue.getId(),
                TMP_SCHEDULE_ITEMS,
                stmt);
            TmpDBIO.copyQueueReferenceData(
                aQueue.getId(),
                obrepDbName_ + ".." + SCHEDULE_ITEMS,
                TMP_SCHEDULE_ITEMS,
                stmt);

    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
        throws SQLException {

        // stdlog_.debug("update");
        delete(stmt, id);
        write(stmt, id, obj);
    }

    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id) throws SQLException {

        //	QueueItemDBIO.delete(stmt,id,TMP_SCHEDULE_ITEMS);
        QueueItemDBIO.delete(stmt, id, obrepDbName_ + ".." + SCHEDULE_ITEMS);
        String sqlString2 =
            "DELETE FROM "
                + obrepDbName_
                + ".."
                + DbaseHandlerQueue.SCHEDULES
                + " WHERE "
                + obrepDbName_
                + ".."
                + DbaseHandlerQueue.SCHEDULES
                + ".sch_id = "
                + id;

        stdlog_.info("sql = " + sqlString2);
        stmt.executeUpdate(sqlString2);
    }

}
