package org.eso.ohs.phase2.apps.masktracker.actions;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.FilePreviewDialog;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.apps.masktracker.MaskDiscarding;
import org.eso.ohs.phase2.apps.masktracker.MaskInsertion;
import org.eso.ohs.phase2.apps.masktracker.MaskManufacturing;

/**
 * Action class to create a business object in the chosen directory.
 * 
 * <p>
 * $Id: ProcessReportAction.java,v 1.2 2005/02/11 11:00:29 tcanavan Exp $
 */

public class ProcessReportAction extends AbstractAction {

    private static FilePreviewDialog filePreviewDialog_ = null;

    private static SwingFileChooser chooser_ = null;


    private static ExtensionFileFilter mmrFilter = new ExtensionFileFilter("mmr","Manufacturing Reports");
    private static ExtensionFileFilter mirFilter = new ExtensionFileFilter("mir","Insertion Reports");
    private static ExtensionFileFilter mdrFilter = new ExtensionFileFilter("mdr","Discarding Reports");
  
  
    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */

    public ProcessReportAction(JFrame f,String name) {
   	super(name);
        if(filePreviewDialog_ == null)
            filePreviewDialog_ = new FilePreviewDialog(f,"Select Report Files",true);
        chooser_ = filePreviewDialog_.getFileChooser();
        File f1 = new File(AppConfig.getAppConfig().getPafDirectory());
        chooser_.setCurrentDirectory(f1);
        chooser_.setSelectedFile(f1);
        filePreviewDialog_.addChoosableFileFilter(mmrFilter);
        filePreviewDialog_.addChoosableFileFilter(mirFilter);
        filePreviewDialog_.addChoosableFileFilter(mdrFilter);
    }

 
    /**
     * Event handler for processing a mask report
     * 
     * @param ev       user-initiated ActionEvent to create an OD.
     */

    public void actionPerformed(ActionEvent ev) {
        // Java1.4 port chooser_.clearSelection();
        chooser_.cancelSelection();
        chooser_.setMultiSelectionEnabled(false);
        filePreviewDialog_.setVisible(true);
        int response = filePreviewDialog_.getReturnValue();
        if(response == JOptionPane.CLOSED_OPTION || response == JOptionPane.CANCEL_OPTION)
            return;
        String value = filePreviewDialog_.getValue();
        String fileName = filePreviewDialog_.getFileName();
        if(fileName == null)
            return;
        if(fileName.indexOf(".mmr")!=-1) {
            MaskManufacturing.getInstance().receiveReport(value);
	}
        if(fileName.indexOf(".mir")!=-1) {
            MaskInsertion.getInstance().receiveReport(value);
	}
        if(fileName.indexOf(".mdr")!=-1) {
            MaskDiscarding.getInstance().receiveReport(value);
	}
    }          
}






