package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectEvent;
import org.eso.ohs.persistence.ObjectListener;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectTableEvent;
import org.eso.ohs.phase2.apps.ot.ObjectTableListener;
import org.eso.ohs.phase2.apps.ot.actions.CloseQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.DeleteQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.DumpQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.NewQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.OBHistoryAction;
import org.eso.ohs.phase2.apps.ot.actions.OpenQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.QuitAction;
import org.eso.ohs.phase2.apps.ot.actions.RefreshQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SaveQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ShowDbaseAction;

/**
 * This class provides the main view for the OT application.
 * 
 * @version $Revision: 1.34 $
 * 
 * @author Gorgio Passera <ESO\DMD><gpassera@eso.org><br>
 *         ChangeLog: cmavros 19/02/02, 1640/OT, Add Refresh Menu option
 */

public class OTMainView extends MenuGroups implements QueueContainer,
        ObjectTableListener {

    private static Logger stdlog_ = Logger.getLogger(OTMainView.class);

    private final static String rcsid = "$Id: OTMainView.java,v 1.34 2005/11/28 12:05:17 tcanavan Exp $";

    public static int TAB_PANE_QUEUE_ID = 0;

    public static int TAB_PANE_OPEN_QUEUE_ID = 1;

    public static int TAB_PANE_EXEC_ID = 2;

    /** A table listing the queues that are in the database */
    protected JTable queueTable_;

    /** A list of open queues in the database */
    protected JTable openQueueTable_;

    /** Model for list of queues in the database */
    protected QueueSummaryModel queueModel_;

    /** Model for list of open queues */
    protected QueueSummaryModel openQueueModel_;

    /** A listener for updates to the Persistence's registry of objects */
    protected QueueObjectListener listener_;

    /** support for handling an object events */
    private transient EventListenerList objectTableListeners_ = new EventListenerList();

    protected JTabbedPane tabPane_ = new JTabbedPane();

    protected JToolBar queueToolBar = new JToolBar();

    protected JToolBar openQueueToolBar = new JToolBar();

    protected JPanel tablePanel_;

    protected Action OBHistoryAction;


    /**
     * Constructor. <br>
     * ChangeLog: A M Chavan, 12-Jun-2001, minor changes to title bar.
     */
    public OTMainView() {
        super("Observing Tool V." + OHSApplication.getVersion() + " User="
                + Config.getCfg().getUserName() + " Server="
                + AppConfig.getAppConfig().getDbServer());

        ObjectManager objMgr = ObjectManager.getObjectManager();
        OTViewManager otViewManager = OTViewManager.getOTViewMgr();
        otViewManager.setOTMainView(this);
        createObjectListeners();
        buildTables();
        createActions();
        buildQueueToolBar();
        buildOpenQueueToolBar();
        buildTabPanes();
        addObjectListeners();
        addTableListeners();
        getContentPane().add(tabPane_);
        setEnabledQueueList(false);
        setEnabledOpenQueueList(false);
        if (queueTable_.getRowCount() > 0)
            queueTable_.setRowSelectionInterval(0, 0);
        buildMenu();
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                dispose();
            }
        });
        //TODO to be reviewed
        ExecutionSequenceBView.getInstance().setDisplayTextAction(this);
        pack();
        setSize(new Dimension(710, 650));
        setVisible(true);

    }

    private void createActions() {
        openQueueAction = new OpenQueueAction(this, "Open");
        deleteQueueAction = new DeleteQueueAction(this, queueTable_, "Delete");
        newQueueAction = new NewQueueAction(this, "New...");
        newQueueAction.setEnabled(true);
        dumpQueueAction = new DumpQueueAction(this, "Dump...");
        saveQueueAction = new SaveQueueAction(this, "Save");
        reloadQueueAction = new ReloadQueueAction(this, "Revert");
        refreshQueueAction = new RefreshQueueAction(this, queueTable_,
                "Refresh");
        refreshQueueAction.setEnabled(AppConfig.getAppConfig().getBoolean(
                OHSKeys.OTRefreshQueueKey));
        closeQueueAction = new CloseQueueAction(this, "Close");
        OBHistoryAction = new OBHistoryAction("OB History");
    }

    protected void buildTabPanes() {
        tablePanel_ = new JPanel(new BorderLayout());
        JScrollPane scrollPane = new JScrollPane(queueTable_);
        tablePanel_.add(scrollPane, BorderLayout.CENTER);
        tablePanel_.add(queueToolBar, BorderLayout.SOUTH);
        tabPane_.addTab("All Queues", tablePanel_);
        scrollPane = new JScrollPane(openQueueTable_);
        tablePanel_ = new JPanel(new BorderLayout());
        tablePanel_.add(scrollPane, BorderLayout.CENTER);
        tablePanel_.add(openQueueToolBar, BorderLayout.SOUTH);
        tabPane_.add("Open Queues", tablePanel_);
        tabPane_
                .add("Execution Sequence", ExecutionSequenceBView.getInstance());
        tabPane_.addChangeListener(new TabChangeListener());
    }

    private void buildQueueToolBar() {
        queueToolBar.setFloatable(false);
        JButton btn = queueToolBar.add(openQueueAction);
        btn.setText("Open");
        btn = queueToolBar.add(deleteQueueAction);
        btn.setText("Delete");
        btn = queueToolBar.add(newQueueAction);
        btn.setText("New");
    }

    private void buildOpenQueueToolBar() {

        openQueueToolBar.setFloatable(false);
        JButton btn = openQueueToolBar.add(saveQueueAction);
        btn.setText("Save");
        btn = openQueueToolBar.add(closeQueueAction);
        btn.setText("Close");
        btn = openQueueToolBar.add(reloadQueueAction);
        btn.setText("Revert");
    }

    private void listSummaryObjects() {
        ObjectManager objMgr = ObjectManager.getObjectManager();
        try {
            Summary[] summaryInformation = objMgr
                    .listObjects(Media.DBASE, null);
            queueModel_ = new QueueSummaryModel(summaryInformation);

            openQueueModel_ = new QueueSummaryModel(new Summary[0]);
        } catch (ObjectIOException e1) {
            ErrorMessages.announceIOError(null, e1);
        }

    }

    private void buildTables() {
        listSummaryObjects();
        JPanel tablePanel = new JPanel(new BorderLayout());
        queueTable_ = new JTable(queueModel_);
        Dimension viewportSize = new Dimension(450, 300);
        queueTable_.setPreferredScrollableViewportSize(viewportSize);
        JScrollPane scrollPane = new JScrollPane(queueTable_);
        tablePanel.add(scrollPane, BorderLayout.CENTER);
        openQueueTable_ = new JTable(openQueueModel_);
        openQueueTable_.setPreferredScrollableViewportSize(viewportSize);
        queueTable_.setAutoCreateColumnsFromModel(false);
        openQueueTable_.setAutoCreateColumnsFromModel(false);
        queueTable_
                .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        stdlog_.debug("Building Open Queue Tables "
                + openQueueTable_.getColumnCount());

    }

    public void createObjectListeners() {
        listener_ = new QueueObjectListener();
    }

    public void addTableListeners() {
        addListSelectionListener(new QueueListListener(), queueTable_);
        addListSelectionListener(new OpenQueueListListener(), openQueueTable_);
    }

    public void addObjectListeners() {
        ObjectManager objMgr = ObjectManager.getObjectManager();
        addObjectTableListener(this);
        addObjectTableListener(queueModel_);
        addObjectTableListener(openQueueModel_);
        objMgr.addObjectListener(listener_);
    }

    public void dispose() {
        OTViewManager.getOTViewMgr().askAndQuit();
    }

    public long getQueueId(int row) {
        Integer id = (Integer) queueTable_.getModel().getValueAt(row, 0);
        return id.intValue();
    }

    public int getRowForQueueId(long queueId) {
        return openQueueModel_.getRowFromQueueId(queueId);
    }

    public JTable getQueueTable() {
        return queueTable_;
    }

    public JTable getOpenQueueTable() {
        return openQueueTable_;
    }

    public JFrame getParentFrame() {
        return this;
    }

    public long[] getSelectedOpenQueueIds() {
        int[] rows = openQueueTable_.getSelectedRows();
        long[] queueIds = new long[rows.length];
        QueueSummaryModel model = (QueueSummaryModel) openQueueTable_
                .getModel();
        for (int i = 0; i < rows.length; i++) {
            queueIds[i] = model.getQueueId(rows[i]);
        }
        return queueIds;
    }

    /**
     * Add a selection listener to the list of listeners to be notified when
     * selecting table rows.
     * 
     * @param listener
     *            List selection listener
     */
    public void addListSelectionListener(ListSelectionListener listener,
            JTable table) {
        ListSelectionModel sel_model = table.getSelectionModel();
        sel_model.addListSelectionListener(listener);
    }

    /**
     * Adds a new listener to be notified of created/read/deleted objects
     * 
     * @param lnr
     *            the listener
     */
    public void addObjectTableListener(ObjectTableListener lnr) {
        objectTableListeners_.add(ObjectTableListener.class, lnr);
    }

    /**
     * Removes a listener, which will no longer be notified of changes.
     * 
     * @param lnr
     *            the listener
     */
    public void removeObjectTableListener(ObjectTableListener lnr) {
        objectTableListeners_.remove(ObjectTableListener.class, lnr);
    }

    /**
     * Inform listeners of a change to the StorableObjects in this Folder.
     */
    public void fireObjectTableEvent(Summary summ, int eventType) {
        SummaryTableEvent objEvent = new SummaryTableEvent(summ, eventType);
        Object[] listeners = objectTableListeners_.getListenerList();

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            Object lnr = listeners[i + 1];
            if (listeners[i] == ObjectTableListener.class) {
                if (eventType == ObjectTableEvent.OBJECT_ADDED) {
                    ((ObjectTableListener) lnr).tableObjectAdded(objEvent);
                } else if (eventType == ObjectTableEvent.OBJECT_CHANGED) {
                    ((ObjectTableListener) lnr).tableObjectChanged(objEvent);
                } else if (eventType == ObjectTableEvent.OBJECT_REMOVED) {
                    ((ObjectTableListener) lnr).tableObjectRemoved(objEvent);
                }
            }
        }
    }

    /**
     * XXX This is a hack to make menus work properly under HP-UX JDK1.1.8.
     * Without this kludge the first three menus are empty when using the JIT,
     * presumably because the system is somehow not notified about the added
     * elements.
     */
    public void addNotify() {
        super.addNotify();
        // XXX Kludge for HP-UX JDK 1.1.8 with JIT enabled
        setJMenuBar(getJMenuBar());
    }

    /**
     * Return the index of all selected rows.
     * 
     * @return the index of all selected rows. An empty array is returned if no
     *         column is selected
     */
    public int[] getSelectedRows() {
        return openQueueTable_.getSelectedRows();
    }

    public int getRowCount() {
        return openQueueTable_.getRowCount();
    }

    /** Builds the pull-down menus */
    private void buildMenu() {
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        buildFileMenu();
    }

    /** Builds the File Menu */
    public void buildFileMenu() {
        fileMenu = new JMenu("Queues");
        menuBar.add(fileMenu);
        newMenuItem = fileMenu.add(newQueueAction);
        openMenuItem = fileMenu.add(openQueueAction);
        reloadMenuItem = fileMenu.add(reloadQueueAction);
        refreshMenuItem = fileMenu.add(refreshQueueAction);
        closeMenuItem = fileMenu.add(closeQueueAction);
        fileMenu.addSeparator();
        saveMenuItem = fileMenu.add(saveQueueAction);
        //dumpMenuItem = fileMenu.add(dumpQueueAction);
        //JMenuItem exportMenuItem = new JMenuItem("Export...");
        //exportMenuItem.setEnabled(false);
        //fileMenu.add(exportMenuItem);
        deleteMenuItem = fileMenu.add(deleteQueueAction);
        fileMenu.addSeparator();
        repositoryBrowserMenuItem = fileMenu.add(new ShowDbaseAction(this));
        fileMenu.add(OBHistoryAction);
        boolean mtEnabled = AppConfig.getAppConfig().getBoolean(
                OHSKeys.MaskTrackerEnabledKey);
        //fileMenu.addSeparator();
        quitMenuItem = fileMenu.add(new QuitAction(this));
        fileMenu.add(repositoryBrowserMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(quitMenuItem);
        newMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                ActionEvent.CTRL_MASK));
        newMenuItem.setMnemonic(KeyEvent.VK_N + KeyEvent.CTRL_MASK);

        openMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                ActionEvent.CTRL_MASK));
        openMenuItem.setMnemonic(KeyEvent.VK_O + KeyEvent.CTRL_MASK);

        repositoryBrowserMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_B, ActionEvent.CTRL_MASK));
        repositoryBrowserMenuItem.setMnemonic(KeyEvent.VK_B
                + KeyEvent.CTRL_MASK);


        quitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                ActionEvent.CTRL_MASK));
        quitMenuItem.setMnemonic(KeyEvent.VK_Q + KeyEvent.CTRL_MASK);

        /** cmavros 25-10-01 [1506] Add Reports to Execution Sequence */
        createReportsMenu(ExecutionSequenceBView.getInstance().getView());
        createFCPrintMenu(ExecutionSequenceBView.getInstance().getView());
        if(!mtEnabled) {
            createOrangMenu(ExecutionSequenceBView.getInstance().getView());        	
        }
        setEnabledReports(false);
        setEnabledFCPrint(false);

    }

    public Class getDisplayedObjectClass() {
        return Queue.class;
    }

    public void setEnabledQueueList(boolean enabled) {
        openQueueAction.setEnabled(enabled);
        deleteQueueAction.setEnabled(enabled);
        //newQueueAction.setEnabled(enabled);
    }

    public void setEnabledOpenQueueList(boolean enabled) {
        dumpQueueAction.setEnabled(enabled);
        saveQueueAction.setEnabled(enabled);
        reloadQueueAction.setEnabled(enabled);
        closeQueueAction.setEnabled(enabled);
    }

    public void tableObjectAdded(SummaryTableEvent ev) {
        if (openQueueTable_.getRowCount() > 0)
            openQueueTable_.setRowSelectionInterval(0, 0);
    }

    public void tableObjectRemoved(SummaryTableEvent ev) {
        if (openQueueTable_.getRowCount() > 0)
            openQueueTable_.setRowSelectionInterval(0, 0);
        if (queueTable_.getRowCount() > 0)
            queueTable_.setRowSelectionInterval(0, 0);
    }

    public void tableObjectChanged(SummaryTableEvent ev) {
    }

    protected class QueueListListener implements ListSelectionListener {

        public QueueListListener() {
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty() && selectedRow >= 0
                    && tabPane_.getSelectedIndex() == TAB_PANE_QUEUE_ID) {
                setEnabledQueueList(true);
            } else {
                setEnabledQueueList(false);
            }
            newQueueAction.setEnabled(true);
        }
    }

    public class OpenQueueListListener implements ListSelectionListener {

        public OpenQueueListListener() {
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty() && selectedRow >= 0
                    && tabPane_.getSelectedIndex() == TAB_PANE_OPEN_QUEUE_ID) {
                setEnabledOpenQueueList(true);
                long id = openQueueModel_.getQueueId(selectedRow);
                OTViewManager manager = OTViewManager.getOTViewMgr();
                QueueView queueView = manager.getQueueView(id);
                if (queueView != null) {
                    stdlog_.debug("IS ACTIVE " + isActive());
                    queueView.setVisible(true);
                    queueView.setState(JFrame.NORMAL);
                }
            } else {
                setEnabledOpenQueueList(false);
            }

        }

    }

    /**
     * Listener for new TabbedPane selections. Fires an ObjectSelectionEvent
     * when a different tabPane is selected. <br>
     * ChangeLog: cmavros 25-10-01 [1506] Add Reports to Execution Sequence
     */
    public class TabChangeListener implements ChangeListener {

        /**
         * Handler for TabbedPane selection events. Fires an
         * ObjectSelectionEvent when a different tabPane is selected.
         */
        public void stateChanged(ChangeEvent ev) {
            ListSelectionModel lsm = queueTable_.getSelectionModel();
            if (tabPane_.getSelectedIndex() == TAB_PANE_QUEUE_ID) {
                ListSelectionModel lsmQueue = queueTable_.getSelectionModel();
                setEnabledReports(false);
                setEnabledFCPrint(false);
                setEnabledOpenQueueList(false);
                if (queueTable_.getRowCount() > 0
                        && !lsmQueue.isSelectionEmpty()) {
                    setEnabledQueueList(true);
                } else {
                    setEnabledQueueList(false);
                }
                newQueueAction.setEnabled(true);
                refreshQueueAction.setEnabled(AppConfig.getAppConfig()
                        .getBoolean(OHSKeys.OTRefreshQueueKey));

            }
            if (tabPane_.getSelectedIndex() == TAB_PANE_OPEN_QUEUE_ID) {
                ListSelectionModel lsmQueueOpen = queueTable_
                        .getSelectionModel();
                setEnabledReports(false);
                setEnabledFCPrint(false);
                setEnabledQueueList(false);
                if (openQueueTable_.getRowCount() > 0
                        && !lsmQueueOpen.isSelectionEmpty()) {
                    setEnabledOpenQueueList(true);
                } else {
                    setEnabledOpenQueueList(false);
                }
                newQueueAction.setEnabled(false);
                refreshQueueAction.setEnabled(false);
            }
            if (tabPane_.getSelectedIndex() == TAB_PANE_EXEC_ID) {
                setEnabledQueueList(false);
                setEnabledReports(true);
                setEnabledFCPrint(true);
                newQueueAction.setEnabled(false);
                refreshQueueAction.setEnabled(false);
            }
        }
    }

    /** Listener for updates to the Persistence's registry of objects */
    public class QueueObjectListener implements ObjectListener {

        public QueueObjectListener() {
            // System.out.println("@@@@@@@@@@@@@ Adding listener to object
            // manager");
        }

        /**
         * Event handler: an object was added to the registry.
         * 
         * @param ev
         *            event describing what object was added.
         */
        public void objectAdded(ObjectEvent ev) {
            if (ev != null) {
                if (ev.getObjectClass().equals(Queue.class)
                        || ev.getObjectClass().equals(MaskQueue.class)) {
                    Queue sobj = (Queue) ev.getStorableObject();
                    Summary summ = new Summary();
                    summ.setPropertyValue("QueueId", new Long(sobj.getId()));
                    summ.setPropertyValue("Name", sobj.getName());
                    summ.setPropertyValue("Size", new Integer(sobj.getSize()));
                    summ.setObjectClass(ev.getObjectClass());
                    fireObjectTableEvent(summ, SummaryTableEvent.OBJECT_ADDED);
                }
            }
        }

        /**
         * Event handler: an object was moved between two folders.
         * 
         * @param ev
         *            event describing what object was removed.
         */
        public void objectMoved(ObjectEvent ev) {

        }

        /**
         * Event handler: an object was removed from the registry.
         * 
         * @param ev
         *            event describing what object was removed.
         */
        public void objectRemoved(ObjectEvent ev) {
            if (ev != null) {
                if (ev.getObjectClass().equals(Queue.class)
                        || ev.getObjectClass().equals(MaskQueue.class)) {
                    Summary summ = new Summary();
                    summ.setPropertyValue("QueueId", new Long(ev
                            .getStorableObject().getId()));
                    fireObjectTableEvent(summ, SummaryTableEvent.OBJECT_REMOVED);
                }
            }
        }

        public void objectRemovedFromRegistry(ObjectEvent ev) {
            if (ev != null) {

                if (ev.getObjectClass().equals(Queue.class)
                        || ev.getObjectClass().equals(MaskQueue.class)) {
                    Queue sobj = (Queue) ev.getStorableObject();
                    int openrow = openQueueModel_.getRowFromQueueId(sobj
                            .getId());
                    int queuerow = queueModel_.getRowFromQueueId(sobj.getId());
                    if (sobj.isNew()) {
                        queueModel_.removeElementAt(queuerow);
                    }
                    openQueueModel_.removeElementAt(openrow);
                    queueModel_.fireTableDataChanged();

                }
            }
        }
    }
}