package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Category;
import org.eso.ohs.core.dbb.client.DbbTable;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerQueue;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.ot.actions.DeselectAllAction;
import org.eso.ohs.phase2.apps.ot.actions.DisplayOBAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveDownRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveUpRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.OBHistoryAction;
import org.eso.ohs.phase2.apps.ot.actions.RemoveAllQueueItemsAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
/**
 * The QueueItemsBView class is a DB search engine used for 
 * displaying queues of OB  according to a user-definable set of 
 * properties 
 *
 * @author Gorgio Passera DMD/ESO <gpassera@eso.org>
 * <br>ChangeLog: cmavros 23May02 [1643/OT] Remove default selection items
 *     so user can customize.  
 * <br>ChangeLog: cmavros 23May02 Add log4j  
 */
public abstract class OTQueueBrowserView extends JPanel implements OBListener {



    private static Category stdlog_ =
        Category.getInstance(OTQueueBrowserView.class);
                    
       

    protected boolean _orderDirtyFlag = false;
    protected int currentRow_;
    protected DbbTextField queueIDTF;
    protected String tmpTable;
    protected String stdTable;
    protected Queue queue;

    protected JButton deselectAllButton;
    protected JButton displayButton;
    protected JButton removeButton;
    protected JButton markButton;
    protected JButton markReadmeButton;
    protected JButton commentButton;
    protected JButton reloadButton;

    protected JButton verifyButton;
    protected JButton appendToExSequenceButton;
    protected JButton moveUpButton;
    protected JButton moveDownButton;
    protected JButton copyButton;
    protected JButton pasteButton;
    protected JButton removeAllButton;
    protected JButton obdFormatButton;
    protected JToolBar toolBar_;

    protected Action deselectAllAction;
    protected Action moveUpAction;
    protected Action moveDownAction;
    protected Action displayAction;
    protected Action removeAllQueueItemsAction;
    
    protected Action obHistoryAction;
    protected Action findingChartAction;


    protected OBTreeView tree_;


    //private static DbaseWindow dbWdw_;
    protected DbbSqlEngine engine_;
    protected OTDbbView view_;

    // obrep2 tables
    protected DbbSqlTable ob_table;
    protected DbbSqlTable od_table;
    protected DbbSqlTable ts_table;
    protected DbbSqlTable tp_table;
    protected DbbSqlTable pa_table;
    protected DbbSqlTable ti_table;
    protected DbbSqlTable cs_table;
    protected DbbSqlTable sr_table;
    protected DbbSqlTable pm_table;
    protected DbbSqlTable sc_table;
    protected DbbSqlTable si_table;
    protected DbbSqlTable es_table;
    protected DbbSqlTable oes_table;
    protected DbbSqlTable masks_table;
    protected DbbSqlTable finds_view;
    protected DbbSqlTable st_tis_table;
    protected DbbSqlTable obsblk_masks_table;
    protected DbbSqlTable readme;    
    protected DbbSqlTable baselines;
    protected DbbSqlTable ephemeris_file;




    // phase2 tables
    //----------------------------------------

    /* <br>ChangeLog: amchavan, 24-Oct-2001: added 
     * <br>ChangeLog: amchavan, 21-Nov-2001: commented out again, see 0704
     *     DbbSqlTable run_states_;
     */

    // opc63 tables
    //----------------------------------------
    protected DbbSqlTable eu_table;

    protected String title_;

    protected TableListSelectionListener listListener_;
    // Listener to clear the tree view when the selection changes
    // Class is defined as a private class in this class.
    protected TreeViewClearer treeViewClearer_;
    
    
    
    /**
     * Used for the Move to top,moveUp and mobe down actions.
     */
    
    private long [] selOBIDs_;
    
    
    
    /**
     * Used for the Move to top,moveUp and mobe down actions.
     */


    
    private long destOBID_;
    

    /**
    * Default constructor
    * <br>ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
    *
    */
    protected void initBaseClass(
        Queue newQueue,
        String title,
        boolean clearButton,
        boolean queryButton,
        boolean sortButton,
        OBTreeView tree
        ) {
        try {
        	initView(newQueue, title, clearButton, queryButton, sortButton, tree);
            buildColumns();
            buildRows();
            buildSort();
            //layoutBrowser();        
        } catch (RemoteException e) {   MsgManager.errDialog(e);
        } catch (MalformedURLException e) {   MsgManager.errDialog(e);
        } catch (NotBoundException e) {   MsgManager.errDialog(e);
        } catch (SQLException e) {   MsgManager.errDialog(e);
        }   
    }
    /**
     * Default constructor
     * <br>ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
     *
     */
     public OTQueueBrowserView() {
     }

    /**
     * @param newQueue
     * @param title
     * @param clearButton
     * @param queryButton
     * @param sortButton
     * @param tree
     * @param OB_ID_COLUMN
     * @param OB_TYPE_COLUMN
     * @param OB_STATUS_COLUMN
     * @throws SQLException void
     */
    protected void initView(Queue newQueue, String title, boolean clearButton, boolean queryButton, boolean sortButton, OBTreeView tree) throws SQLException {
     
        engine_ = BrowserConfig.getEngine();
        queue = newQueue;
        tree_ = tree;
        title_ = title;

        buildDefaults();

        view_ =
            new OTDbbView(
                engine_,
                clearButton,
                queryButton,
                sortButton);
        listListener_ = new TableListSelectionListener();
        view_.addListSelectionListener(listListener_);
        treeViewClearer_ = new TreeViewClearer();
        view_.addListSelectionListener(treeViewClearer_);
        view_.getTable().removeSortListener();
        QueueManager.addOBListener(this);
    }
    
    public String [] getTypes(long [] obIds) {
        String [] types = new String[obIds.length];
        for(int i=0;i<obIds.length;i++) {
            Class c = Config.getCfg().getClassFromId(obIds[i]);
            if(c.equals(ObservationBlock.class)) {
                types[i] = new String(CalibrationBlock.OBSBLK_TYPE);
            } else {
                types[i] = new String(CalibrationBlock.CALBLK_TYPE);
            }
        }
        return types;
    }


    protected abstract void addDefaultColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException; 

    /**Builds the toolBar*/
    protected void createActions() {

        deselectAllAction = new DeselectAllAction(view_);
        //deselectAllButton.addActionListener(deselectAllAction);

        moveUpAction = new MoveUpRowsAction(this);
        //moveUpButton.addActionListener(moveUpAction);

        moveDownAction = new MoveDownRowsAction(this);

        removeAllQueueItemsAction = new RemoveAllQueueItemsAction(this);

        displayAction = new DisplayOBAction(this, tree_);
        
        
        obHistoryAction = new OBHistoryAction(view_);
        findingChartAction = new ShowFindingChartsAction(Media.DBASE,view_,false);

        //moveDownButton.addActionListener(moveDownAction);                
    }

    protected void removeListeners() {
        view_.removeListSelectionListener(listListener_);
        view_.removeListSelectionListener(treeViewClearer_);
        QueueManager.removeOBListener(this);
    }

    protected void buildDefaults() {
        ob_table = BrowserConfig.ob;
        od_table = BrowserConfig.od;
        ts_table = BrowserConfig.ts;
        tp_table = BrowserConfig.tp;
        pa_table = BrowserConfig.pa;
        ti_table = BrowserConfig.ti;
        cs_table = BrowserConfig.cs;
        pm_table = BrowserConfig.pm;
        sr_table = BrowserConfig.sr;
        sc_table = BrowserConfig.sc;
        si_table = BrowserConfig.si_tmp;
        es_table = BrowserConfig.es_tmp;
        eu_table = BrowserConfig.eu;
        ephemeris_file = BrowserConfig.ephemeris_file;
        if(!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            masks_table = BrowserConfig.masks_view;
            obsblk_masks_table = BrowserConfig.masks_view;
        } else {
            masks_table = BrowserConfig.masks;
            obsblk_masks_table = BrowserConfig.obs_masks;
        }
        oes_table = BrowserConfig.oes;
        finds_view = BrowserConfig.finds_view;
        readme = BrowserConfig.readme;
        baselines = BrowserConfig.baselines;
        st_tis_table = BrowserConfig.sidereal_time_intervals_view;
        // <br>ChangeLog: amchavan, 21-Nov-2001: commented out again, see 0704
        // run_states_ = BrowserConfig.run_states_; //<br>ChangeLog: amchavan, 24-Oct-200
    }

    public void layoutBrowser() {
        JPanel tmpPanel = new JPanel(new BorderLayout());
        tmpPanel.add(toolBar_, BorderLayout.NORTH);
        tmpPanel.add(view_, BorderLayout.CENTER);
        setLayout(new BorderLayout());
        add(new JLabel(title_), BorderLayout.NORTH);
        add(tmpPanel, BorderLayout.CENTER);
    }

    public void obPropertyChange(OBEvent event) {
        try {
            long[] OBIDs = event.getOBsId();
            for (int i = 0; i < OBIDs.length; i++) {
                if (containsOB(OBIDs[i])) {
                    if (isOutOfOrder()) {
                        QueueManager.updateOBsSequences(
                            getAllOBsId(false),
                            queue.getId(),
                            DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
                        setOrderDirtyFlag(false);
                    }
// fix for 2.7.2 
                    if(!event.getValue().equals(DELETE_EVENT))
                         refreshViewsStatus(OBIDs[i], event.getValue());
                }
            }

        } catch (ObjectIOException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(
                null,
                message,
                "ERROR",
                JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        }
    }
    
    
   
    protected void refreshViewsStatus(long obId, String status) {
        int[] rows = getRowsForOBID(obId);
        for (int i = 0; i < rows.length; i++) {
            view_.setValueAt(status, rows[i],view_.STATUS);
        }
        if (rows.length > 0)
            currentRow_ = rows[0];
        setCurrentSelection();
    }
    


    public Action getDeselectAction() {
        return deselectAllAction;
    }

    public Action getMoveUpAction() {
        return moveUpAction;
    }

    public Action getMoveDownAction() {
        return moveDownAction;
    }

    public Action getRemoveAllQueueItemsAction() {
        return removeAllQueueItemsAction;
    }

    public Queue getQueue() {
        return queue;
    }

    public void setQueue(Queue queue) {
        this.queue = queue;
    }

    public void addNotify() {
        super.addNotify();
        stdlog_.debug("!!!!!!!!!!!!!!!!! Before start query");
        startQuery();
    }

    public OTDbbView getView() {
        return view_;
    }

    /**Builds the toolBar*/
    protected void buildToolBar() {
        toolBar_ = new JToolBar();
        toolBar_.setFloatable(false);

        deselectAllAction = new DeselectAllAction(view_);
        //deselectAllButton.addActionListener(deselectAllAction);

        moveUpAction = new MoveUpRowsAction(this);
        //moveUpButton.addActionListener(moveUpAction);

        moveDownAction = new MoveDownRowsAction(this);

        removeAllQueueItemsAction = new RemoveAllQueueItemsAction(this);

        displayAction = new DisplayOBAction(this, tree_);
        //moveDownButton.addActionListener(moveDownAction);                
    }

    protected Action getDisplayAction() {
        return displayAction;
    }

    /**Removes a set ob OBs from  the QueueItemsview*/
    public void removeOBs() {
        long[] selectedOBIDs = getSelectedOBIDs(false);
        removeOBs(selectedOBIDs);
    }

    protected void setCurrentSelection() {
        int rowCount = view_.getTable().getRowCount();
        if (rowCount == 0) {
            setEnabled(false);
            currentRow_ = -1;
            return;
        }
        if (currentRow_ < view_.getTable().getRowCount()) {
            view_.setSelectedRow(currentRow_);
        } else if (currentRow_ == rowCount) {
            view_.setSelectedRow((currentRow_) - 1);
        }
        JTable table = view_.getTable().getTable();
        Rectangle r = table.getCellRect(currentRow_, 1, true);
        table.scrollRectToVisible(r);
    }

    public void removeOBs(long[] selectedOBIDs) {
        try {
            QueueManager.deleteOBsFromQueueTable(
                selectedOBIDs,
                queue.getId(),
                DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
            int currentRow = view_.getSelectedRow();
            int[] rowIDs = getRowIDs(selectedOBIDs);
            removeRows(rowIDs);
            queue.removeQueueItems(selectedOBIDs);
            setOrderDirtyFlag(true);
            if (tree_ != null && tree_.getOB() != null) {
                for (int i = 0; i < selectedOBIDs.length; i++) {
                    if (tree_.getOB().getId() == selectedOBIDs[i]) {
                        tree_.updateView(null);
                        break;
                    }
                }
            }
            currentRow_ = currentRow;
            setCurrentSelection();
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(view_.getTable().getTable(), e);
        }
    }

    public void removeAllOBs() {
        long[] OBIDs = getAllOBsId(true);
        removeOBs(OBIDs);
        view_.updateDisplayedRowCount();
    }

    protected void selectAll() {
        view_.selectAllRows();
    }

    protected void clearSelection() {
        view_.clearSelection();
    }

    /**Sets the oderDirtyFlag parameter*/
    public void setOrderDirtyFlag(boolean newValue) {
        _orderDirtyFlag = newValue;
    }

    protected boolean isOutOfOrder() {
        return _orderDirtyFlag;
    }

    /**
    *Test if the View contains a specific obId
    *@param obId specifies the ID of the OB to be checked 
    *@return true if a specific obId is contained into the view
    */

    public boolean containsOB(long obId) {
        for (int i = 0; i < view_.getRows(); i++) {
            if (obId == getOBID(i))
                return true;
        }
        return false;
    }

    public int getRowID(long OBId) {
        for (int i = 0; i < view_.getRows(); i++) {
            if (OBId == getOBID(i))
                return i;
        }
        return -1;
    }

    /**
     * Given a database ob/cb id return the type 
     * @param OBId
     * @return String
     */

    private String getObType(long OBId) {
        int row = getRowID(OBId);
        return (String) view_.getValueAt(row,OTDbbView.OB_ID);
    }

    public int[] getRowsForOBID(long OBId) {
        Vector newVec = new Vector();
        for (int i = 0; i < view_.getRows(); i++) {
            if (OBId == getOBID(i)) {
                newVec.addElement(new Integer(i));
            }
        }
        int[] rows = new int[newVec.size()];
        for (int j = 0; j < rows.length; j++) {
            rows[j] = ((Integer) newVec.elementAt(j)).intValue();
        }
        return rows;
    }
    
    public int[] getRowsForRunID(long runId,String runIdColumn) {
        Vector newVec = new Vector();
         
        for (int i = 0; i < view_.getRows(); i++) {
            if (runId == getRunID(i,runIdColumn)) {
                newVec.addElement(new Integer(i));
            }
        }
        int[] rows = new int[newVec.size()];
        for (int j = 0; j < rows.length; j++) {
            rows[j] = ((Integer) newVec.elementAt(j)).intValue();
        }
        return rows;
    }


    public int[] getRowIDs(long[] OBIds) {
        Vector resultV = new Vector();
        for (int i = 0; i < OBIds.length; i++) {
            int row = getRowID(OBIds[i]);
            if (row >= 0)
                resultV.addElement(new Integer(row));
        }
        int[] returnSet = new int[resultV.size()];
        for (int j = 0; j < resultV.size(); j++) {
            Integer tmp = (Integer) resultV.elementAt(j);
            returnSet[j] = tmp.intValue();
        }
        return returnSet;
    }

    /**
    *Returns the selected rows
    *@return an array with the index of selected rows 
    *        an array of size 0 is returned if no rows 
    *        are selected
    */
    public int[] getSelectedRows() {
        return view_.getSelectedRows();
    }

    /**
    *Removes a set of rows from the view
    *@param rows specifies the row's indeces to be removed
    */
    public void removeRows(int[] rows) {
        if (rows.length < 1)
            return;
        long[] OBIDs = getOBIDs(rows, false);
        setOrderDirtyFlag(true);
        view_.removeRows(getDuplicateRows(rows));
        view_.updateDisplayedRowCount();
    }

    /**
     *Return the index of all rows that contain  the same OBID
     * of a specific row
     *@param row specified the row 
     */
    public int[] getDuplicatedRows(int row) {
        Vector v = new Vector();
        long[] OBIDs = getAllOBsId(true);
        for (int i = 0; i < OBIDs.length; i++) {
            if (getOBID(row) == OBIDs[i]) {
                v.addElement(new Integer(i));
            }
        }
        int[] returnRows = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnRows[i] = ((Integer) v.elementAt(i)).intValue();
        }
        return returnRows;
    }

    private int[] getDuplicateRows(int[] rows) {
        Vector v = new Vector();
        for (int i = 0; i < rows.length; i++) {
            int[] result = getDuplicatedRows(rows[i]);
            for (int j = 0; j < result.length; j++) {
                if (!v.contains(new Integer(result[j]))) {
                    v.addElement(new Integer(result[j]));
                }
            }
        }
        int[] returnRows = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnRows[i] = ((Integer) v.elementAt(i)).intValue();
            //stdlog_.debug("Multiple [" + i +"]" + returnRows[i]);
        }
        return returnRows;
    }
    
    /**
     * After OB's have been movedUp,toTop,etc. make the OB's 
     * that have moved the selected items.
     * @param selOBIDs
     */
    
    
    private void setInitialOBRowSelection(long [] selOBIDs) {
         DbbTable table = view_.getTable();
         table.clearSelection();
         int[] newRows = getDuplicateRows(getRowIDs(selOBIDs));
         for(int i=0;i<newRows.length;i++)
            table.getTable().addRowSelectionInterval(newRows[i],newRows[i]);
        
    }
    
    
    /**
     * Move Up rows at the level of the gui
     * @return boolean
     */

    public boolean moveUpRows() {
        selOBIDs_ = getSelectedOBIDs(false);
        int[] rows = getDuplicateRows(getRowIDs(selOBIDs_));
        int[] destRows = {
        };
        if (rows.length > 0) {
            DbbTable table = view_.getTable();
            if (rows[0] - 1 >= 0) {
                destOBID_ = getOBID(rows[0] - 1);
                destRows = getDuplicatedRows(getRowID(destOBID_));
                int[] moveRows = getDuplicateRows(rows);
                int index = (rows[0] - destRows.length);
                table.moveUpRows(moveRows, index);
                setInitialOBRowSelection(selOBIDs_);
                setOrderDirtyFlag(true);
                return true;
            }
        }
        return false;
    }
    
    
    /**
     * Move up Items in the queue and save the model.
     */
    
    public void moveUpRowsQueueItems() {
        if(moveUpRows())
             queue.moveUpQueueItems(selOBIDs_, destOBID_);
    }

    /**Moves down a set of rows
     *@param rows specifies the row's index to be moved down
     */
    public boolean moveDownRows() {
        selOBIDs_ = getSelectedOBIDs(false);
        int[] rows = getDuplicateRows(getRowIDs(selOBIDs_));
        int[] destRows = {
        };
        if (rows.length > 0) {
            DbbTable table = view_.getTable();
            if (rows[rows.length - 1] + 1 < table.getTable().getRowCount()) {
                destOBID_ = getOBID(rows[rows.length - 1] + 1);
                destRows = getDuplicatedRows(getRowID(destOBID_));
                int index = (rows[rows.length - 1]) + (destRows.length);
                table.moveDownRows(rows, index);
                setInitialOBRowSelection(selOBIDs_);
                setOrderDirtyFlag(true);
                return true;
            }
        }
        return false;
    }
    
    public void moveDownRowsQueueItems() {
        if(moveDownRows())
            queue.moveDownQueueItems(selOBIDs_, destOBID_);

    }

    /**
     *Move to the top od the sequence  a set of rows
     *@param rows specifies the row's index to be movedup
     */
    public boolean moveToTop() {
        selOBIDs_ = getSelectedOBIDs(false);
        int[] rows = getDuplicateRows(getRowIDs(selOBIDs_));
        int[] destRows = {
        };
        if (rows.length > 0) {
            DbbTable table = view_.getTable();
            if (rows[0] - 1 >= 0) {
                destOBID_ = getOBID(0);
                int[] moveRows = getDuplicateRows(rows);
                table.moveUpRows(moveRows, 0);
                setInitialOBRowSelection(selOBIDs_);
                setOrderDirtyFlag(true);
                return true; 
            }            
        }
        return false;
    }
    
    
    public void moveToTopRowsQueueItems() {
        if(moveToTop())
            queue.moveDownQueueItems(selOBIDs_, destOBID_);

    }


    public QueueItem[] getQueueItems() {
        Vector queueItemVec = new Vector();
        long[] obIds = getAllOBsId(false);
        for (int i = 0; i < obIds.length; i++) {
            QueueItem queueItem =
                new QueueItem(
                    obIds[i],
                    queue.getId(),
                    i);
            queueItem.setType(getObType(obIds[i]));
            queueItemVec.addElement(queueItem);
        }
        QueueItem[] queueItems = new QueueItem[queueItemVec.size()];
        queueItemVec.copyInto(queueItems);
        return queueItems;
    }

    /**
      *Set the new Queue that will be displayed into the view
      */
    public void updateView(Queue newQueue) throws SQLException {
        queue = newQueue;
        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
        setOrderDirtyFlag(false);
        startQuery();
    }

    /**
     *Returns all OBIDs displayed into the view
     */
    public long[] getAllOBsId(boolean allowDuplicate) {
        Vector v = new Vector();
        for (int i = 0; i < view_.getRows(); i++) {
            Long value = new Long(getOBID(i));
            if ((!v.contains(value)) || (allowDuplicate)) {
                v.addElement(value);
            }
        }
        long[] returnOBIDs = new long[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnOBIDs[i] = ((Long) v.elementAt(i)).longValue();
            //stdlog_.debug("["+ i + "] " + returnOBIDs[i]);
        }
        return returnOBIDs;
    }

    /**
     *Return the ids of all selected OBs
     *An array of size 0 is returned if no OBs are selected
     *@return the id of all selected OB
     */
    public long[] getSelectedOBIDs(boolean allowDuplicate) {
        int[] selectedRows = view_.getSelectedRows();
        return getOBIDs(selectedRows, allowDuplicate);
    }

    /**Returns  the ID of an OB of a specific row*/
    public long getOBID(int row) {
        
        Integer value =   (Integer) view_.getValueAt(row,view_.OB_ID);
        String type = (String) view_.getValueAt(row,view_.TYPE);
        if (type.equals("O"))
            return Config.getCfg().tableToUniqueId(
                value.longValue(),
                ObservationBlock.class);

        return Config.getCfg().tableToUniqueId(
            value.longValue(),
            CalibrationBlock.class);
    }
    
    /**Returns  the ID of an OB of a specific row*/
    public long getRunID(int row,String column) {
        Long value = (Long) view_.getValueAt(row,column);
        return value.longValue();
    }


    /**Returns a set of IDs of OBs of specific rows*/
    public long[] getOBIDs(int[] rows, boolean allowDuplicate) {
        Vector v = new Vector();
        for (int i = 0; i < rows.length; i++) {
            Long value = new Long(getOBID(rows[i]));
            if ((allowDuplicate) || (!v.contains(value))) {
                v.addElement(value);
            }
        }
        long[] returnOBIDs = new long[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnOBIDs[i] = ((Long) v.elementAt(i)).longValue();
        }
        return returnOBIDs;
    }

    /**
     *Appends OBs to the view
     *@param obIds specifies the id of obs to be added to the view
     */
    public abstract boolean appendOBs(long[] obIds);

    /** 
     * start the sql engine query mechanism. 
     */
    public void startQuery() {
        view_.sqlStartAction();
    }

    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
    * Build DB browser view: SELECT panel
    *
    * <br>ChangeLog: Nick Kornweibel
    * Add the default SQLChunks. Classes extending this class should call super.buildColumns()
    * and then add their specific SQLChunk requirements.
    * Queue ID has been added at the end - it is required, but not useful to the users.
    * Part of item: 1509/OT
    *
    * <br>ChangeLog: Gorgio Passera
    * @exception SQLException JDBC exception occurs
    * @exception RemoteException RMI exception occurs
    * @exception MalformedURLException malformed url i.e. servername//port
    * @exception NotBoundException client could not bind to server
    */

    protected abstract void buildColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException;
    

    /**Build DB browser view: rows selection panel*/
    public abstract void buildRows();

    /** Build DB browser view: sort selection panel*/
    public abstract void buildSort();

    private class TreeViewClearer implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            // clear displayed OB (from tree view).
            if (tree_ != null)
                tree_.updateView(null);
        }
    }

    private class TableListSelectionListener implements ListSelectionListener {

        public TableListSelectionListener() {
        }
 
        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            currentRow_ = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty() && currentRow_ >= 0) {
                setEnabled(true);
            } else {
                setEnabled(false);
            }
        }
    }
    /**
     * Returns the obHistoryAction.
     * @return Action
     */
    public Action getObHistoryAction() {
        return obHistoryAction;
    }

    /**
     * Sets the obHistoryAction.
     * @param obHistoryAction The obHistoryAction to set
     */
    public void setObHistoryAction(Action obHistoryAction) {
        this.obHistoryAction = obHistoryAction;
    }
    /**
     * Returns the findingChartAction.
     * @return Action
     */
    public Action getFindingChartAction() {
        return findingChartAction;
    }

    /**
     * Sets the findingChartAction.
     * @param findingChartAction The findingChartAction to set
     */
    public void setFindingChartAction(Action findingChartAction) {
        this.findingChartAction = findingChartAction;
    }

}
