package org.eso.ohs.phase2.apps.ot.wizard;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DBdataHelper;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;
import org.eso.ohs.phase2.orang.OBRankingEngine;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.orang.OrangScriptingBox;

public class OrangWidgetsData {
    public static final String ALL_NIGHT = "All Night";

    public static final int STEP_INTERVAL_DEFAULT = 10;

    public static final int SUN_DAWN_DEFAULT = -18;

    /**
     * Field stdlog_
     */
    private static Logger stdlog_ = Logger.getLogger(OrangWidgetsData.class);

    private static final String[] allNightArray =
    { ALL_NIGHT };

    private String[] telescopeList = new String[0];

    private String[] queueList =new String[0];

    private String[] instrumentList = new String[0];

    private HashMap instQueueAssociation;

    private HashMap telQueueAssociation;

    public OrangWidgetsData() {
        super();
        Config cfg = Config.getCfg();
        try {
            queueList = DBdataHelper.getInstance().readStringArray(
                    Phase2SelectStmt.selectQueues(String.valueOf(cfg
                            .getUserName()), cfg.isSuperUser()));
        } catch (ObjectIOException e) {
            stdlog_.error("No queues found ", e);
        }
        
        if (instrumentList == null || instrumentList.length == 0) {
            OHSConfig config = AppConfig.getInstance();
            String regexp = config.getValue("ORANG.TODAY.INST.PATTERN");
            int instPatternIndex = config.getInteger("ORANG.TODAY.INST.INDEX",
                    1);
            Set instrumentTodayQueues = new TreeSet();
            Pattern pattern = Pattern.compile(regexp);

            for (int i = 0; i < queueList.length; i++) {
                String currentQueue = queueList[i];
                
                Matcher matcher = pattern.matcher(currentQueue);
                if (matcher.matches()
                        && matcher.groupCount() > instPatternIndex) {
                    String instrument = matcher.group(instPatternIndex);
                    instrumentTodayQueues.add(instrument);
                    stdlog_.debug("Instrument today queue: "+instrument);
                    addQueueInstAssociation(instrument,currentQueue);
                }

            }
            instrumentList = (String[]) instrumentTodayQueues
                    .toArray(new String[instrumentTodayQueues.size()]);
        }
        
        if (telescopeList == null || telescopeList.length == 0) {
            String[] instrumentList = getInstruments();
            OHSConfig config = AppConfig.getInstance();
            String nowDate = getCurrentDate();
             try {
                String[][] telInstList = DBdataHelper.getInstance().readDoubleStringArray(
                        Phase2SelectStmt.selectActiveTelInsAssociation(instrumentList,
                                nowDate));
                Set telList = new TreeSet();
                for ( int i =0 ; i < telInstList.length; i++){
                    String[] telInstAssociation = telInstList[i];
                    String currentTelescope = telInstAssociation[0];
                    String currentInstrument = telInstAssociation[1];
                    addQueueTelAssociation(currentTelescope,currentInstrument);
                telList.add(currentTelescope); 
               }
                telescopeList = (String[]) telList.toArray(new String[telList.size()]);
            } catch (ObjectIOException e) {
                stdlog_.error("no tel list", e);
                MsgManager.errmsg("Telescope List not retrieved");
            }
        }
    }

    private void addQueueInstAssociation(String instrument, String currentQueue) {
        if (instQueueAssociation == null) {
            instQueueAssociation = new HashMap ();
        }
        Set queueSet = (TreeSet) instQueueAssociation.get(instrument);
        if (queueSet == null) {
            stdlog_.debug(" added instrument "+ instrument);
            queueSet = new TreeSet();
            instQueueAssociation.put(instrument,queueSet);
        }
        queueSet.add(currentQueue);
    }
    
    private void addQueueTelAssociation(String tel, String inst) {
        if (telQueueAssociation == null) {
            telQueueAssociation = new HashMap ();
        }
        TreeSet queueSet = (TreeSet) telQueueAssociation.get(tel);
        if (queueSet == null) {
            queueSet = new TreeSet();
            telQueueAssociation.put(tel,queueSet);
        }
        TreeSet queueInstSet = (TreeSet) instQueueAssociation.get(inst);
        if (queueInstSet != null) {
           queueSet.addAll(queueInstSet);
        }
    }

    /**
     * retrieve the telescopes having today queues
     * 
     * @return String[]
     */
    public String[] getTelescopes() {
        return telescopeList;
    }

    public String[] getQueues() { 
        return queueList;
    }
    
    public String[] getQueueInst() { 
        return queueList;
    }

    public String getStartISODate() {
        Calendar currentCal = Calendar.getInstance();
        String dateStart = Convert.longToISODateMilliSec(currentCal
                .getTimeInMillis());
        return dateStart;
    }
    
    public String getCurrentDate() {
        Calendar currentCal = Calendar.getInstance();
        String dateStart = Convert.longToDBDateMilliSec(currentCal
                .getTimeInMillis());
        return dateStart;
    }

    public String getStepInterval() {
        return String.valueOf(STEP_INTERVAL_DEFAULT);
    }

    public String[] getAlgorithmList() throws OrangScriptException {
        OBRankingEngine engine = OrangScriptingBox.getInstance();
        OHSConfig config = AppConfig.getInstance();
        String scriptDir = config.getValue("ORANG.SCRIPTDIR");
        engine.setup(scriptDir);
        String[] algorithmList = engine.getAlgorithmNames();
        return algorithmList;
    }

    /**
     * retrieve the instruments having today queues. It uses
     * ORANG.QUEUE.TODAY.INST_PATTERN and ORANG.QUEUE.TODAY.INST_INDEX for
     * retrieving the instruments from the queues.
     * 
     * @return String[]
     */
    public String[] getInstruments() {
        return instrumentList;
    }

    public String[] getStepRangeList() {
        OHSConfig config = AppConfig.getInstance();
        String[] itemList = config.getValueStringArray("ORANG.STEP.RANGE", " ");
        return itemList;
    }

    public String[] getDurationRangeList() {
        OHSConfig config = AppConfig.getInstance();
        String[] itemList = config.getValueStringArray("ORANG.DURATION.RANGE",
                " ");
        itemList = TextUtils.concatenateStringArrays(itemList, allNightArray);
        return itemList;
    }

    public String[] getSkyTranspList() {
        return SkyTransparency.skyTransparencyDbRange;
    }

    public String[] getQueueTel(String tel) { 
        Set queueSet = (Set) telQueueAssociation.get(tel);
        String[] queueList = (String[]) queueSet.toArray(new String[queueSet.size()]);
        return queueList;
      } 
    public String[] getQueueInst(String inst) { 
          Set queueSet = (Set) instQueueAssociation.get(inst);
          stdlog_.debug(queueSet+" instrument selected: "+inst);
          String[] queueList = (String[]) queueSet.toArray(new String[queueSet.size()]);
          return queueList;
        }

}
