package org.eso.ohs.phase2.apps.p2pp;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentException;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.instruments.TemplateSignatureMetaData;
import org.eso.ohs.instruments.TemplateSignatureSummary;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.ObjectContainer;

public class EngMode {

    private final static TextDisplayWidget displayResults_ =
        new TextDisplayWidget("Engineering Mode Report");

    private static SwingFileChooser fileChooser_ =
        new SwingFileChooser(AppConfig.getAppConfig().getInstrumDir());

    private static ExtensionFileFilter isfFilter =
        new ExtensionFileFilter("isf", "Instrument Files");
    private static ExtensionFileFilter tsfFilter =
        new ExtensionFileFilter("tsf", "Template Signature Files");

    public EngMode(String title, ObjectContainer folderView) {
        displayResults_.setVisible(false);
        fileChooser_.setMultiSelectionEnabled(true);
        fileChooser_.addChoosableFileFilter(isfFilter);
        fileChooser_.addChoosableFileFilter(tsfFilter);
        fileChooser_.setDialogTitle("Engineering Mode : Select .isf/tsf file");
        displayResults_.clearTextArea();
        if (fileChooser_
            .getFileFilter()
            .equals(fileChooser_.getAcceptAllFileFilter()))
            fileChooser_.setFileFilter(tsfFilter);
        fileChooser_.setFileFilter(tsfFilter);
        // Java1.4 port fileChooser_.clearSelection();        
        fileChooser_.cancelSelection();
        int respone = fileChooser_.showOpenDialog(null);
        File[] files = fileChooser_.getSelectedFiles();
        String instrument = null;
        String templateName = null;
        if (files != null && respone == SwingFileChooser.APPROVE_OPTION) {

            MainWindow.getView().hideViews();
            new IncrementalActionAutomaton(
                new IncrementalEngMode(files),
                (MenuGroups) folderView.getTopLevelAncestor(),
                "Verifying files...");
        }
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */

    private class IncrementalEngMode implements IncrementalAction {

        File[] files_ = null;
        private String instrument_;
        private String rootInstDir_;
        private boolean indexExists = true;
        private boolean instrumentExists = true;
        private Vector tmplList_ = new Vector();
        private Vector tmplErrList_ = new Vector();
        private Instrument inst_;
        private boolean processTsfs = false;
        private boolean processIsfs = false;
        private boolean updateTsfCache_ = false;
        private boolean updateIsfCache_ = false;

        TemplateSignatureSummary[] templateSignatureSummary_ = null;

        public IncrementalEngMode(File[] files) {
            files_ = files;
            if (files_[0].getName().endsWith(".tsf")) {
                processTsfs = true;
                File instDir = new File(files_[0].getParent());
                 
                if (instDir.exists()) {
                    File rootDir = new File(instDir.getParent());
                    if (rootDir.exists()) {
                        try {
                            rootInstDir_ = rootDir.getCanonicalPath();
                        } catch (IOException e) {
                        }
                    }
                }
                if (rootInstDir_
                    .equals(AppConfig.getAppConfig().getInstrumDir())) {
                    updateTsfCache_ = true;
                }
                inst_ = InstrumentList.getInstance().findInstrumentByDirectory(instDir);
                /*
                 * 
                  if (!InstrumentList
                    .getInstance()
                    .instrumentExists(instrument_))
                 */
                if (inst_ == null) {  
                    indexExists = false;
                } else {
                    instrument_ = inst_.getInstrumentName();
                    templateSignatureSummary_ = inst_.getTemplateSummaryList();
                    //displayResults_.setVisible(true);
                    displayResults_.clearTextArea();
                }
                //System.out.println(" Root Instrument Directory " + rootInstDir_);
                //System.out.println(" Instrument Directory " + instrument_);
            }
            if (files_[0].getName().endsWith(".isf")) {
                //System.out.println(".....Process isfs = true");
                processIsfs = true;
            }
            displayResults_.clearTextArea();
            displayResults_.setVisible(true);
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(files_, counter);
        }

        public void startOfAction() {
        }

        public void endOfAction() {
            if (processTsfs == true) {
                if (templateSignatureSummary_ != null) {
                    for (int i = 0;
                        i < templateSignatureSummary_.length;
                        i++) {
                        boolean inError = false;
                        for (int j = 0; j < tmplErrList_.size(); j++) {
                            String templateName_ =
                                (String) tmplErrList_.elementAt(j);
                            if (templateName_
                                .equals(
                                    templateSignatureSummary_[i]
                                        .getTemplateName())) {
                                inError = true;
                            }
                        }
                        if (!inError) {
                            tmplList_.addElement(templateSignatureSummary_[i]);
                        }
                    }
                }
                if (updateTsfCache_ && inst_ != null) {
                    //System.out.println("......Updating Cache");
                    TemplateSignatureSummary[] newTemplList =
                        new TemplateSignatureSummary[tmplList_.size()];
                    tmplList_.copyInto(newTemplList);
                    inst_.setTemplateSummaryList(newTemplList);
                }
            }
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return	    true if the duplicating should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            if (files_[counter] != null) {
                // Main body of the "loop" + fall thru
                if (files_[counter].getName().endsWith(".tsf")
                    && processTsfs == true) {

                    Instrument inst =
                        InstrumentList.getInstance().findInstrumentByDirectory(
                            new File(files_[counter].getParent()));

                    String templateName = files_[counter].getName();
                    int index = templateName.indexOf(".");

                    if (index != -1 && inst != null) {
                        templateName = templateName.substring(0, index);
                        //doErrorReport(instDir.getName(), templateName);
                        doErrorReport(inst, templateName);
                    }
                } else if (
                    files_[counter].getName().endsWith(".isf")
                        && processIsfs == true) {
                    doErrorIsf(files_[counter]);
                }
            }
        }

        public void doErrorIsf(File f) {
            try {
                File parent = new File(f.getParent());
                if (AppConfig.getAppConfig().isIPVersioningEnabled()) {
                    // need to move an added directory higher is IP versioning
                    // is configured
                    parent = new File(parent.getParent());
                }
                String instDir = parent.getCanonicalPath();
                if (instDir.equals(AppConfig.getAppConfig().getInstrumDir())) {
                    updateIsfCache_ = true;
                }
                String tsfdir;
                if (AppConfig.getAppConfig().isIPVersioningEnabled()) {
                    // when using IP versioning, the ISF file is in the same
                    // directory as the TSF files.
                    File tsfDirFile = new File(f.getParent());
                    tsfdir = tsfDirFile.getName();
                } else {
                    String name = f.getName();
                    int index = name.indexOf(".");
                    tsfdir = name.substring(0, index);
                }
                Instrument inst =
                    new Instrument(
                        instDir + File.separatorChar + tsfdir,
                        f.getAbsolutePath());
                displayResults_.setTextArea(inst.toEngString());
                if (!inst.getCriticalStatus() && updateIsfCache_) {
                    InstrumentList.getInstance().insertInstrument(inst);
                }
            } catch (IOException e) {
                ErrorMessages.announceIOError(null, e);
            } catch (InstrumentException e1) {
                ErrorMessages.announceIOError(null, e1);
            }
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id	    id of business object to duplicate
         */
        public void doErrorReport(Instrument instrument, String templateName) {
            try {

                TemplateSignatureMetaData ts = null;

                ts =
                    TemplateSignatureFactory.createTemplateMetaData(
                        instrument,
                        templateName);

                if (ts.getCriticalStatus()) {
                    tmplErrList_.addElement(templateName);
                }
                displayResults_.setTextArea(ts.toEngString());
                if (templateSignatureSummary_ != null && updateTsfCache_) {
                    boolean found = false;
                    for (int i = 0;
                        i < templateSignatureSummary_.length;
                        i++) {
                        if (templateName
                            .equals(
                                templateSignatureSummary_[i]
                                    .getTemplateName())) {
                            found = true;
                            if (!ts.getCriticalStatus()) {
                                templateSignatureSummary_[i].setTemplateType(
                                    ts.getTemplateType());
                                templateSignatureSummary_[i].setTemplateDesc(
                                    ts.getTemplateDesc());
                                templateSignatureSummary_[i].setTemplateMode(
                                    ts.getMode());
                                //tmplList_.addElement(templateSignatureSummary_[i]);
                            }
                        }
                    }
                    if (!found && !ts.getCriticalStatus()) {
                        TemplateSignatureSummary tsSummary =
                            new TemplateSignatureSummary();
                        tsSummary.setTemplateType(ts.getTemplateType());
                        tsSummary.setTemplateDesc(ts.getTemplateDesc());
                        tsSummary.setTemplateMode(ts.getMode());
                        tsSummary.setTemplateName(ts.getTemplateName());
                        tmplList_.addElement(tsSummary);
                    }
                }
            } catch (IOException e) {
                ErrorMessages.announceIOError(null, e);
            }
        }
    }
}
