package org.eso.ohs.phase2.apps.p2pp;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to move a folder beneath the chosen directory.
 * 
 * <p>
 * $Id: MoveDirectoryMenuHandler.java,v 1.9 2005/10/24 13:51:12 tcanavan Exp $
 */
public class MoveDirectoryMenuHandler extends HierarchicalMenu {

    /** folder hierarchy view in this window. */
    HierarchyView hView_;

    private JFrame parentFrame_;

    /**
     * Constructs a move BusinessObject menu handler.
     * 
     * @param device      medium containing the parent directory.
     * @param hView	  GUI view of the folder hierarchy.
     * @param menuParent  JMenu containing the hierarchical submenu
     */
    public MoveDirectoryMenuHandler(
        JFrame parentFrame,
        Media device,
        HierarchyView hView,
        JMenu menuParent) {
        super(device, menuParent, hView, "Move");

        hView_ = hView;
        parentFrame_ = parentFrame;
    }

    /**
     * Handler method for "Move A Folder" menu item.
     * 
     * @param ev	event giving details of the requested action
     */
    public void actionPerformed(ActionEvent ev) {
        try {

            Class c = null;

            Folder folder = hView_.getSelectedFolder();
            ObservingRun obsRun = folder.getObsRun();
            if (folder == null) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    new Object[] {
                        "There is no selected folder.",
                        "Which folder should be moved?",
                        " ",
                        "Move failed." },
                    "Nothing selected",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }
            Summary[] summaryInformation = folder.getSummaryInformation();
            Vector idVec = new Vector();
            if (summaryInformation != null) {
                for (int i = 0; i < summaryInformation.length; i++) {
                    Object obj = summaryInformation[i].getPropertyValue("Id");
                    Long id = Long.valueOf(obj.toString());
                    c = Config.getCfg().getClassFromId(id.longValue());
                    if (c.equals(ObservationBlock.class)) {
                        idVec.addElement(id);
                    }
                }
            }
            //for() 
            DirectoryNode node = folder.getNode();
            DirectoryNode oldParent = node.getParent();

            if (oldParent == null || oldParent.getParent() == null) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    new Object[] {
                        "You cannot move top-level folders",
                        " ",
                        "Move failed" },
                    "Move failed",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

            Persistence objMgr = Persistence.getInstance();
            DirectoryNode newParent = getNodeFromMenuItem(ev.getSource());

            if (!checkSameInstrument(oldParent, newParent)) {
                return;
            }

            objMgr.moveDirectory(getMedia(), node, newParent);
            
            if(!checkSameRun(oldParent,newParent) &&  idVec.size() > 0) {
                Long [] idLongs = new Long[idVec.size()];
                idVec.copyInto(idLongs);
                new IncrementalActionAutomaton(
                    new IncrementalMove(idLongs,obsRun),
                    parentFrame_,
                    "Moving...");                
            }
            
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(hView_.getTopLevelAncestor(), e);
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(hView_.getTopLevelAncestor(), e);
        }
    }

    /**
     * Utility class to nullify a findingchart value. if the
     * observinf run has changed.
     */
    
    public class IncrementalMove implements IncrementalAction {

        /** Array of ids of objects to be duplicated */
        private Long[] idLongs_;
        
        
        
        
        private ObservingRun obsRun;
        

        /**
         * Constructor.
         *
         * @param device    media containing the objects to be duplicated
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalMove(Long[] idLongs,ObservingRun obsRun) {
            idLongs_ = idLongs;
            this.obsRun=obsRun;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
        }

        public void endOfAction() {
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         *
         * @param counter counter to show which element of obs to check in.
         * @return true if the duplicating should continue
         *   and false otherwise.
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                doMove(idLongs_[counter].longValue());
                //cbList.addElement(cb);
            }
        }

        /**
         * ObservationBlock in PAF form.
         */
        public void doMove(long id) {
            try {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                ObservationBlock ob =
                    (ObservationBlock) objMgr.getBusObj(
                        Media.LOCAL,
                        id,
                        ObservationBlock.class);        

            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                ErrorMessages.announceNoObject(parentFrame_,e);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(parentFrame_,e);
            }
        }
    }

}
