package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;

/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ClearFindingChartsMultiOBSel extends ActionSuperclass {

    private ObjectContainer folderView_;

    private JFrame frm_;


    /**
     * Clear finding charts from multiple OB's
     * @param folderView
     */


    public ClearFindingChartsMultiOBSel(ObjectContainer folderView) {
        super((JComponent) folderView, "Clear Finding Charts");
        folderView_ = folderView;
    }


    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long idLong = folderView_.getSelected();
        ObjectManager objMgr = ObjectManager.getObjectManager();
        int option =
            JOptionPane.showConfirmDialog(
                folderView_.getTopLevelAncestor(),
                "Are you sure you want to clear finding charts for these OB(s) ?",
                "Clear",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null);

        switch (option) {
            case JOptionPane.CLOSED_OPTION :
                return;
            case JOptionPane.CANCEL_OPTION :
                return;
        }

        try {
            Long[] idLongs = folderView_.getAllSelected();
            new IncrementalActionAutomaton(
                new IncrementalClear(
                    Media.LOCAL,
                    idLongs),
                (MenuGroups) folderView_.getTopLevelAncestor(),
                "Clearing...");
        } catch (ObjectIOException e) {
            announceIOError(e);
        } catch (ObjectNotFoundException e) {
            announceNoObject(e);
        }

    }
    /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalClear implements IncrementalAction {

        /** cached reference to MediaMgr */
        private ObjectManager mMgr_ = ObjectManager.getObjectManager();

        /** array of ids of objects to be duplicated */
        private Long[] idLongs_;

        /** Media containing the objects to be duplicated */
        private Media device_;


        /**
         * Constructor.
         *
         * @param frm       window on which to base any dialog boxes.
         * @param device    media containing the objects to be duplicated
         * @param deepCopy  should we duplicate attached objects?
         * @param dir       directory containing the ObsBlocks
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalClear(
            Media device,
            Long[] idLongs)
            throws ObjectNotFoundException, ObjectIOException {

            idLongs_ = idLongs;
            device_ = device;
        }

        /**
         * Number of steps in the action that are still remaining to be done.
         *
         * @return number of steps remaining (eg objects to acted upon)
         */
        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         *
         * @param stepNo        Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null)
                doClear(idLongs_[counter].longValue());
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {

        }

        /**
         * Utility method to duplicate a given business object
         *
         * @param id        id of business object to duplicate
         */
        public void doClear(long id) {
            ObservationBlock ob;
            try {
                ob =
                    (ObservationBlock) mMgr_.getBusObj(
                        device_,
                        id,
                        ObservationBlock.class);
                if (ob.isCheckedIn()) {
                    return;
                }
                Vector vec = new Vector();
                ob.setFindingCharts(null);
            } catch (ObjectNotFoundException e) {
                 announceNoObject(e);
            } catch (ObjectIOException e) {
                 announceIOError(e);
            } 
        }
    }


}
