package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
/**
 * Action class to display the detailed contents of a BusinessObject.
 *
 * <p>
 * $Id: ShowDetailsAction.java,v 1.7 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class ShowDetailsAction extends ActionSuperclass {

    private ObjectContainer folderView_;

    private Media device_;

    /**
     * Constructs a ShowDetailsAction action.
     */
    public ShowDetailsAction(Media device, ObjectContainer folderView) {
	super((JComponent)folderView, "Display Object Details");

	device_     = device;
	folderView_ = folderView;
    }

    /**
     * Event handler for the ShowDetailsAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {     
        Long idLong = folderView_.getSelected();

	if (idLong == null) {
	    JOptionPane.showMessageDialog(
		folderView_.getTopLevelAncestor(),
		new Object[] {
		  "There is no selected object to display.",
		  "Which object should be displayed?",
		},
		"Nothing selected",
		JOptionPane.ERROR_MESSAGE
	    );
	    return;
	}

	try {
//	    System.out.println("Class === " + Config.getCfg().getClassFromId(idLong.longValue()));
	    MainWindow.getView().showObject(
			device_,
			idLong.longValue(),
			Config.getCfg().getClassFromId(idLong.longValue()),
			true,
			true
		    );            
	 }
	 catch(ObjectIOException e) {
		ErrorMessages.announceIOError((JComponent)folderView_,e);
	 }
	 catch(ObjectNotFoundException e) {
		ErrorMessages.announceNoObject((JComponent)folderView_,e);
	 }
    }

}


