package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.FCS;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.SimpleStorableObject;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.TIS;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.BOSelection;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.LocalClipboard;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.PropertyClipboard;

/**
 * Action class to create a subdirectory of the selected directory.
 * <p>
 * $Id: SynchronizeAction.java,v 1.27 2005/11/17 10:00:27 tcanavan Exp $<br>
 * Author: Bob Kemp
 */

public class SynchronizeAction extends ActionSuperclass {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(SynchronizeAction.class);

    private FolderGridView folderView_;

    private boolean syncFolder_ = false;

    /**
     * nb_errs and nb_good give the number of BO's which have been verified
     * positifely or negatively
     */
    private static int nb_errs = 0, nb_good = 0;

    /**
     * indicates whether the end of the iteration is reached or if we are still
     * processing the body of set of data upon which the iteration is being
     * performed
     */
    private static boolean finished = false;

    private static StringBuffer sb_log = null;

    private static TextDisplayWidget fileMenu_ = new TextDisplayWidget(
            "Paste Action error log");

    private String syncName_;

    /**
     * Constructs a create directory action.
     * 
     * @param device
     *            medium containing the parent directory.
     * @param path
     *            path in HierarchyView tree to the selected directory.
     */
    public SynchronizeAction(FolderGridView folderView, String name,
            boolean syncFolder) {
        super((JComponent) folderView, name);
        folderView_ = folderView;
        syncFolder_ = syncFolder;
    }

    /**
     * Event handler for the create directory action, eg from a menu item.
     * 
     * @param ev
     *            user-initiated ActionEvent to create a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {

        nb_errs = 0;
        nb_good = 0;
        finished = false;
        sb_log = new StringBuffer();

        fileMenu_.clearTextArea();
        fileMenu_.setVisible(false);

        PropertyClipboard c = LocalClipboard.CLIPBOARD;
        Transferable t = c.getContents(this);
        Object[] options = { "Yes", "No", "Cancel" };
        if (t == null) {
            JOptionPane
                    .showMessageDialog(
                            folderView_.getTopLevelAncestor(),
                            new Object[] {
                                    "No object was copied to the clipboard",
                                    "Select Copy Target, Copy OD, Copy CS or Copy Acquisition ",
                                    "from the details view if you want to Synchronise data." },
                            "Clipboard is empty", JOptionPane.ERROR_MESSAGE);
            return; // nothing selected?
        }

        Class objType = folderView_.getDisplayedObjectClass();
        Long[] idLongs = null;
        if (!syncFolder_) {
            idLongs = folderView_.getAllSelected();
        } else {
            idLongs = folderView_.getAll();
        }
        // System.out.println("p2pp.actions.SynchronizeAction Length of selected
        // list " +
        // idLongs.length);
        if (idLongs.length == 0) {
            JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
                    new Object[] {
                            "There are no selected "
                                    + SimpleStorableObject
                                            .getDisplayName(objType),
                            "Which objects should be Synchronised?" }, "No "
                            + SimpleStorableObject.getDisplayName(objType)
                            + " selected", JOptionPane.ERROR_MESSAGE);
            return; // nothing selected?
        }

        DataFlavor flavor = ((BOSelection) t).getSelectedFlavor();
        Object obj = null;
        syncName_ = flavor.getHumanPresentableName();
        try {
            obj = t.getTransferData(flavor);
        } catch (IOException e) {
            stdlog_.debug(e);
        } catch (UnsupportedFlavorException e) {
            stdlog_.debug(e);
        }

        if (obj instanceof TemplateSignature
                || obj instanceof ObservationDescription
                || obj instanceof ConstraintSet) {        	
            if (!checkInstrument(obj)) {
                JOptionPane.showMessageDialog(
                        folderView_.getTopLevelAncestor(), new Object[] {
                                "The destination observing run ",
                                "uses a different instrument.", " ",
                                "Synchronisation failed." },
                        "Synchronisation failed", JOptionPane.ERROR_MESSAGE);
                return;
            }
        }
        if (syncName_.equals("STTimeIntervals")) {
            ObservingRun obsRun = ((FolderGridView) folderView_).getFolder()
                    .findObsRun();
            String instCode = obsRun.getInstCode();
            if (!AppConfig.getAppConfig().hasSTTime(instCode)) {
                JOptionPane.showMessageDialog(
                        folderView_.getTopLevelAncestor(), new Object[] {
                                "The destination observing run ",
                                "does not use Sidereal Time ", " ",
                                "Synchronisation failed." },
                        "Synchronisation failed", JOptionPane.ERROR_MESSAGE);
                return;
            }
        }

        String boName = null;
        if (obj instanceof BusinessObject) {
            boName = ((BusinessObject) obj).getName();
        }

        String questMsg = getMsgText(objType, obj, boName);

        int result = JOptionPane.showOptionDialog((JComponent) folderView_,
                questMsg, "Synchronise", JOptionPane.DEFAULT_OPTION,
                JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);

        switch (result) {
        case JOptionPane.YES_OPTION:
            new P2PPIncrementalActionAutomaton(new IncrementalSynchronise(
                    objType, Media.LOCAL, idLongs, obj),
                    (MenuGroups) folderView_.getTopLevelAncestor(),
                    "Synchronise....");
        default:
            return;
        }
    }

	private String getMsgText(Class objType, Object obj, String boName) {
		String msg = null;
        if (objType.equals(ObservationBlock.class)) {
            if (obj instanceof Target) {
                msg = "Target : " + boName;
            } else if (obj instanceof ConstraintSet) {
                msg = "ConstraintSet : " + boName;
            } else if (obj instanceof TemplateSignature) {
                msg = "Acquisiton Template : "
                        + ((TemplateSignature) obj).getTemplateName();
            } else if (obj instanceof FCS) {
                msg = "Finding Charts";
            } else if (obj instanceof TIS) {
                if (syncName_.equals("STTimeIntervals")) {
                    msg = "Sidereal Time Intervals";
                } else {
                    msg = "Time Intervals";
                }
            }
        }
        if (obj instanceof ObservationDescription) {
            msg = "ObsDesc : " + boName;
        }
        if (syncName_.equals("UserComments")) {
            msg = "Comments";
        }
        
        if (syncName_.equals("InstrumentComments")) {
            msg = "Instrument Comments";
        }                
        String questMsg = null;
        if (!syncFolder_) {
            questMsg = "Do you want to synchronise the "
                    + SimpleStorableObject.getDisplayName(objType) + "(s)"
                    + "\nselected in the grid with " + msg + "\n";
        } else {
            questMsg = "Do you want to synchronise the "
                    + SimpleStorableObject.getDisplayName(objType) + "(s)"
                    + "\nin this folder with " + msg + "\n";
        }
		return questMsg;
	}

    /**
     * Utility class to duplicate a sequence of OBs, one at a time. The user is
     * also given the opportunity to stop the process using a dialog box.
     */
    public class IncrementalSynchronise implements IncrementalAction {

        /** cached reference to MediaMgr */
        private Persistence mMgr_ = Persistence.getInstance();

        /** array of ids of objects to be duplicated */
        private Long[] idLongs_;

        /** Type of business object to be duplicated */
        private Class objType_;

        /** Media containing the objects to be duplicated */
        private Media device_;


        private Object obj_;

        private int objCount_;

        /**
         * Constructor.
         * 
         * @param frm
         *            window on which to base any dialog boxes.
         * @param device
         *            media containing the objects to be duplicated
         * @param deepCopy
         *            should we duplicate attached objects?
         * @param dir
         *            directory containing the ObsBlocks
         * @param objType
         *            type of object to be duplicated
         * @param idLongs
         *            array of ids of objects to be duplicated
         */
        public IncrementalSynchronise(Class objType, Media device,
                Long[] idLongs, Object obj) {

            objType_ = objType;
            idLongs_ = idLongs;
            device_ = device;
            obj_ = obj;
            objCount_ = 0;

            for (int j = 0; j < idLongs.length; j++) {
                if (idLongs[j] != null) {
                    objCount_++;
                }
            }
        }

        /**
         * Number of steps in the action that are still remaining to be done.
         * 
         * @return number of steps remaining (eg objects to acted upon)
         */
        public int getNumStepsRemaining(int counter) {
            if (!finished)
                return ObjUtils.howManyNotNull(idLongs_, counter);
            //else{
            //return (-1);
            //}
            return (0);
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo
         *            Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {
            if (!finished) {
                if (idLongs_[counter] != null)
                    // Main body of the "loop" + fall thru
                    doSynchronisation(idLongs_[counter].longValue());
            }
            if (counter + 1 == idLongs_.length) {
                finished = true;
            }
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {
            showFinalMsg();
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id
         *            id of business object to duplicate
         */
        public void doSynchronisation(long id) {

            BusinessObject bo;
            try {
                bo = (BusinessObject) mMgr_.getBusObj(device_, id, objType_);
                if (bo.isCheckedIn()) {
                    sb_log.append(((StorableObject) bo).getName()
                            + ": Can not paste, Business Object is locked!\n");
                    nb_errs++;
                    return;
                }

                if (objType_.equals(ObservationBlock.class)) {
                    ObservationBlock obsBlock = (ObservationBlock) bo;
                    if (obj_ instanceof BusinessObject) {
                        String[] properties = ((BusinessObject) obj_)
                                .simpleProperties();

                        if (obj_ instanceof Target) {
                            ((BusinessObject) obj_).copyProperties(properties,
                                    obsBlock.getTarget());
                        } else if (obj_ instanceof ConstraintSet) {
                            String instrument = obsBlock.getOd()
                                    .getInstrument();
                            ObservingRun obsRun = ((FolderGridView) folderView_)
                                    .getFolder().findObsRun();
                            if (!instrument.equals(obsRun.getInstCode())) {
                                JOptionPane.showMessageDialog(folderView_
                                        .getTopLevelAncestor(), new Object[] {
                                        "The destination observing run ",
                                        "uses a different instrument.", " ",
                                        "Synchronisation failed." },
                                        "Synchronisation failed",
                                        JOptionPane.ERROR_MESSAGE);
                                finished = true;
                                return;
                            }
                            ((BusinessObject) obj_).copyProperties(properties,
                                    obsBlock.getConstraintSet());
                        }
                    } else if (obj_ instanceof TemplateSignature) {
                        obsBlock.setTemplateSignature(new TemplateSignature(
                                (TemplateSignature) obj_));
                        if (((TemplateSignature) obj_).getTemplateType()
                                .equals("acquisition")) {
                            processTemplate(obsBlock);
                        }
                    }
                    if (obj_ instanceof FCS) {
                        processFCS(obsBlock);
                    }
                    if (obj_ instanceof TIS) {
                        processTIS(obsBlock);

                        //FindingChartSession.incFcCount(obsBlock);
                    }
                }
                if (objType_.equals(ObservationBlock.class)
                        || objType_.equals(CalibrationBlock.class)) {
                    CalibrationBlock cb = (CalibrationBlock) bo;
                    if (syncName_.equals("UserComments")) {
                        cb.setUserComments(obj_.toString());
                        return;
                    }
                    if (obj_ instanceof ObservationDescription) {
                        String[] properties = ((BusinessObject) obj_)
                                .simpleProperties();
                        ((BusinessObject) obj_).copyProperties(properties,
                                ((CalibrationBlock) bo).getOd());
                        /** cmavros 21-11-01 Add execution calculation time */
                        ((CalibrationBlock) bo).setExecTimeUpToDate(false);
                    }
                }

                if (objType_.equals(ObservationBlock.class)) {
                    ObservationBlock cb = (ObservationBlock) bo;
                    if (syncName_.equals("InstrumentComments")) {
                        cb.setInstrumentComments(obj_.toString());
                        return;
                    }
                }

            } catch (ObjectNotFoundException e) {
                announceNoObject(e);
            } catch (ObjectIOException e) {
                announceIOError(e);
            } catch (IOException e) {
                announceIOError(e);
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }

		private void processTIS(ObservationBlock obsBlock) {
			TIS tis = (TIS) obj_;
			TimeInterval[] timeIntervals = tis.getTIs();
			stdlog_.debug("Syncname " + syncName_);
			if (syncName_.equals("TimeIntervals"))
			    obsBlock.setTimeIntervals(timeIntervals);
			if (syncName_.equals("STTimeIntervals"))
			    obsBlock.setSTTimeIntervals(timeIntervals);
		}

		private void processFCS(ObservationBlock obsBlock) throws ObjectNotFoundException, IOException {
			FCS fcs = (FCS) obj_;
			FindingChart[] fCharts = fcs.getFcCharts();
			ObservingRun obsRun = ((FolderGridView) folderView_)
			        .getFolder().findObsRun();
			OHSPersistence persistence = (OHSPersistence) OHSPersistence
			        .getObjectManager();
			if (obsRun.getId() != fCharts[0].getObsRun().getId()) {
			    String destFolder = persistence
			            .getRootFolderPath(obsRun);
			    for (int i = 0; i < fCharts.length; i++) {
			        byte[] bdata = fCharts[i].getFcDataLocal(fcs
			                .getRootName());
			        File f = new File(destFolder + File.separator
			                + fCharts[i].getCheckSum() + "."
			                + fCharts[i].getFcType());
			        if (!f.exists())
			            FileIOUtils.writeBinaryFile(bdata, f);

			    }
			}
			obsBlock.setFindingCharts(fCharts);
		}

		private void processTemplate(ObservationBlock obsBlock) {
			Parameter plist[] = ((TemplateSignature) obj_)
			        .getParamList();
			//obsBlock.getTarget().freeLocks();
			for (int i = 0; i < plist.length; i++) {
			    if (plist[i].getType().equals("paramfile")) {

			        ParamFileParameter pfp = (ParamFileParameter) plist[i];
			        try {
			            obsBlock.getTarget()
			                    .updateFromParamFileParameter(
			                            pfp);
			        } catch (Exception e) {
			            String name = pfp.getParameterName();
			            String msg = "Error while synchronizing paramfile\n"
			                    + "attached to parameter "
			                    + name
			                    + ":\n"
			                    + e.getMessage()
			                    + "\nTarget info was not updated.";
			            // System.out.println(msg);
			            JOptionPane
			                    .showMessageDialog(
			                            MainWindow
			                                    .getPrincipalWindow(),
			                            msg,
			                            "Error in paramfile",
			                            JOptionPane.ERROR_MESSAGE);
			        }
			        try {
			            ConstraintSet cs = obsBlock
			                    .getConstraintSet();
			            if (cs != null) {
			                cs
			                        .updateFromParamFileParameter(pfp);
			            }
			        } catch (Exception e) {
			            String name = pfp.getParameterName();
			            String msg = "Error while synchronizing paramfile\n"
			                    + "attached to parameter "
			                    + name
			                    + ":\n"
			                    + e.getMessage()
			                    + "\nConstraint Set info was not updated.";
			            // System.out.println(msg);
			            JOptionPane
			                    .showMessageDialog(
			                            MainWindow
			                                    .getPrincipalWindow(),
			                            msg,
			                            "Error in paramfile",
			                            JOptionPane.ERROR_MESSAGE);
			        }
			    }
			}
		}

        /**
         * showFinalMsg shows a dialog box with information according to the
         * verification result. It evaluates "nb_errs" to distinguish between
         * the cases, errors found and error free. Also make the log window
         * visible. <br>
         * ChangeLog: Toni Schlichting 11-May-2001
         */
        public void showFinalMsg() {
            if (nb_errs != 0) {
                fileMenu_.setTextArea(sb_log.toString());
                fileMenu_.setSize(new Dimension(600, 400));
                fileMenu_.setVisible(true);

                JOptionPane
                        .showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                new Object[] {
                                        "Errors have been detected! Please consult"
                                                + "\nthe log window for further information.",
                                        " ", "" }, "Paste action",
                                JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    private boolean checkInstrument(Object obj) {
        String instrument = null;
        if (obj instanceof TemplateSignature) {
            instrument = ((TemplateSignature) obj).getInstrumentName();
        }
        if (obj instanceof ObservationDescription) {
            instrument = ((ObservationDescription) obj).getInstrument();
        }
        if (obj instanceof ConstraintSet) {
            ConstraintSet cs = (ConstraintSet) obj;
            long id = Config.getCfg()
                    .uniqueToTableId(cs.getCloneParentId());
            id = Config.getCfg().tableToUniqueId(id,
                    ObservationDescription.class);
            ObservationDescription od;
            try {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                od = (ObservationDescription) objMgr.getBusObj(Media.LOCAL,
                        id, ObservationDescription.class);
                instrument = od.getInstrument();
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
            } catch (ObjectIOException e) {
                e.printStackTrace();
                announceIOError(e);
            }
        }

        ObservingRun obsRun = ((FolderGridView) folderView_).getFolder()
                .findObsRun();
        if (!instrument.equals(obsRun.getInstCode())) {        	
            return false;
        }
        return true;
    }
    
}