package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Category;
import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.ReadmeEditView;
/**
 * Action class to open a readme file defibition.
 *
 * <p>
 * $Id: ViewEditReadmeAction.java,v 1.5 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * CHANGELOG cmavros 25JUNE02 Rename from 'Open' to 'Open New Window'
 */

public class ViewEditReadmeAction extends ActionSuperclass {
    
    private Category stdlog_ = Logger.getInstance(ViewEditReadmeAction.class);

    private HierarchyView hView_;

    /**
     * Constructs a view edit readme action
     * 
     * @param device   medium containing the directory.
     * @param path     path in HierarchyView tree to the selected directory.
     */
    public ViewEditReadmeAction(HierarchyView hView) {
        super((JComponent) hView, "View/Edit Readme");

        hView_ = hView;
    }

    /**
     * Event handler for the open directory action, eg from a menu item.
     * 
     * @param ev       user-initiated ActionEvent to view a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        try {
            Folder folder = hView_.getSelectedFolder();
            if(folder != null) {
                ObservingRun obsRun = folder.findObsRun();
                stdlog_.debug(obsRun.getObsMode());
                if(!obsRun.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)) {
                    JOptionPane.showMessageDialog(
                            hView_,
                            new Object[] {
                                "The readme file option only applies to Service mode Observing Runs",
                                },
                            "Readme Available",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                    
                }
                
                
                Instrument inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),obsRun.getIPVersion());
                
                
                if(!inst.hasReadmeData()) {
                        JOptionPane.showMessageDialog(
                            hView_,
                            new Object[] {
                                "There is no readme configuration available for this observing run.",
                                "Try download refresh observing runs",
                                "If this still does not work then contact usd-help@eso.org",
                                },
                            "Readme Available",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                }
                ReadmeEditView view = ReadmeEditView.getInstance();
                try {
                    if(obsRun.getReadme() == null) {
                        obsRun.setReadme(new Readme(obsRun.getIPVersion()));
                    } 
                    view.init(obsRun);
                } catch (IOException e1) {
                    ErrorMessages.announceIOError(hView_,e1);
                } catch (ObjectNotFoundException e1) {
                    ErrorMessages.announceNoObject(hView_,e1);
                }
            } 

        } catch (ObjectIOException e) {
            announceIOError(e);
        }
    }

}
