package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.DocViewObject;
import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.persistence.Config;

/**
 * This is a specialised JPanel with subcomponents linked to the
 * given Target object's fields.  When this component is
 * displayed, changes to it will be reflected in the Target
 * object.  Likewise changes to the Target object will be
 * reflected in the displayed fields.  Some fields of an Target
 * may not be visible.  Some fields may not be modifiable.
 * <p>
 * $Id: TIView.java,v 1.11 2004/10/11 08:10:54 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class TIView extends ObjectView implements PropertyChangeListener {

	private static Logger stdlog_ = Logger.getLogger(TIView.class);

    private ObservationBlock currentOB_;

    private TimeIntervalsGUI tiGui_;

    private DocView timeIntervals_;

    private String type_;
    
    private String dateFormat_;

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an CS may not be visible.
     * Some fields may not be modifiable.
     * <p>
     * Initially, no CS is displayed.
     */
    public TIView(String dateFormat, String type) {
        setLayout(new BorderLayout());
        dateFormat_ = dateFormat;
        type_ = type;
        if(type.equals(TimeInterval.STANDARD_TI_TYPE)) {        
             tiGui_ = new TimeIntervalsGUI(Convert.ENDTIME_MAX,type);
        }     
        else {
            tiGui_ = new TimeIntervalsGUI(0,type);                 
        }
        timeIntervals_ = new DocView(new TimeIntGuiAdaptor(tiGui_), tiGui_);
        add(timeIntervals_, BorderLayout.CENTER);

        setObject(null);
    }

    /**
     * Enable or disable the fields of this panel and any nested panels
     *
     * @param enable    True if the fields are to be enabled, false otherwise
     */
    public void setEnabled(boolean enable) {
        if (tiGui_ != null) {
            tiGui_.setEnabled(enable);
        }
    }

    /**
     * Returns a reference to the currently displayed OB
     *
     * @return reference to an OB
     */
    public BusinessObject getObject() {
        return currentOB_;
    }

    /**
     * Updates the DocView components to refer to the correct CS
     *
     * @param newCS     ConstraintSet object to be displayed
     */
    public void setObject(BusinessObject newBO) {
        setObject((ObservationBlock) newBO);
    }

    /**
     * Updates the DocView components to refer to TimeIntervals of correct OB
     *
     * @param newOB     ObservationBlock object with the
     *                  Time Intervals to be displayed
     */
    public void setObject(ObservationBlock newOB) {
        currentOB_ = newOB;
        if (type_.equals(TimeInterval.STANDARD_TI_TYPE)) {
            timeIntervals_.setObjectProperty(newOB, "TimeIntervals");
        } else {
            timeIntervals_.setObjectProperty(newOB, "STTimeIntervals");
        }
        if (currentOB_ != null)
            setEnabled(isAllowedToModify(currentOB_));
    }

    public void propertyChange(PropertyChangeEvent ev) {
        setEnabled(isAllowedToModify(currentOB_));
    }

    /**
     * GUI object to display a single TimeInterval object
     */
    private abstract class IntervalGUI extends JPanel {

        /**
         * Is this TimeInterval active?
         * In other words, does it contain data to be saved
         * or is it just a placeholder and the data junk?
         */
        private boolean isActive_;

        /** Start date/time for this time interval */
        private JTextField startTime_;

        /** End date/time for this time interval */
        private JTextField endTime_;

        /** Checkbox to allow the user to (de)activate the time interval */
        private JCheckBox checkBox_;

        /** Format for encoding/decoding text for the time interval */
        private SimpleDateFormat timeFormat_;

        /** List of listeners for property change events */
        private EventListenerList listenerList = new EventListenerList();

        /** Property change event to be passed to listeners */
        private PropertyChangeEvent propertyChangeEvent =
            new PropertyChangeEvent(this, null, null, null);

        /**
         * Default constructor
         */
        public IntervalGUI() {
            JPanel timesPanel = new JPanel(new GridLayout(1, 0, 2, 2));
            setLayout(new BorderLayout());
            startTime_ = new JTextField();
            endTime_ = new JTextField();
            checkBox_ = new JCheckBox();
            timeFormat_ = new SimpleDateFormat(dateFormat_);
            timesPanel.add(startTime_);
            timesPanel.add(endTime_);
            add(timesPanel, BorderLayout.CENTER);
            add(checkBox_, BorderLayout.EAST);
            checkBox_.setSelected(true);
            checkBox_.addActionListener(new ChkBoxListener());
            DateTimeDocumentListener docListener =
                new DateTimeDocumentListener();
            startTime_.getDocument().addDocumentListener(docListener);
            endTime_.getDocument().addDocumentListener(docListener);
            setActive(true);
        }

        /**
         * This constructor is only to support test purposes,
         * where a GUI is not wanted
         */
        public IntervalGUI(boolean noGUI) {
            timeFormat_ = new SimpleDateFormat(Convert.ISO_DATE_FORMAT);
        }

        /**
         * Enable/disable changes to this GUI component
         *
         * @param enable        whether the fields are to be enabled
         */
        public void setEnabled(boolean enable) {
            startTime_.setEnabled(enable && isActive_);
            endTime_.setEnabled(enable && isActive_);
            checkBox_.setEnabled(enable);
        }

        /**
         * Does this component model a live TimeInterval?
         * If it is not active, the user cannot edit it and the
         * value will not be saved.
         *
         * @return      whether the component is connected to a TimeInterval
         */
        public boolean isActive() {
            return isActive_;
        }

        /**
         * Activate/Deactivate this component.
         * If it is active, the user can edit it and the value will be saved.
         *
         * @param active    True if the component is to be activated,
         *                  false if it is to be deactivated
         */
        public void setActive(boolean active) {
            isActive_ = active;
            if (!isAllowedToModify(currentOB_)
                && !Config.getCfg().isInOperationsMode()) {
                startTime_.setEnabled(false);
                endTime_.setEnabled(false);

            } else {
                startTime_.setEnabled(active);
                endTime_.setEnabled(active);
            }
            checkBox_.setSelected(active);
        }
		/**
		 * Returns the start time for this time interval
		 *
		 * @return      Time stored as a Java long
		 */
		public long getStart() {
			return getTime(getStartTime(), true);
		}
		
		/**
		 * Returns the start time for this time interval
		 *
		 * @return      Time stored as a Java long
		 */
		public long getEnd() {
			return getTime(getEndTime(), true);
		}

		public abstract long getTime(JTextField tf, boolean start);
		public abstract void setTime(JTextField tf, long time);
        public abstract void verify();

        /**
         * Sets the start time for this time interval
         *
         * @param start Time stored as a Java long
         */
        public void setStart(long start) {
            setTime(startTime_, start);
        }


        /**
         * Sets the end time for this time interval
         *
         * @param end   Time stored as a Java long
         */
        public void setEnd(long end) {
            setTime(endTime_, end);
        }


        /**
         * Adds a focus listener to each of the nested components
         *
         * @param lnr   Object with an interest in focus events
         */
        public void addFocusListener(FocusListener lnr) {
            if (startTime_ != null) {
                startTime_.addFocusListener(lnr);
                endTime_.addFocusListener(lnr);
                checkBox_.addFocusListener(lnr);
            }
        }

        /**
         * Removes a focus listener from each of the nested components
         *
         * @param lnr   Object with no interest in focus events
         */
        public void removeFocusListener(FocusListener lnr) {
            if (startTime_ != null) {
                startTime_.removeFocusListener(lnr);
                endTime_.removeFocusListener(lnr);
                checkBox_.removeFocusListener(lnr);
            }
        }


        /**
         * Utility class to keep the isActive flag in sync with the checkbox
         */
        private class ChkBoxListener implements ActionListener {
            public void actionPerformed(ActionEvent ev) {
                if (isActive() != checkBox_.isSelected()) {
                    setActive(checkBox_.isSelected());
                }
                firePropertyChangeEvent();
            }
        }

        private class DateTimeDocumentListener implements DocumentListener {

            /**
             * Calls verify every time, a value in the "Time Intervals" sheet
             * has changed
             */
            public void changedUpdate(DocumentEvent e) {
                verify();
                firePropertyChangeEvent();
            }
            /**
             * Calls verify every time, a value in the "Time Intervals" sheet
             * has changed
             */
            public void insertUpdate(DocumentEvent e) {
                verify();
                firePropertyChangeEvent();
            }
            public void removeUpdate(DocumentEvent e) {
                firePropertyChangeEvent();
            }
        }

        /**
         * Informs listeners of a property change.
         * No details of which property of the old/new value are given.
         */
        protected void firePropertyChangeEvent() {
            Object[] listeners = listenerList.getListenerList();
            PropertyChangeListener lnr;

            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                lnr = (PropertyChangeListener) listeners[i + 1];

                if (listeners[i] == PropertyChangeListener.class) {
                    lnr.propertyChange(propertyChangeEvent);
                }
            }
        }

        public void addPropertyChangeListener(PropertyChangeListener lnr) {
            listenerList.add(PropertyChangeListener.class, lnr);
        }

        public void removePropertyChangeListener(PropertyChangeListener lnr) {
            listenerList.remove(PropertyChangeListener.class, lnr);
        }
        
        /**
         * The StartTime JTextField.
         * @return get the start time JTextField
         */
        public JTextField getStartTime() {
            return startTime_;
        }

        /**
         * The endtime JTextField.
         * @return get the end time JTextField.
         */
        public JTextField getEndTime() {
            return endTime_;
        }

        /**
         * Set the start time JTextfield.
         * @param field Start time JTextfield
         */
        public void setStartTime(JTextField field) {
            startTime_ = field;
        }

        /**
         * Return the date format for display. 
         * @return SimpleDataFormat.
         */
        public SimpleDateFormat getTimeFormat() {
            return timeFormat_;
        }

        /**
         * Set the dateformat for display.
         * @param format
         */
        public void setTimeFormat(SimpleDateFormat format) {
            timeFormat_ = format;
        }

    }


    private class StadardTimeIntervalGUI
        extends IntervalGUI {

        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.TIView.TimeIntervalInterface#verify()
         */
        public void verify() {
            // TODO Auto-generated method stub

            long st = getTime(getStartTime(), true),
                et = getTime(getEndTime(), false);

            if (st >= Convert.TIME_MAX)
                getStartTime().setForeground(Color.red);
            else
                getStartTime().setForeground(Color.black);
            if (st >= Convert.TIME_MAX)
                getEndTime().setForeground(Color.red);
            else
                getEndTime().setForeground(Color.black);
            if (st > et) {
                getEndTime().setForeground(Color.red);
                getStartTime().setForeground(Color.red);
            } else {
                getEndTime().setForeground(Color.black);
                getStartTime().setForeground(Color.black);
            }
        }
        
        
		/**
		 * Returns the time stored in a JTextField.
		 * It is assumed to be formatted according to the timeFormat_ string.
		 *
		 * @param tf    JTextField in which the time is to be stored.
		 *
		 * @return      Time taken from the JTextField, or the current
		 *              time if the contents were in the wrong format.
		 */
		public long getTime(JTextField tf, boolean start) {
			String dt_entered = tf.getText();
			try {
				dt_entered =
					Convert.completeISODateString(
						dt_entered,
						mk_default_dt(start));
				tf.setForeground(Color.black);
			} catch (Exception e) {
				tf.setForeground(Color.red);
				dt_entered = mk_default_dt(start);
			}

			Date date = getTimeFormat().parse(dt_entered, new ParsePosition(0));
			//stdlog_.debug("CLASS TIView - getTime:"
			//                         + " timeFormat delivered = " + date);
			if (date == null && start)
				return 0;
			else if (date == null)
				return Convert.ENDTIME_MAX;
			else
				return date.getTime() / Convert.MILLISECS_PER_SEC;
			//return date == null ? 0 : date.getTime()/Convert.MILLISECS_PER_SEC;
		}

		/**
		 * returns a default time string which allways is current date at zero hours
		 * zero minutes and zero seconds
		 */
		public String mk_default_dt(boolean start) {
			// SimpleDateFormat l_timeFormat_;
			// l_timeFormat_ = new SimpleDateFormat(Convert.ISO_DATE_FORMAT);

			long time = System.currentTimeMillis() / Convert.MILLISECS_PER_SEC;
			if (!start) {
				time = Convert.ENDTIME_MAX;
			}
			Date dt = new Date(time * Convert.MILLISECS_PER_SEC);
			String fmt = getTimeFormat().format(dt);

			// int T_pos = 0; // position of 'T' in a good dt String
			/*
			 * ... replace the current time with 00:00:00
			 */
			int T_pos = fmt.lastIndexOf("T");
			fmt = fmt.substring(0, T_pos);
			if (start)
				fmt = fmt + "T20:00:00";
			else
				fmt = fmt + "T03:14:07";

			return (fmt);
		}
		


        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.TIView.TimeIntervalInterface#setTime(javax.swing.JTextField, long)
         */
        public void setTime(JTextField tf, long time) {
			Date dt = new Date(time * Convert.MILLISECS_PER_SEC);
			String fmt = getTimeFormat().format(dt);
			tf.setText(fmt);                	
            // TODO Auto-generated method stub
            
        }

        
    }
    
	private class STTimeIntervalGUI
		extends IntervalGUI {

		/* (non-Javadoc)
		 * @see org.eso.ohs.phase2.apps.p2pp.views.TIView.TimeIntervalInterface#verify()
		 */
		public void verify() {
			// TODO Auto-generated method stub
			
			//long st = getTime(getStartTime(), true);
			//long et = getTime(getEndTime(), false);
		}
        
        
		/**
		 * Returns the time stored in a JTextField.
		 * It is assumed to be formatted according to the timeFormat_ string.
		 *
		 * @param tf    JTextField in which the time is to be stored.
		 *
		 * @return      Time taken from the JTextField, or the current
		 *              time if the contents were in the wrong format.
		 */
		public long getTime(JTextField tf, boolean start) {
			if(tf.getText()== null || tf.getText().length()<=0) {
				return 0;        		
			}
			try {
				int seconds = Convert.convertHHMMTOSeconds(tf.getText());
				int firstSeconds = 0;
				if(!start) {
					try {
						firstSeconds = Convert.convertHHMMTOSeconds(getStartTime().getText());						
					} catch(NumberFormatException e) {
						return seconds;
					}
					stdlog_.debug("Current Textfield Seconds " + seconds);
					stdlog_.debug("First Seconds  " + firstSeconds);										
					if(firstSeconds > seconds ) {
						seconds = seconds + (60*60*24);						
					}					     
				}
				return seconds;        		
			} catch(NumberFormatException e ) {
				return 0;
			}

		}		
		/* (non-Javadoc)
		 * @see org.eso.ohs.phase2.apps.p2pp.views.TIView.TimeIntervalInterface#setTime(javax.swing.JTextField, long)
		 */
		public void setTime(JTextField tf, long time) {
			Date dt = new Date(time * Convert.MILLISECS_PER_SEC);
			String fmt = getTimeFormat().format(dt);
			tf.setText(fmt);                	
			// TODO Auto-generated method stub
            
		}        
	}


    public class TimeIntervalsGUI extends JPanel implements DocViewObject {

        private IntervalGUI[] rows_ = new IntervalGUI[5];

        long max_;

        public TimeIntervalsGUI(long max,String type) {
            setLayout(new GridLayout(0, 1));
            if(type.equals(TimeInterval.STANDARD_TI_TYPE)) {
				for (int j = 0; j < rows_.length; j++) {
					rows_[j] = new StadardTimeIntervalGUI();
					add(rows_[j]);
				}            	
            } else {
				for (int j = 0; j < rows_.length; j++) {
					rows_[j] = new STTimeIntervalGUI();
					add(rows_[j]);
				}            	
            }            
            max_ = max;
        }

        public void setEnabled(boolean enable) {
            if (rows_ != null) {
                for (int j = 0; j < rows_.length; j++) {
                    rows_[j].setEnabled(enable);
                }
            }
        }

        public void addFocusListener(FocusListener lnr) {
            if (rows_ != null) {
                for (int j = 0; j < rows_.length; j++) {
                    rows_[j].addFocusListener(lnr);
                }
            }
        }

        public void removeFocusListener(FocusListener lnr) {
            if (rows_ != null) {
                for (int j = 0; j < rows_.length; j++) {
                    rows_[j].removeFocusListener(lnr);
                }
            }
        }

        public long[][] getValue() {
            Vector values = new Vector();

            for (int j = 0; j < rows_.length; j++) {
                if (rows_[j].isActive()) {
                    values.addElement(
                        new long[] { rows_[j].getStart(), rows_[j].getEnd()});
                }
            }

            long[][] result = new long[values.size()][];
            values.copyInto(result);

            return result;
        }

        public void setValue(long[][] newValue) {
            //long now = System.currentTimeMillis()/Convert.MILLISECS_PER_SEC;
            /*
             * This change goes back to bug 1253
             */
            long now = 0;
            long then = max_;

            if (newValue.length > rows_.length) {
                throw new IllegalArgumentException("Too many TimeIntervals");
            }

            for (int j = 0; j < rows_.length; j++) {
                IntervalGUI ig = rows_[j];

                if (j >= newValue.length) {
                    ig.setActive(false);
                    ig.setStart(now);                    
                    ig.setEnd(then);
                } else {
                    ig.setActive(true);
                    ig.setStart(newValue[j][0]);
                    ig.setEnd(newValue[j][1]);
                }
            }
        }

        public void addPropertyChangeListener(PropertyChangeListener lnr) {
            for (int j = 0; j < rows_.length; j++) {
                rows_[j].addPropertyChangeListener(lnr);
            }
        }

        public void removePropertyChangeListener(PropertyChangeListener lnr) {
            for (int j = 0; j < rows_.length; j++) {
                rows_[j].removePropertyChangeListener(lnr);
            }
        }
    }
}
