package org.eso.ohs.phase2.readmestatushistory;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbISODateTime;
import org.eso.ohs.core.dbb.client.DbbIntegerTextField;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.utilities.Utilities;
import org.eso.ohs.core.utilities.ObjectIOException;

/**
 * Application database view window
 * <p>
 * @version $Revision: 1.8 $
 * @author Ron J Lobo
 */
public class ReadmeStatusHistoryBrowser extends JFrame {

    public static final String cvsID_ =
        "$Id: ReadmeStatusHistoryBrowser.java,v 1.8 2005/04/11 13:53:46 tcanavan Exp $";

    private static Logger stdlog_ = Logger.getLogger(ReadmeStatusHistoryBrowser.class);

    private DbbView view_;
    private DbbSqlEngine engine_;

    private final int nullOB = -1;
    private int obId = nullOB;

    // DB tables
    private DbbSqlTable readmeStsTbl;
    private DbbSqlTable obTbl;
    private DbbSqlTable schTbl;


    private DbbIntegerTextField obITF;
    private DbbTextField lTF;

    // column titles
    private static final String OBID_ = "OB ID";
    private static final String OBNAME_ = "OB Name";
    private static final String SITE_ = "Site";
    private static final String HOSTNAME_ = "Hostname";
    private static final String TIME_STAMP_ = "Time Stamp";

    private static final String appName_ = "Readme History";

    private static final Dimension screen_ =
        java.awt.Toolkit.getDefaultToolkit().getScreenSize();

    private static ReadmeStatusHistoryBrowser dbWin = null;

    public static ReadmeStatusHistoryBrowser getInstance(int obId) {
        if (dbWin == null)
            dbWin = new ReadmeStatusHistoryBrowser(obId);
        else {
            dbWin.initOBID(obId);
            dbWin.setOBID(obId);
            dbWin.resetDbbView();
        }
        return dbWin;
    }

    /**
     * Constructor
     */
    public ReadmeStatusHistoryBrowser(int obId) {
        init(obId);
    }

    private void initOBID(int obId) {
        this.obId = obId >= 0 ? obId : nullOB;
    }

    private void init(int obId) {
        try {
            this.obId = obId >= 0 ? obId : nullOB;
            engine_ = BrowserConfig.getEngine();
            view_ = new DbbView(engine_);

            // setup database tables
            readmeStsTbl = BrowserConfig.readme_status_history; // ob_event_register Table
            obTbl = BrowserConfig.obs_blocks;
            schTbl = BrowserConfig.sch_table;
            // build the display
            createFileMenu();
            buildRows();
            buildColumns();
            buildSort();
            setOBID(obId);

            // remove unwanted default buttons
            view_.hideCountButton();
            // view_.hideQueryButton();

        } catch (ObjectIOException e) {
            // throw new ObHistoryException(e);
        } catch (RemoteException e) {
            // throw new ObHistoryException(e);
        } catch (MalformedURLException e) {
            // throw new ObHistoryException(e);
        } catch (NotBoundException e) {
            // throw new ObHistoryException(e);
        } catch (SQLException e) {
            // throw new ObHistoryException(e);
        }

        // ask before exiting the application
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                closeWindow();
            }
        });
        /*
                // map double click events to something
                view_.getTable().getTable().addMouseListener(new MouseAdapter() {
                    public void mouseClicked(MouseEvent e) {
                        if (e.getClickCount() == 2) {
                            someClass.actionPerformedImpl(null);
                        }
                    }
                });
        */
        // prepare window for display
        getContentPane().add(view_);
        setTitle("Readme Status History");
        pack();
        setSizeAndLocation();
    }
    
    private void buildRows() {
        String column;
        String label;
        DbbSqlChunk sqlChunk;
        
        DbbSqlOperator opEquals=null;
        try {
            opEquals = new DbbSqlOperator("=");
        } catch (DbbInvalidSqlOperatorType e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        column = "ob_id";
        label = OBID_;
        sqlChunk =
            new DbbSqlChunk(obTbl, column, label, opEquals, Integer.class);
        obITF = new DbbIntegerTextField(label, sqlChunk);
        
        DbbPanel panel = new DbbPanel("Selection Criteria", 5);
        panel.addWidget(obITF);
        view_.setWherePanel(panel, "North", true, 1, 1);
        panel.setVisible(false);

    }

    private void setSizeAndLocation() {
        final int maxWid = 1200;
        final int maxScr = screen_.width * 9 / 10;
        int wid = (maxWid > maxScr) ? maxScr : maxWid;
        int ht = screen_.height * 3 / 5;
        setSize(wid, ht);
        // place window near the top left
        int posX = (screen_.width - wid) / 4;
        int posY = (screen_.height - ht) / 4;
        setLocation(posX, posY);
    }

    /**
     * Build DB browser view: Select panel
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    private void buildColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException {

        DbbSqlChunk c;
        
        c = new DbbSqlChunk(schTbl,"progid","ProgId");
        engine_.addDefaultColumnsChunk(c);
        
        c = new DbbSqlChunk(readmeStsTbl, "readme_status","Readme Status");
        engine_.addDefaultColumnsChunk(c);
        
        c = new DbbSqlChunk(readmeStsTbl, "version","Readme Version");
        engine_.addDefaultColumnsChunk(c);
        
        
        c = new DbbSqlChunk(readmeStsTbl, "application_name","Application Name");
        engine_.addDefaultColumnsChunk(c);




        c = 
            new DbbSqlChunk(
                readmeStsTbl,
                "event_time",
                TIME_STAMP_,
                DbbISODateTime.class);
        engine_.addDefaultColumnsChunk(c);

        // there are no user selectable columns

        DbbPanel panel = new DbbPanel("cols");
        view_.setColumnsPanel(panel, "North", false);        
    }
    
    
    private void buildSort() {
        String column = "event_time";
        String label = TIME_STAMP_;
        DbbSqlChunk c = new DbbSqlChunk(readmeStsTbl, column, label, DbbISODateTime.class);
        DbbWidget timeWgt = new DbbArrowButton(label, c,DbbArrowButton.ASCENDING);
        DbbPanel sortPanel = new DbbPanel("Order By", 1);
        sortPanel.addWidget(timeWgt);
        view_.setSortPanel(sortPanel, "West", true, 1, 1);
        timeWgt.set(String.valueOf(DbbArrowButton.ASCENDING_CHAR));
        timeWgt.fireDbbWidgetAction(); 
    }
    


    private void setOBID(int obId) {
        this.obId = obId >= 0 ? obId : nullOB;
        if (obId != nullOB) {
            stdlog_.debug("Setting OB ID " + obId);
            obITF.set(new Integer(obId).toString());
        }
    }


    /**
     * Create and populate the "File" menu in the database browser view
     * @exception ObjectIOException object IO Exception
     */
    private void createFileMenu() throws ObjectIOException {
        JMenuBar menubar = new JMenuBar();
        setJMenuBar(menubar);
        JMenu menu = new JMenu("File");
        menubar.add(menu);
        menu.addSeparator();
        JMenuItem mnItem = menu.add(new CloseAction(this, "Close"));
        mnItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
    }

    /**
     * ensure that the query is executed after the frame appears
     */
    public void addNotify() {
        super.addNotify();
        if (obId != nullOB) {
            // automatically launch the query
            resetDbbView();
        }
    }

    /**
     * Rereads the table, and sets focus in the JTable
     */
    public void resetDbbView() {
        Utilities.waitCursor(this);
        view_.sqlStartAction(null);
        view_.getTable().getTable().requestFocusInWindow();
    }

    private void closeWindow() {
        /* mainApp.askAndQuit(); */
        setVisible(false);
    }

    /**
     * Class for closing a window or exiting the application
     */
    class CloseAction extends ActionSuperclass {

        /** Constructs a Close/Exit action */
        public CloseAction(JFrame frm, String closeStr) {
            super(frm, closeStr);
        }

        /**
         * Event handler for the Quit action
         *
         * @param ev user-initiated ActionEvent to quit
         */
        public void actionPerformedImpl(ActionEvent ae) {
            closeWindow();
        }
    }
}
