package org.eso.ohs.phase2.visibility;

/**
 * Chart data model used by the BarChart for displying data contained in
 * a set of PCFs.
 * Note: this is not a data feed, althouhg it uses the SimpleDataFeed bean,
 * it generates the data only once and never changes the data.
 *
 * @author Nick Kornweibel for ESO
 */

import javachart.beans.data.SimpleDataFeed;

public class PCFDataFeed extends SimpleDataFeed {
  
    /**
     * Set up this data bean with the data from the PCF using a 60 minute
     * precision.
     * @param PCF data source to be used in this graph
     */

    public PCFDataFeed(PCF pcf){
        this(pcf, 60);
    }

    /**
     * Set up this data bean with the data from the PCF
     * -ve infinite and +ve infinity are ignored in the PCF, therefore,
     * if there are 2 or less intervals in the PCF, there is effectively
     * nothing to plot, and an empty plot will be generated.
     * @param PCF data source to be used in this graph
     * @param interval (in minutes) generate data points from the PCF.
     */
  
    public PCFDataFeed(PCF pcf, int precision){
        double[] vals = PCF2Dataset.convertPCF2YAxisValues(pcf, precision);

        // Move the data taken from the PCF into the bean data structures
        // (xArray and yArray).

        if(xArray == null) {
            xArray = new double[1][vals.length];
        }
        if(yArray == null) {
            yArray = new double[1][vals.length];
        }
        for (int i = 0; i < vals.length; i++) {
            xArray[0][i] = (double)i;
            yArray[0][i] = vals[i];
        }
    }

    /**
     * @return the number of values in this data source
     */

    public int getValueCount() {
        int rc = 0;
        if (yArray.length > 0) {
            rc = yArray[0].length;
        }
        return rc;
    }

    /**
     * update the bean data with the data taken from the PCF and trigger
     * updates in any listening clients.
     */
    
    public void update() {
         updateDatasets();
         fireEvent();
    }
}
